/*
	Copyright (C) 2005  Michael C. Shultz

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions
	are met:

	1. Redistributions of source code must retain the above copyright
	   notice, this list of conditions and the following disclaimer.
	2. Redistributions in binary form must reproduce the above copyright
	   notice, this list of conditions and the following disclaimer in the
	   documentation and/or other materials provided with the distribution.

	THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
	IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
	OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
	IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
	INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
	DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
	THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
	THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

	Michael C. Shultz
	ringworm01@gmail.com
	Box 3238 Landers, CA 92285
*/

#include	<libMG.h>
#include	<libMG.ph>

int	MGdbAdd(classDb* db, ...)
{
	FILE*	dbFileStream;
	char	id[]		= "MGdbAdd";
	char*	paramPtr	= NULL;
	int	errorCode	= 0;
	int	idx		= 0;
	va_list	paramList;
	
	/* check if we have a valid object */
	if( db->parent.type != CLASS_TYPE_DB )
	{
		fprintf( stderr, "%s error: invalid object type\n", id);
		assert(0);
	}
	paramPtr		= (char*)1;

	/*zzzzzzzzzzzzzz*/
	db->parent.fopenIdx++;
	MGmFopen( dbFileStream, db->parent.path, "a" );
	/*zzzzzzzzzzzzzz*/

	/*zzzzzzzzzzzzzz*/
	db->parent.mallocIdx++;
	va_start(paramList, db);		/* Initialize the argument list. */	
	/*zzzzzzzzzzzzzz*/

	idx	= 0;
	while( paramPtr )
	{
		paramPtr		= va_arg(paramList, char*);
		if(!paramPtr)
		{
			break;
		}

		/*
		 * prevent null data from being placed into record fields
		 */ 
		if( MGrBufferlen( paramPtr, MAXBUFFERSIZE ) < 1 )
		{
			assert(0);
		}
		fprintf( dbFileStream, "%s%c", paramPtr, 0 );

		/*
		 * prevents random LINEFEEDS!!!
		 */
		while( fflush( dbFileStream ) );	

		idx++;
		if(idx > db->fieldQty)
		{
			fprintf( stderr, "%s error: fields added (%d) exceed field quantity (%d)\n did you forget to terminate MGdbAdd with a NULL field?\n",
				id, idx, db->fieldQty );
			assert(0);
		}
	}

	/*zzzzzzzzzzzzzz*/
	db->parent.mallocIdx--;
	va_end(paramList);		/* Clean up. */
	/*zzzzzzzzzzzzzz*/

	fprintf( dbFileStream, "\n" );

	/***************************************************************/
	/* free old records                                            */
	/***************************************************************/
	db->parent.recordIdx = db->parent.recordQty;
	while( db->parent.recordIdx > 0 )
	{
		db->parent.recordIdx--;
		/*zzzzzzzzzzzzzz*/
		db->parent.mallocIdx--;
		if(DEBUG_MGdb)
		{
			fprintf( stdout, "%s debug: %s db->parent.recordIdx = %d\n", 
				id, db->parent.path, db->parent.recordIdx );
			fprintf( stdout, "%s debug: %s freeing db->record[%d] = %s\n", 
				id, db->parent.path, db->parent.recordIdx, db->record[db->parent.recordIdx][0] );
		}
		free( db->record[db->parent.recordIdx] );
		/*zzzzzzzzzzzzzz*/
	}

	/*zzzzzzzzzzzzzz*/
	db->parent.fopenIdx--;
	fclose( dbFileStream );
	/*zzzzzzzzzzzzzz*/

	/*zzzzzzzzzzzzzz*/
	db->parent.mallocIdx--;
	free(db->dataBuffer);
	/*zzzzzzzzzzzzzz*/
	/*zzzzzzzzzzzzzz*/
	db->parent.mallocIdx--;
	free(db->record);
	/*zzzzzzzzzzzzzz*/

	/***************************************************************/
	/* increase recordQty to reflect new record                    */
	/***************************************************************/
	db->parent.recordQty++;

	MGdbRead( db );
	return(errorCode);
}
			
