/*
	Copyright (C) 2004,2005  Michael C. Shultz

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions
	are met:

	1. Redistributions of source code must retain the above copyright
	   notice, this list of conditions and the following disclaimer.
	2. Redistributions in binary form must reproduce the above copyright
	   notice, this list of conditions and the following disclaimer in the
	   documentation and/or other materials provided with the distribution.

	THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
	IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
	OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
	IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
	INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
	DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
	THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
	THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

	Michael C. Shultz
	ringworm01@gmail.com
	Box 3238 Landers, CA 92285
*/
#include	<libMG.h>
#include	<libMG.ph>

char* MGdbSeek( classDb* db, char* fieldName, char* keyValue, char* returnField, char* exact )
{
	char	id[]	= "MGdbSeek";
	
	/***************************************************************/
	/* check if we have a valid object                             */
	/***************************************************************/
	if(db == NULL || db->parent.type != CLASS_TYPE_DB  )
	{
		fprintf( stderr, "%s error: invalid object type\n", id);
		assert(0);
	}

	if( db->parent.recordIdx >= db->parent.recordQty )
	{
		return(NULL);
	} 

	if( !fieldName )
	{
		fprintf( stderr, "%s error: parameter 2 (char* fieldName) is NULL!\n", id );
		assert(0);
	}

	if( !keyValue )
	{
		fprintf( stderr, "%s error: parameter 3 (char* keyValue) is NULL!\n", id );
		assert(0);
	}

	if( !returnField )
	{
		fprintf( stderr, "%s error: parameter 4 (char* returnField) is NULL!\n", id );
		assert(0);
	}

	if( !exact )
	{
		fprintf( stderr, "%s error: parameter 5 (char* exact) is NULL! Must be either \"exact\" or \"anything else\" \n", id );
		assert(0);
	}

	if( !db->record[0][0] )
	{
		fprintf( stderr, "%s error: record 0 field 0 contains nothing!\n",
			id );
		fprintf( stderr, "fieldName = %s\ndb->parent.recordQty = %d\ndb->fieldQty = %d\n",
			fieldName, db->parent.recordQty, db->fieldQty );
		assert(0);
	}

	db->fieldIdx	= 0;
	while( strcmp( fieldName, db->record[0][db->fieldIdx] ) )
	{
		db->fieldIdx++;
		if( db->fieldIdx >= db->fieldQty )
		{
			fprintf( stderr, "%s error: field \"%s\" not found in %s\n",
				id, fieldName, db->parent.path );
			assert(0);
		}
	}	

	if( strcmp( exact, "exact" ) == 0 )
	{
		while( strncmp( db->record[db->parent.recordIdx][db->fieldIdx], keyValue, MGrStrlen(keyValue) ) ||
			MGrStrlen( db->record[db->parent.recordIdx][db->fieldIdx] ) != MGrStrlen( keyValue ) )
		{
			db->parent.recordIdx++;
			if( db->parent.recordIdx  == db->parent.recordQty )
			{
				return(NULL);
			}		 
		}
	}
	else
	{
		if( strcmp( exact, "contains" ) == 0 )
		{
			while( strnstr( db->record[db->parent.recordIdx][db->fieldIdx],
					keyValue,
					MGrStrlen( db->record[db->parent.recordIdx][db->fieldIdx] ) ) == NULL )
			{
				db->parent.recordIdx++;
				if( db->parent.recordIdx  == db->parent.recordQty )
				{
					return(NULL);
				}		 
			}		
		}
		else
		{
			if( strcmp( exact, "endsWith" ) == 0 )
			{
				while( MGrStrlen( db->record[db->parent.recordIdx][db->fieldIdx] ) < MGrStrlen( keyValue )
				||
				strncmp( db->record[db->parent.recordIdx][db->fieldIdx] + 
				( MGrStrlen( db->record[db->parent.recordIdx][db->fieldIdx] ) - MGrStrlen( keyValue ) ),
				keyValue, MGrStrlen( keyValue ) ) )
				{
					db->parent.recordIdx++;
					if( db->parent.recordIdx  == db->parent.recordQty )
					{
						return(NULL);
					}		 
				}
			}
			else
			{
				/* anything else */
				while( strncmp( db->record[db->parent.recordIdx][db->fieldIdx],
						keyValue,
						MGrStrlen( keyValue ) ) )
				{
					db->parent.recordIdx++;
					if( db->parent.recordIdx  == db->parent.recordQty )
					{
						return(NULL);
					}		 
				}
			}
		}
	}
	
	db->fieldIdx	= 0;
	while( strcmp( returnField, db->record[0][db->fieldIdx] ) )
	{
		db->fieldIdx++;
		if(db->fieldIdx == db->fieldQty)
		{
			fprintf( stderr, "%s error: field \"%s\" not found in %s\n",
				id, returnField, db->parent.path );
			fflush(stderr);
			assert(0);
		}
	}	

	db->parent.recordIdx++;
	return( db->record[db->parent.recordIdx-1][db->fieldIdx] );
}
