/*
	Copyright (C) 2004,2005  Michael C. Shultz

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions
	are met:

	1. Redistributions of source code must retain the above copyright
	   notice, this list of conditions and the following disclaimer.
	2. Redistributions in binary form must reproduce the above copyright
	   notice, this list of conditions and the following disclaimer in the
	   documentation and/or other materials provided with the distribution.

	THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
	IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
	OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
	IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
	INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
	DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
	THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
	THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

	Michael C. Shultz
	ringworm01@gmail.com
	Box 3238 Landers, CA 92285
*/

#include	<libMG.h>
#include	<libMG.ph>

/* mode 0: full path/all entries                                             */
/* mode 1: full path/all entries less . and ..                               */
/* mode 2: full path/directory entries less . and ..                         */
/* mode 3: full path/file entries less . and ..                              */
/* mode 4: no path/all entries                                               */
/* mode 5: no path/all entries less . and ..                                 */
/* mode 6: no path/directory entries less . and ..                           */
/* mode 7: no path/file entries less . and ..                                */

classDir*	MGdirCreate( int mode, char* path )
{
	DIR*	dirStream		= NULL;
	FILE*	fileStream;
	char	id[]			= "MGdirCreate";
	char*	fileName;
	int	idx			= 0;
	static	classDir*	dir;
	struct	dirent*	dirRecord;

	/*zzzzzzzzzzzzzz*/
	dir			= (classDir*)malloc(sizeof(classDir));
	/*zzzzzzzzzzzzzz*/

	dir->parent.mallocIdx	= 0;
	dir->parent.fopenIdx	= 0;
	dir->parent.mallocIdx++;

	dir->parent.bof		= 1;
	dir->parent.eof		= 0;
	dir->parent.recordIdx	= 0;
	dir->parent.type	= CLASS_TYPE_DIR;

	/*zzzzzzzzzzzzzz*/
	dir->parent.mallocIdx++;
	dir->record		= (char**)malloc(sizeof(char**)*MAXENTRIES);
	/*zzzzzzzzzzzzzz*/

	/*z(1)zzzzzzzzzz*/
	dir->parent.fopenIdx++;
	dirStream	= opendir( path );
	/*zzzzzzzzzzzzzz*/

	if(!dirStream)
	{
		fprintf(stderr, "%s ERROR: could not open dir %s\n", id, path );
		perror("system message");
		assert(0);
	}

	idx			= 0;
	dir->parent.recordQty	= 0;
	dirRecord = readdir(dirStream);
	while( dirRecord )
	{
		switch(mode)
		{
			/* mode 5: no path/all entries less . and .. */
			case	5:
			{
				if(idx > 1)
				{
					/*zzzzzzzzzzzzzz*/
					dir->parent.mallocIdx++;
					dir->record[dir->parent.recordQty]	= calloc( MAXSTRINGSIZE, 1 );
					MGmStrcpy( dir->record[dir->parent.recordQty], dirRecord->d_name );
					/*zzzzzzzzzzzzzz*/

					dir->parent.recordQty++;
				}
				break;
			}
			/* mode 7: no path/file entries less . and .. */
			case	7:
			{
				if(idx > 1)
				{
					/*zzzzzzzzzzzzzz*/
					dir->parent.mallocIdx++;
					fileName	= calloc( MAXSTRINGSIZE, 1 );
					/*zzzzzzzzzzzzzz*/

					MGmStrcpy( fileName, path );
					MGmStrcat( fileName, "/" );
					MGmStrcat( fileName, dirRecord->d_name );

					/*z(2)zzzzzzzzzz*/
					dir->parent.fopenIdx++;
					fileStream	= fopen( fileName, "a" );	/* Flawfinder: ignore */
					/*zzzzzzzzzzzzzz*/

					/* if fileStream can be opened mode "a" then it is a file */
					if( fileStream )
					{
						/*zzzzzzzzzzzzzz*/
						dir->parent.mallocIdx--;
						free(fileName);
						/*zzzzzzzzzzzzzz*/

						/*z(2)zzzzzzzzzz*/
						dir->parent.fopenIdx--;
						fclose(fileStream);
						/*zzzzzzzzzzzzzz*/

						break;
					}
					/*z(2)zzzzzzzzzz*/
					dir->parent.fopenIdx--;
					/*zzzzzzzzzzzzzz*/
					/*zzzzzzzzzzzzzz*/
					dir->parent.mallocIdx--;
					free(fileName);
					/*zzzzzzzzzzzzzz*/

					/*zzzzzzzzzzzzzz*/
					dir->parent.mallocIdx++;
					dir->record[dir->parent.recordQty]	= calloc( MAXSTRINGSIZE, 1 );
					MGmStrcpy( dir->record[dir->parent.recordQty], dirRecord->d_name );
					/*zzzzzzzzzzzzzz*/

					dir->parent.recordQty++;
				}
				break;
			}
			default:
			{
				fprintf( stderr, "%s error: unknown mode %d\n", id, mode);

				dir->parent.mallocIdx--;
				free(dir->record);

				dir->parent.fopenIdx--;
				closedir(dirStream);
				assert(0);
			}
		}
		idx++;
		dirRecord = readdir(dirStream);
	}

	/*z(1)zzzzzzzzzz*/
	dir->parent.fopenIdx--;
	closedir(dirStream);
	/*zzzzzzzzzzzzzz*/

	return(dir);
}
