/*
	Copyright (C) 2005  Michael C. Shultz

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions
	are met:

	1. Redistributions of source code must retain the above copyright
	   notice, this list of conditions and the following disclaimer.
	2. Redistributions in binary form must reproduce the above copyright
	   notice, this list of conditions and the following disclaimer in the
	   documentation and/or other materials provided with the distribution.

	THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
	IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
	OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
	IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
	INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
	DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
	THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
	THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

	Michael C. Shultz
	ringworm01@gmail.com
	Box 3238 Landers, CA 92285
*/

#include	<libMGPM.h>

#define HELP			0
#define VER 			2
#define PMSTATUS		3
#define PMUPGRADE		4
#define	SHOWLEAVES		5
#define	SHOWLEAVESDELETE	6

void	MGPMrCatchSignal( int signalId )
{
	/*
	* re-set the signal handler again to MGPMrCatchSignal, for next time
	*/
	signal( SIGINT, MGPMrCatchSignal );
	fprintf( stdout, "MGPMrCatchSignal received signal -=>%d\n", signalId );
	while( fflush( stdout ) );
	MGPMrBsdPortMkRevertPatch( globalProperty );
	exit(0);
}

void	MGPMrTimer( int signalId )
{
	/*
	* re-set the signal handler again to MGPMrCatchSignal, for next time
	*/
	signal( SIGALRM, MGPMrTimer );
	fprintf( stderr, "MGPMrTimer timeout started signal -=>%d\n", signalId );
	fclose( globalProperty->STDIN );
	globalProperty->timeOut	= 1;
}

int	rCreateCommandLineDb( structProperty* property, int argc, char** argv );
int	rDeleteCacheFile(  structProperty* property );
int	rParseCommandLine( structProperty* property, int argc, char** argv );
int	rSetXtermTitle( void );

int	MGPMrCommandLine( int argc, char** argv, char** argp )
{
	DIR*	dirStream	= NULL;
	char	id[]		= "MGPMrCommandLine";
	char*	path		= NULL;
	int	errorCode	= 0;
	char**	cmd;
	
	structProperty	property;

	globalProperty	= &property;	

	signal( SIGINT, MGPMrCatchSignal );


	property.argp	= argp;

	path		= (char*)calloc( 511, 1 );

	/*
	 * set default values here
	 */
	property.autoConflicts		= 0;	/* 1 = yes to all */
	property.autoMoved		= 0;	/* if 1 then remove moved ports is Y */
	property.backUp			= 0;	/* 0 = no packages 1 = build backup packages */
	property.bsdPortMkPatched	= 0;	/* 1 = bsd.port.mk is in patched state */
	property.bsdPortMkPatchedBlock	= 0;	/* 1 = bsd.port.mk can't be patch so quit trying */
	property.forced			= 0;	/* set by -f on command line */
	property.interactive		= 0;	/* set by -ui on command line */
	property.log			= 0;	/* set by -l --LOG on command line */
	property.optionsChanged		= 0;	/* set by MGPMrUpdate for example to indicate a ports's options just changed */ 
	property.pmMode			= 0;	/* 0 = multiple port updates/1 = single */
	property.pristine		= 0;	/* 1 = run in pristine mode */
	property.resume			= 0;    /* 1 = portmanager updated itself, don't rebuild databases in -u -f mode */
	property.timeout		= 300;	/* default timeout in seconds */
	property.NoPkgtools		= 0;	/* 1 = pkgtools.conf or ruby not installed */

	if( argv[2] && strcmp( "package-depends", argv[2] ) == 0 )
	{
		property.pmMode	= 2;
	} 

/*
 * set by Makefile options in ports
 * 1 = treat build depends as if they don't exist in availableDependencies.db
 */
#ifndef	WITH_BUILD_DEPENDS_ARE_LEAVES
	property.buildDependsAreLeaves	= 0;
#else
	property.buildDependsAreLeaves	= 1;
#endif

	/*
	 * 	Make /usr/ports/packages of it is missing
	 *
	 */
	MGmStrcpy( path, PORTSDIR );
	MGmStrcat( path, "/packages" );
	dirStream	= opendir( path );
	if( dirStream == 0 )	/* bad dir */
	{
		fprintf( stderr, "%s %s info: created %s\n", id, PACKAGE_VERSION, path );

		cmd	= malloc( sizeof( char* ) * 3 );
		cmd[0]	= malloc( MAXSTRINGSIZE );
		cmd[1]	= malloc( MAXSTRINGSIZE );
		cmd[2]	= 0;

		MGmStrcpy( cmd[0], "/bin/mkdir" );
		MGmStrcpy( cmd[1], PORTSDIR );
		MGmStrcat( cmd[1], "/packages" );
		
		MGmSystem( cmd, NULL );

		free( cmd[1] );		
		free( cmd[0] );		
		free( cmd );
/*
		system( path );
*/

	}
	else
	{
		closedir( dirStream );
	}
	free( path );

	/*
	 * set xterm title
	 */
	errorCode	= rSetXtermTitle();

	/*
	 * MGPMrInitize.c - allocate property structure
	 */
	if( ( errorCode = MGPMrInitialize( &property ) ) )
	{
		fprintf( stdout, "%s %s error: %s returned errorCode %d\n", id, PACKAGE_VERSION, "MGPMrInitialize", errorCode );
		return( 1 );
	}

	/*
	 * Parse command line options
	 */
	if( ( errorCode = rParseCommandLine( &property, argc, (char**)argv ) ) )
	{
		fprintf( stdout, "%s %s error: %s returned errorCode %d\n", id, PACKAGE_VERSION, "rParseCommandLine", errorCode );
		MGPMrCleanUp( &property );
		return( 1 );
	}

	MGPMrCleanUp( &property );
	return( 0 );
}

/*
 * set xterm title
 */
int	rSetXtermTitle( void )
{
	char	id[]		= "rSetXtermTitle";
	char*	term		= NULL;
	char*	xtermTitle	= 0;
		
	term	= getenv("TERM");
	if( term && strncmp( term, "xterm", 5 ) == 0 && isatty(fileno(stdout)) )
	{
		xtermTitle	= calloc( MAXSTRINGSIZE, 1 );
		MGmStrcpy( xtermTitle, "portmanager" );
		MGmStrcat( xtermTitle, " " );
		MGmStrcat( xtermTitle, PACKAGE_VERSION );
		printf( "%c]0;%s%c", '\033', xtermTitle, '\007' );
		free( xtermTitle );
	}
	return( 0 );
}

/*
 * Parse command line options
 */
int	rParseCommandLine( structProperty* property, int argc, char** argv )
{
	DIR*	dirStream		= NULL;
	FILE*	handle			= NULL;
	char	showLeavesDelete[]	= "SHOWLEAVESDELETE";
	char	id[]			= "rParseCommandLine";
	char	pmStatus[]		= "PMSTATUS";
	char	pmUpgrade[]		= "PMUPGRADE";
	char	showLeaves[]		= "SHOWLEAVES";
	char*	buffer			= NULL;
	char*	ignorePortDir		= NULL;
	char*	path			= NULL;
	int	bufferSize		= 0xffff;
	int	errorCode		= 0;
	int	idx			= 0;
	int	selection		= 0;
	
	/*
	 * add -ip ignored ports to ignoreDb
	 */
	property->ignoreDb	= MGdbCreate( property->ignoreDbFileName, property->fieldignoreDbPortDir, property->fieldIgnoreDbReason, NULL ); 
	if( property->ignoreDb == 0 )
	{
		fprintf( stdout, "%s %s error: uable to create %s\n", id, PACKAGE_VERSION, property->ignoreDbFileName );
		return( 1 );
	}
	ignorePortDir		= calloc( bufferSize + 1, 1 );
	idx	= 1;
	while( idx < argc )
	{
		if( ( MGrStrlen( argv[idx] ) == 3  ) && ( strcmp( argv[idx], "-ip" ) == 0 ) )
		{
			idx++;
			fprintf( stdout, "%s\n", SINGLE_LINES );
			while( idx < argc && argv[idx][0] != '-' )
			{
				if( argv[idx][0] != '/' )
				{
					ignorePortDir[0]	= '/';
					ignorePortDir[1]	= 0;
					MGmStrcat( ignorePortDir, argv[idx] );
				}
				else
				{
					ignorePortDir[0]	= 0;
					MGmStrcat( ignorePortDir, argv[idx] );
				}
				fprintf( stdout, "%s %s info: adding %s to ignore.db  reason: command line option -ip\n",
					id, PACKAGE_VERSION, ignorePortDir );
				while( fflush( stdout ) );
				MGdbAdd( property->ignoreDb, ignorePortDir, "command line option -ip", NULL );
				idx++;
			}
			idx--;
		}
		if( ( MGrStrlen( argv[idx] ) == 2 && strcmp( argv[idx], "-l" ) == 0 ) ||
			 ( MGrStrlen( argv[idx] ) == 5  && strcmp( argv[idx], "--log" ) == 0 ) )
		{
			property->log	= 1;
		}
		idx++;
	}
	free( ignorePortDir );
	MGdbDestroy( property->ignoreDb );

	idx	= 0;
	while( idx < argc )
	{
		if( ( MGrStrlen( argv[idx] ) == 6 && strcmp( argv[idx], "--help" ) == 0 ) ||
			( MGrStrlen( argv[idx] ) == 2 && strcmp( argv[idx], "-h" ) == 0 ) )
		{
			bufferSize	= MGrFileSize( property->helpFile ) + 1;
			buffer		= calloc( bufferSize, 1 );
			MGmFopen( handle, property->helpFile, "r" );
			fread( buffer, bufferSize, 1, handle );
			fprintf( stdout, "%s\n", buffer );
			free( buffer );
			fclose( handle );
			return( 0 );
		}

		if( ( MGrStrlen( argv[idx] ) == 13  && strcmp( argv[idx], "--show-leaves" ) == 0 ) ||
			( MGrStrlen( argv[idx] ) == 3  && strcmp( argv[idx], "-sl" ) == 0 ) )
		{
			selection	= SHOWLEAVES;
			break;
		}

		if( ( MGrStrlen( argv[idx] ) == MGrStrlen( "--show-leaves-interactive-delete" )  && strcmp( argv[idx], "--show-leaves-interactive-delete" ) == 0 ) ||
			( MGrStrlen( argv[idx] ) == 5 && strcmp( argv[idx], "-slid" ) == 0 ) )
		{
			selection	= SHOWLEAVESDELETE;
			break;
		}

		if( ( MGrStrlen( argv[idx] ) == 8 && strcmp( argv[idx], "--status" ) == 0 ) ||
			( strcmp( argv[idx], "-s" ) == 0 && MGrStrlen( "-s" ) == MGrStrlen( argv[idx] ) ) )
		{
			selection	= PMSTATUS;
			break;
		}

		if( strcmp( argv[idx], "--upgrade" ) == 0 || ( strcmp( argv[idx], "-u" ) == 0 && MGrStrlen(  argv[idx] ) == 2  ) )
		{
			errorCode	= rDeleteCacheFile( property );
			property->interactive = 0;
			selection	= PMUPGRADE;
			break;
		}

		if( strcmp( argv[idx],"--version" ) == 0 || ( strcmp( argv[idx], "-v" ) == 0 && MGrStrlen(  argv[idx] ) == 2  ) )
		{
			selection	= VER;
			break;
		}
		idx++;
	}

	/*
	 * Erase old commandLine.db, create/fill new commandLine.db
	 */
	if( ( errorCode = rCreateCommandLineDb( property, argc, (char**)argv ) ) )
	{
		fprintf( stdout, "%s %s error: %s returned errorCode %d\n", id, PACKAGE_VERSION, "rCreateCommandLineDb", errorCode );
		return( 1 );
	}

	switch( selection )
	{
		case	VER:
		{
			fprintf( stdout, "\n%s %s\n", id, PACKAGE_VERSION );
			exit( 0 );
		}
		case	PMSTATUS:
		{
			if( ( errorCode = MGPMrController(  property, pmStatus, argv ) ) )
			{
				fprintf(stderr, "%s %s error: %s returned errorCode %d\n", id, PACKAGE_VERSION, "PMGRrStatus", errorCode);
			}
			break;
		}
		case	PMUPGRADE:
		{
			if( ( errorCode = MGPMrController(  property, pmUpgrade, argv ) ) )
			{
				fprintf(stderr, "%s %s error: %s returned errorCode %d\n", id, PACKAGE_VERSION, "MGPMrUpgrade", errorCode);
			}
			break;
		}
		case	SHOWLEAVES:
		{

			if( ( errorCode = MGPMrController(  property, showLeaves, argv ) ) )
			{
				fprintf(stderr, "%s %s error: %s returned errorCode %d\n", id, PACKAGE_VERSION, "MGPMrShowLeaves", errorCode);
			}
			break;
		}
		case	SHOWLEAVESDELETE:
		{
			if( ( errorCode = MGPMrController(  property, showLeavesDelete, argv ) ) )
			{
				fprintf(stderr, "%s %s error: %s returned errorCode %d\n", id, PACKAGE_VERSION, "MGPMrShowLeavesDelete", errorCode);
			}
			break;
		}
		default:
			if( argv[1] )
			{
				property->pmMode	= 1;
				/*
				 * check for valid ports path
				 */
				path		= (char*)calloc( MAXSTRINGSIZE, 1 );
				MGmStrcat( path, PORTSDIR );
				MGmStrcat( path, "/" );
				MGmStrcat( path, argv[1] );
				dirStream	= opendir( path );
				if( dirStream == 0 )	/* bad dir */
				{
					fprintf( stderr, "%s %s error: %s is not a valid port directory\n", id, PACKAGE_VERSION, path );
					fprintf( stdout, "%s %s info:either you have used an invalid command line switch as the first option or you are trying to update a non existent port path, see man portmanager(1) \n",
						id, PACKAGE_VERSION );
					free( path );
					return( 1 );
				}
				closedir( dirStream );
/*
				fprintf( stdout, "upgrading/installing %s\n", path );
*/
				if( argv[1][0] != '/' )
				{
					path[0]	= '/';
					path[1] = 0;
				}
				else
				{
					path[0] = 0;
				}
				MGmStrcat( path, argv[1] );
				if( ( errorCode = MGPMrController(  property, path, argv ) ) )
				{
					fprintf( stdout, "%s %s error: %s returned errorCode %d\n", id, PACKAGE_VERSION, "MGPMrControler", errorCode );
					fprintf( stdout, "%s %s info:either you have used an invalid command line switch as the first option or you are trying to update a non existent port path, see man portmanager(1) \n",
						id, PACKAGE_VERSION );
					free( path );
					return( 1 );
				}
				free( path );
				return(0);
			}
			break;
	}
	return( 0 );
}

/*
 * Erase old commandLine.db, create/fill new commandLine.db
 */
int	rCreateCommandLineDb( structProperty* property, int argc, char** argv )
{
	char	id[]		= "rCreateCommandLineDb";
	char*	commandLineKey	= 0;
	char**	cmd;
	int	idx		= 0;
		
	cmd	= malloc( sizeof( char* ) * 4 );
	cmd[0]	= malloc( MAXSTRINGSIZE );
	cmd[1]	= malloc( MAXSTRINGSIZE );
	cmd[2]	= malloc( MAXSTRINGSIZE );
	cmd[3]	= 0;

	MGmStrcpy( cmd[0], "/bin/rm" );
	MGmStrcpy( cmd[1], "-f" );
	MGmStrcpy( cmd[2], property->commandLineDbFileName );

	MGmSystem( cmd, NULL );

	free( cmd[2] );		
	free( cmd[1] );		
	free( cmd[0] );		
	free( cmd );

	/*
	 * build commandLineDb from command line options
	 */
	property->commandLineDb	= MGdbCreate( property->commandLineDbFileName, property->fieldCommandLineDbKey, property->fieldCommandLineDbValue, NULL ); 

	if( property->commandLineDb == 0 )
	{
		fprintf( stderr, "%s %s error: portmanager must be run as root\n", id, PACKAGE_VERSION );
		return( 1 );
	}

	idx	= 1;
	while( idx < argc )
	{
		/* command line switch found */
		if( argv[idx][0]	== '-' )
		{
			commandLineKey	= argv[idx];
			MGdbAdd( property->commandLineDb, commandLineKey, " ", NULL );
			/* if next argv exists */
			if( argv[idx+1] && argv[idx+1][0] )
			{
				/* if next argv is a commandLine switch option */
				if( argv[idx+1][0] != '-' )
				{
					idx++;
					/* while argv[idx] exists] */
					while( argv[idx] )
					{
						MGdbAdd( property->commandLineDb, commandLineKey, argv[idx], NULL );
						if( !argv[idx+1] )
						{
							break;
						}
						/* if next argv[idx] is a commandLine switch */
						if( argv[idx+1][0]	== '-' )
						{
							break;
						}
						idx++;
					}
				}
			}
		}
		idx++;			
	}
	MGdbDestroy( property->commandLineDb );
	return( 0 );
}

int	rDeleteCacheFile(  structProperty* property )
{
	char	id[]		= "rDeleteCacheFile";
	char**	cmd;

	cmd	= malloc( sizeof( char* ) * 4 );
	cmd[0]	= malloc( MAXSTRINGSIZE );
	cmd[1]	= malloc( MAXSTRINGSIZE );
	cmd[2]	= malloc( MAXSTRINGSIZE );
	cmd[3]	= 0;

	MGmStrcpy( cmd[0], "/bin/rm" );
	MGmStrcpy( cmd[1], "-f" );
	MGmStrcpy( cmd[2], property->cacheDbFileName );

	MGmSystem( cmd, NULL );

	free( cmd[2] );		
	free( cmd[1] );		
	free( cmd[0] );		
	free( cmd );

	return( 0 );
}
