/*
	Copyright (C) 2005  Michael C. Shultz

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions
	are met:

	1. Redistributions of source code must retain the above copyright
	   notice, this list of conditions and the following disclaimer.
	2. Redistributions in binary form must reproduce the above copyright
	   notice, this list of conditions and the following disclaimer in the
	   documentation and/or other materials provided with the distribution.

	THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
	IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
	OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
	IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
	INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
	DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
	THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
	THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

	Michael C. Shultz
	ringworm01@gmail.com
	Box 3238 Landers, CA 92285
*/

#include	<libMGPM.h>

int	rControllerCleanUp( structLocalProperty* localProperty );
int	rPmUpgradeNewMethod( structProperty* property, structLocalProperty* localProperty );

int	MGPMrController( structProperty* property, char* path, char** argv )
{
/*
 * Signal handler
 */
	char	backUp[]			= "--back-up";
	char	bu[]				= "-bu";
	char	doneMsg1[]			= "INFO: finished with some ports not updated  if --log was used see /var/log/portmanager.log";
	char	doneMsg2[]			= "ports are up to date";
	char	exact[]				= "exact";
	char	f[]				= "-f";
	char	fix[]				= "--fix";
	char	id[]				= "MGPMrController";
	char	p[]				= "-p";
	char	package_version[]		= PACKAGE_VERSION;
	char	portmanager[]			= "portmanager";
	char	pristine[]			= "--pristine";
	char	resume[]			= "--resume";
	char	space[]				= " ";
	char	ui[]				= "-ui";
	char	upgradeInteractive[]		= "--upgrade-interactive";
	char	y[]				= "-y";
	char	yes[]				= "--yes";
	char*	oldPortsDbNewPortDirPtr		= NULL;
	char*	oldPortsDbNewPortNamePtr	= NULL;
	char*	oldPortsDbReasonPtr		= NULL;
	char*	term				= NULL;
	float	percentDone			= 0;
	int	errorCode			= 0;
	int	oldPortsDbIDX			= 0;
	int	oldPortsDbQTY			= 0;
	int	oldPortsDbTOTALIZER		= 0;

	structLocalProperty	localProperty;

	localProperty.bufferSize	= 0xffff;
	localProperty.optionsBufferSize	= 0x1ffff;

	localProperty.command		= calloc( localProperty.bufferSize, 1 );
	localProperty.oldPortDir	= calloc( 512,1 );
	localProperty.oldPortName	= calloc( 512,1 );
	localProperty.options		= calloc( localProperty.optionsBufferSize, 1 );
	localProperty.xtermTitle	= calloc( localProperty.bufferSize, 1 );

	property->cacheDb	= MGdbCreate(
					property->cacheDbFileName,
					property->fieldCacheDbPortDir,
					property->fieldCacheDbAvailablePortName,
					property->fieldCacheDbMakeFileSize,
					property->fieldCacheDbMakeFileTime,
					NULL );
	MGdbDestroy( property->cacheDb );

	property->strikesDb	= MGdbCreate( property->strikesDbFileName, property->fieldStrikesDbPortDir, property->fieldStrikesIDX, NULL );
	MGdbDestroy( property->strikesDb );

	property->commandLineDb	= MGdbOpen( property->commandLineDbFileName );

	MGdbGoTop( property->commandLineDb );
	if( MGdbSeek( property->commandLineDb, property->fieldCommandLineDbKey, f, property->fieldCommandLineDbKey, exact ) )
	{
		property->forced	= 1;
		fprintf( stdout, "%s %s info: running in forced update mode\n", id, PACKAGE_VERSION );
		while( fflush( stdout ) );
	}
	MGdbGoTop( property->commandLineDb );
	if( MGdbSeek( property->commandLineDb, property->fieldCommandLineDbKey, fix, property->fieldCommandLineDbKey, exact ) )
	{
		property->forced	= 1;
		fprintf( stdout, "%s %s info: running in forced update mode\n", id, PACKAGE_VERSION );
		while( fflush( stdout ) );
	}

	MGdbGoTop( property->commandLineDb );
	if( MGdbSeek( property->commandLineDb, property->fieldCommandLineDbKey, bu, property->fieldCommandLineDbKey, exact ) )
	{
		property->backUp	= 1;
	} 
	MGdbGoTop( property->commandLineDb );
	if( MGdbSeek( property->commandLineDb, property->fieldCommandLineDbKey, backUp, property->fieldCommandLineDbKey, exact ) )
	{
		property->backUp	= 1;
	} 
	MGdbGoTop( property->commandLineDb );
	if( MGdbSeek( property->commandLineDb, property->fieldCommandLineDbKey, y, property->fieldCommandLineDbKey, exact ) )
	{
		property->autoConflicts	= 1;
		property->autoMoved	= 1;
	} 
	if( MGdbSeek( property->commandLineDb, property->fieldCommandLineDbKey, yes, property->fieldCommandLineDbKey, exact ) )
	{
		property->autoConflicts	= 1;
		property->autoMoved	= 1;
	} 
	MGdbGoTop( property->commandLineDb );
	if( MGdbSeek( property->commandLineDb, property->fieldCommandLineDbKey, resume, property->fieldCommandLineDbKey, exact ) )
	{
		property->resume	= 1;
	} 

	MGdbGoTop( property->commandLineDb );
	if( MGdbSeek( property->commandLineDb, property->fieldCommandLineDbKey, ui, property->fieldCommandLineDbKey, exact ) )
	{
		property->interactive	= 1;
	} 

	MGdbGoTop( property->commandLineDb );
	if( MGdbSeek( property->commandLineDb, property->fieldCommandLineDbKey, upgradeInteractive, property->fieldCommandLineDbKey, exact ) )
	{
		property->interactive	= 1;
	} 

	MGdbGoTop( property->commandLineDb );
	if( MGdbSeek( property->commandLineDb, property->fieldCommandLineDbKey, p, property->fieldCommandLineDbKey, exact ) )
	{
		property->pristine	= 1;
	} 

	MGdbGoTop( property->commandLineDb );
	if( MGdbSeek( property->commandLineDb, property->fieldCommandLineDbKey, pristine, property->fieldCommandLineDbKey, exact ) )
	{
		property->pristine	= 1;
	} 

	MGdbDestroy( property->commandLineDb );

	property->logFile	= MGPMlogCreate( property );

	/*
	 * Need to check configDb for ignored ports and add them to ignoreDb here
	 */
	MGPMrReadConfigure( property, " " );

	if( path && ( strcmp( path, "SHOWLEAVES" ) == 0 ) )
	{
		if( property->resume == 0 ) /* skip this if portmanager has upgraded/rebuilt its self */
		{
			MGPMrReadConfigure( property, " " );
			if( ( errorCode	= MGPMrCreateAllUpdateStatusDb( property ) ) )
			{
				fprintf( stdout, "%s %s error: %s returned errorCode %d\n", id, PACKAGE_VERSION,
					"MGPMrCreateAllUpdateStatusDb", errorCode );
				rControllerCleanUp( &localProperty );
				return( 1 );
			}
		}

		if( ( errorCode	= MGPMrShowLeaves( property ) ) )
		{
			fprintf( stdout, "%s %s error: %s returned errorCode %d\n", id, PACKAGE_VERSION, "MGPMrShowLeaves", errorCode );
			rControllerCleanUp( &localProperty );
			return( 1 );
		}
		rControllerCleanUp( &localProperty );
		MGPMlogDestroy( property );
		return( 0 );
	}

	if( path && ( strcmp( path, "SHOWLEAVESDELETE" ) == 0 ) )
	{
		if( property->resume == 0 ) /* skip this if portmanager has upgraded/rebuilt its self */
		{
			if( ( errorCode	= MGPMrCreateAllUpdateStatusDb( property ) ) )
			{
				fprintf( stdout, "%s %s error: %s returned errorCode %d\n", id, PACKAGE_VERSION,
					"MGPMrCreateAllUpdateStatusDb", errorCode );
				rControllerCleanUp( &localProperty );
				return( 1 );
			}
		}

		if( ( property->ignoreDb = MGdbCreate( property->ignoreDbFileName, property->fieldignoreDbPortDir, property->fieldIgnoreDbReason, NULL ) ) == NULL )
		{
			fprintf( stdout, "%s %s error: failed to open %s\n", id, PACKAGE_VERSION, property->ignoreDbFileName );
			perror( "system" );
			rControllerCleanUp( &localProperty );
			MGPMlogDestroy( property );
			return( 1 );
		}
		MGdbDestroy( property->ignoreDb );
		
		if( ( errorCode	= MGPMrShowLeavesDelete( property ) ) )
		{
			fprintf( stdout, "%s %s error: %s returned errorCode %d\n", id, PACKAGE_VERSION, "MGPMrShowLeavesDelete", errorCode );
			rControllerCleanUp( &localProperty );
			MGPMlogDestroy( property );
			return( 1 );
		}
		rControllerCleanUp( &localProperty );
		MGPMlogDestroy( property );
		return( 0 );
	}

	if( path && ( strcmp( path, "PMSTATUS" ) == 0 ) )
	{
		property->pmMode	= 1; /* fake like we are in single port update */
		if( property->resume == 0 )
		{
			MGPMrReadConfigure( property, " " );
			if( ( errorCode	= MGPMrCreateAllUpdateStatusDb( property ) ) )
			{
				fprintf( stdout, "%s %s error: %s returned errorCode %d\n", id, PACKAGE_VERSION,
					"MGPMrCreateAllUpdateStatusDb", errorCode );
				return( 1 );
			}
		}

		if( ( errorCode	= MGPMrStatus( property ) ) )
		{
			fprintf( stdout, "%s %s error: %s returned errorCode %d\n", id, PACKAGE_VERSION, "MGPMrStatus", errorCode );
			return( 1 );
		}
		return( 0 );
	}

	if( path && strcmp( path, "PMUPGRADE" ) == 0 )
	{
		rPmUpgradeNewMethod( property, &localProperty );
		rControllerCleanUp( &localProperty );
		MGPMlogDestroy( property );
		return( 0 );
	}

	if( path  )
	{
		/*
		 * Generate package-depends list
		 */
		if( argv[2] && strcmp( "package-depends", argv[2] ) == 0 )
		{
			property->pmMode	= 2;

			if( property->buildDependsAreLeaves && property->pmMode != 2 )
			{
				fprintf( stdout, "%s\n", SINGLE_LINES );
				fprintf( stdout, "%s\n", "running in WITH_BUILD_DEPENDS_ARE_LEAVES mode" );
				fprintf( stdout, "%s\n", "using this mode is not endorsed by author, if ports" );
				fprintf( stdout, "%s\n", "fail building due to missing build dependencies" );
				fprintf( stdout, "%s\n", "in sysutils/portmanager run \"make config\" and shut this mode off" );
				fprintf( stdout, "%s\n", "then rebuild/reinstall portmanager before contacting author/maintainer" );
				fprintf( stdout, "%s\n", DOUBLE_LINES );
				while( fflush( stdout ) );
			}

			errorCode		= MGPMrPackageDepends( property, path );
			rControllerCleanUp( &localProperty );
			MGPMlogDestroy( property );
			return( 0 );
		}

		/*
		 * Single port upgrade
		 */
		if( property->resume == 0 ) /* skip this if --resume  */
		{
			MGPMrReadConfigure( property, " " );
			if( ( errorCode	= MGPMrSinglePortCreateStatusDb( property, path ) ) )
			{
				fprintf( stdout, "%s %s error: %s returned errorCode %d\n", id, PACKAGE_VERSION, "MGPMrSinglePortCreateStatusDb", errorCode );
				while( fflush( stdout ) );
				rControllerCleanUp( &localProperty );
				MGPMlogDestroy( property );
				return( 1 );
			}
		}
		while( 1 )
		{
			MGPMrReadConfigure( property, " " );
			if( ( errorCode	= MGPMrStatus( property ) ) )
			{
				fprintf( stdout, "%s %s error: %s returned errorCode %d\n", id, PACKAGE_VERSION, "MGPMrStatus", errorCode );
				while( fflush( stdout ) );
				rControllerCleanUp( &localProperty );
				MGPMlogDestroy( property );
				return( 1 );
			}
			localProperty.oldPortDir[0]	= 0;
			localProperty.oldPortName[0]	= 0;
			if( ( errorCode	= MGPMrBestOldPort( property, localProperty.oldPortDir, localProperty.oldPortName ) ) )
			{
				if( errorCode == 100 )
				{
					MGPMlogAdd( property,
						portmanager,
						package_version,
						doneMsg1,
						space, space );
					fprintf( stdout, "%s\n", SINGLE_LINES );
					fprintf( stdout, "%s %s INFO: finished with some ports not updated  if --log was used see /var/log/portmanager.log\n",
						"portmanager", PACKAGE_VERSION );
					while( fflush( stdout ) );
					fprintf( stdout, "%s\n", SINGLE_LINES );
					errorCode = 0;
					break;
				}
				else
				{
					fprintf( stdout, "%s %s error: %s returned errorCode %d\n", id, PACKAGE_VERSION, "MGPMrBestOldPort", errorCode );
					rControllerCleanUp( &localProperty );
					MGPMlogDestroy( property );
					return( 1 );
				}
			}
			if( MGrStrlen(localProperty.oldPortDir) > 1 )
			{
				property->oldPortsDb	= MGdbOpen( property->oldPortsDbFileName );
				property->configDb	= MGdbOpen( property->configDbFileName );
				MGdbGoTop( property->configDb );
				if( MGdbSeek( property->configDb, 
					property->fieldConfigDbKey, 
					localProperty.oldPortDir, 
					property->fieldConfigDbValue, 
					exact ) )
				{
					MGmStrcpy( localProperty.options, MGdbGet( property->configDb, 
										MGdbGetRecno( property->configDb ) - 1,
										property->fieldConfigDbValue ) );				
				}
				MGdbGoTop( property->oldPortsDb );
				oldPortsDbReasonPtr	= MGdbSeek( property->oldPortsDb, property->fieldOldPortsDbOldPortDir, localProperty.oldPortDir,
								property->fieldOldPortsDbReason, exact );
				if( oldPortsDbReasonPtr )
				{
					oldPortsDbIDX	= MGdbGetRecno( property->oldPortsDb ) - 1;
					oldPortsDbNewPortNamePtr	= MGdbGet( property->oldPortsDb, 
										oldPortsDbIDX, 
										property->fieldOldPortsDbNewPortName );
					oldPortsDbNewPortDirPtr		= MGdbGet( property->oldPortsDb,
										oldPortsDbIDX, 
										property->fieldOldPortsDbNewPortDir );
				}
				if( strcmp( localProperty.oldPortName, "not installed" ) == 0 )
				{
					MGmStrcpy( localProperty.command, "adding " );
					MGmStrcat( localProperty.command, localProperty.oldPortName );
					MGmStrcat( localProperty.command, " " );
					MGmStrcat( localProperty.command, localProperty.oldPortDir );
					MGmStrcat( localProperty.command, " " );
				}
				else
				{
					MGmStrcpy( localProperty.command, "updating " );
					MGmStrcat( localProperty.command, localProperty.oldPortName );
					MGmStrcat( localProperty.command, " " );
					MGmStrcat( localProperty.command, localProperty.oldPortDir );
					MGmStrcat( localProperty.command, " " );
				}
				if( localProperty.options )
				{
					MGmStrcat( localProperty.command, " options " );
					MGmStrcat( localProperty.command, localProperty.options );
				}
				if( oldPortsDbReasonPtr )
				{
					MGmStrcat( localProperty.command, " reason: " );
					MGmStrcat( localProperty.command, oldPortsDbReasonPtr );
					MGmStrcat( localProperty.command, " " );
					MGmStrcat( localProperty.command, oldPortsDbNewPortNamePtr );
					MGmStrcat( localProperty.command, " " );
					MGmStrcat( localProperty.command, oldPortsDbNewPortDirPtr );
					MGmStrcat( localProperty.command, " " );
				}
				fprintf( stdout, "%s\n", localProperty.command );
				fprintf( stdout, "%s\n", SINGLE_LINES );

				/*
				 *  Set xterm title bar indicator
				 */
				oldPortsDbQTY		= MGdbGetRecordQty( property->oldPortsDb );

				if( oldPortsDbQTY - 1 > oldPortsDbTOTALIZER )
				{
					oldPortsDbTOTALIZER	= oldPortsDbQTY - 1;
				}
				percentDone	= 100 - ( 100 * ( ( (float)oldPortsDbQTY - 1 ) / (float)oldPortsDbTOTALIZER ) );
				fprintf( stdout, "percentDone-=>%d = 100 - ( 100 * ( oldPortsDbQTY-=>%d / oldPortsDbTOTALIZER-=>%d ) ) \n",
					(int)percentDone, oldPortsDbQTY-1, oldPortsDbTOTALIZER );
				while( fflush( stdout ) );

				term	= getenv("TERM");
				if( term && strncmp( term, "xterm", 5 ) == 0 && isatty(fileno(stdout)) )
				{
					/*
					pr 81481
					To fix the modification of xterm title, even when redirecting, one can
					test isatty(fileno(stdout)) before printing the escape sequence.
					*/
					MGmStrcpy( localProperty.xtermTitle, "portmanager" );
					MGmStrcat( localProperty.xtermTitle, " done:% " );
					MGmStrcat( localProperty.xtermTitle, MGrIntToString( (long)percentDone ) );
					MGmStrcat( localProperty.xtermTitle, " updating " );
					MGmStrcat( localProperty.xtermTitle, localProperty.oldPortName );
					MGmStrcat( localProperty.xtermTitle, " " );
					MGmStrcat( localProperty.xtermTitle, localProperty.oldPortDir );
					printf( "%c]0;%s%c", '\033', localProperty.xtermTitle, '\007' );
				}
				MGPMlogAdd( property, localProperty.oldPortName,  localProperty.oldPortDir, oldPortsDbReasonPtr, oldPortsDbNewPortNamePtr, oldPortsDbNewPortDirPtr );
				MGdbDestroy( property->configDb );
				MGdbDestroy( property->oldPortsDb );
			}
			else
			{
				/*
				 *  Set xterm title bar indicator
				 *
				 * pr 81481
				 * To fix the modification of xterm title, even when redirecting, one can
				 * test isatty(fileno(stdout)) before printing the escape sequence.
				 */
				if( term && strncmp( term, "xterm", 5 ) == 0 && isatty(fileno(stdout)) )
				{
					MGmStrcpy( localProperty.xtermTitle, "portmanager" );
					MGmStrcat( localProperty.xtermTitle, " " );
					MGmStrcat( localProperty.xtermTitle, " finished" );
					printf( "%c]0;%s%c", '\033', localProperty.xtermTitle, '\007' );
				}
				MGPMlogAdd( property, portmanager, package_version, doneMsg2, space, space );
				fprintf( stdout, "%s %s INFO: all ports are up to date\n", "portmanager", PACKAGE_VERSION );
				while( fflush( stdout ) );
				break;
			}
			MGPMrUpdate( property, localProperty.oldPortDir, localProperty.oldPortName );
		}
	}
	MGPMlogDestroy( property );
	rControllerCleanUp( &localProperty );
	return( errorCode );
}

/*******************************************************/
int	rControllerCleanUp( structLocalProperty* localProperty )
/*******************************************************/
{
	free( localProperty->command );
	free( localProperty->oldPortDir );
	free( localProperty->oldPortName );
	free( localProperty->options );
	free( localProperty->xtermTitle );
	return( 0 );
}
/*******************************************************/
int	rPmUpgradeNewMethod( structProperty* property, structLocalProperty* localProperty )
/*******************************************************/
{
	char	doneMsg2[]			= "ports are up to date";
	char	exact[]				= "exact";
	char	id[]				= "rPmUpgradeNewMethod";
	char	package_version[]		= PACKAGE_VERSION;
	char	portmanager[]			= "portmanager";
	char	space[]				= " ";
	char*	oldPortsDbNewPortDirPtr		= NULL;
	char*	oldPortsDbNewPortNamePtr	= NULL;
	char*	oldPortsDbReasonPtr		= NULL;
	char*	term				= NULL;
	float	percentDone			= 0;
	int	errorCode			= 0;
	int	oldPortsDbIDX			= 0;
	int	oldPortsDbQTY			= 0;
	int	oldPortsDbTOTALIZER		= 0;

	property->pmMode	= 1; /* fake like we are in single port update */
	if( property->resume == 0 ) /* skip this if portmanager has upgraded/rebuilt its self */
	{
		MGPMrReadConfigure( property, " " );
		if( ( errorCode	= MGPMrCreateAllUpdateStatusDb( property ) ) )
		{
			fprintf( stdout, "%s %s error: %s returned errorCode %d\n", id, PACKAGE_VERSION,
				"MGPMrCreateAllUpdateStatusDb", errorCode );
			return( 1 );
		}
	}
	while( 1 )
	{
		MGPMrReadConfigure( property, " " );
		if( ( errorCode	= MGPMrStatus( property ) ) )
		{
			fprintf( stdout, "%s %s error: %s returned errorCode %d\n", id, PACKAGE_VERSION, "MGPMrStatus", errorCode );
			return( 1 );
		}
		localProperty->oldPortDir[0]	= 0;
		localProperty->oldPortName[0]	= 0;
		if( ( errorCode	= MGPMrBestOldPort( property, localProperty->oldPortDir, localProperty->oldPortName ) ) )
		{
			if( errorCode == 100 )
			{
				fprintf( stdout, "%s\n", SINGLE_LINES );
				fprintf( stdout, "%s %s INFO: finished with some ports not updated  if --log was used see /var/log/portmanager.log\n",
					"portmanager", PACKAGE_VERSION );
				while( fflush( stdout ) );
				fprintf( stdout, "%s\n", SINGLE_LINES );
				errorCode = 0;
				break;
			}
			else
			{
				fprintf( stdout, "%s %s error: %s returned errorCode %d\n", id, PACKAGE_VERSION, "MGPMrBestOldPort", errorCode );
				return( 1 );
			}
		}
		if( MGrStrlen(localProperty->oldPortDir) > 1 )
		{
			property->oldPortsDb	= MGdbOpen( property->oldPortsDbFileName );
			property->configDb	= MGdbOpen( property->configDbFileName );

			MGdbGoTop( property->configDb );
			if( MGdbSeek( property->configDb, 
				property->fieldConfigDbKey, 
				localProperty->oldPortDir, 
				property->fieldConfigDbValue, 
				exact ) )
			{
				MGmStrcpy( localProperty->options, MGdbGet( property->configDb, 
									MGdbGetRecno( property->configDb ) - 1,
									property->fieldConfigDbValue ) );				
			}

			MGdbGoTop( property->oldPortsDb );
			oldPortsDbReasonPtr	= MGdbSeek( property->oldPortsDb, property->fieldOldPortsDbOldPortDir, localProperty->oldPortDir,
							property->fieldOldPortsDbReason, exact );
			if( oldPortsDbReasonPtr )
			{
				oldPortsDbIDX	= MGdbGetRecno( property->oldPortsDb ) - 1;
				oldPortsDbNewPortNamePtr	= MGdbGet( property->oldPortsDb, oldPortsDbIDX, 
									property->fieldOldPortsDbNewPortName );
				oldPortsDbNewPortDirPtr	= MGdbGet( property->oldPortsDb, oldPortsDbIDX, 
									property->fieldOldPortsDbNewPortDir );
			}
			if( strcmp( localProperty->oldPortName, "not installed" ) == 0 )
			{
				MGmStrcpy( localProperty->command, "adding " );
				MGmStrcat( localProperty->command, localProperty->oldPortName );
				MGmStrcat( localProperty->command, " " );
				MGmStrcat( localProperty->command, localProperty->oldPortDir );
				MGmStrcat( localProperty->command, " " );
			}
			else
			{
				MGmStrcpy( localProperty->command, "updating " );
				MGmStrcat( localProperty->command, localProperty->oldPortName );
				MGmStrcat( localProperty->command, " " );
				MGmStrcat( localProperty->command, localProperty->oldPortDir );
				MGmStrcat( localProperty->command, " " );
			}
			if( localProperty->options )
			{
				MGmStrcat( localProperty->command, " options " );
				MGmStrcat( localProperty->command, localProperty->options );
			}
			if( oldPortsDbReasonPtr )
			{
				MGmStrcat( localProperty->command, " reason: " );
				MGmStrcat( localProperty->command, oldPortsDbReasonPtr );
				MGmStrcat( localProperty->command, " " );
				MGmStrcat( localProperty->command, oldPortsDbNewPortNamePtr );
				MGmStrcat( localProperty->command, " " );
				MGmStrcat( localProperty->command, oldPortsDbNewPortDirPtr );
				MGmStrcat( localProperty->command, " " );
			}
			fprintf( stdout, "%s\n", localProperty->command );
			fprintf( stdout, "%s\n", SINGLE_LINES );

			/*
			 *  Set xterm title bar indicator
			 */
			oldPortsDbQTY		= MGdbGetRecordQty( property->oldPortsDb );

			if( oldPortsDbQTY - 1 > oldPortsDbTOTALIZER )
			{
				oldPortsDbTOTALIZER	= oldPortsDbQTY - 1;
			}
			percentDone	= 100 - ( 100 * ( ( (float)oldPortsDbQTY - 1 ) / (float)oldPortsDbTOTALIZER ) );
			fprintf( stdout, "percentDone-=>%d = 100 - ( 100 * ( oldPortsDbQTY-=>%d / oldPortsDbTOTALIZER-=>%d ) ) \n",
				(int)percentDone, oldPortsDbQTY-1, oldPortsDbTOTALIZER );
			while( fflush( stdout ) );

			term	= getenv("TERM");
			if( term && strncmp( term, "xterm", 5 ) == 0 && isatty(fileno(stdout)) )
			{
				/*
				pr 81481
				To fix the modification of xterm title, even when redirecting, one can
				test isatty(fileno(stdout)) before printing the escape sequence.
				*/
				MGmStrcpy( localProperty->xtermTitle, "portmanager" );
				MGmStrcat( localProperty->xtermTitle, " done:% " );
				MGmStrcat( localProperty->xtermTitle, MGrIntToString( (long)percentDone ) );
				MGmStrcat( localProperty->xtermTitle, " updating " );
				MGmStrcat( localProperty->xtermTitle, localProperty->oldPortName );
				MGmStrcat( localProperty->xtermTitle, " " );
				MGmStrcat( localProperty->xtermTitle, localProperty->oldPortDir );
				printf( "%c]0;%s%c", '\033', localProperty->xtermTitle, '\007' );
			}
			MGPMlogAdd( property, localProperty->oldPortName,  localProperty->oldPortDir, oldPortsDbReasonPtr, oldPortsDbNewPortNamePtr, oldPortsDbNewPortDirPtr );
			MGdbDestroy( property->configDb );
			MGdbDestroy( property->oldPortsDb );
		}
		else
		{
			/*
			 *  Set xterm title bar indicator
			 *
			 * pr 81481
			 * To fix the modification of xterm title, even when redirecting, one can
			 * test isatty(fileno(stdout)) before printing the escape sequence.
			 */
			if( term && strncmp( term, "xterm", 5 ) == 0 && isatty(fileno(stdout)) )
			{
				MGmStrcpy( localProperty->xtermTitle, "portmanager" );
				MGmStrcat( localProperty->xtermTitle, " " );
				MGmStrcat( localProperty->xtermTitle, " finished" );
				printf( "%c]0;%s%c", '\033', localProperty->xtermTitle, '\007' );
			}
			MGPMlogAdd( property, portmanager, package_version, doneMsg2, space, space );
			fprintf( stdout, "%s %s INFO: all ports are up to date\n", portmanager, PACKAGE_VERSION );
			while( fflush( stdout ) );
			break;
		}
		MGPMrUpdate( property, localProperty->oldPortDir, localProperty->oldPortName );
	}
	return( 0 );
}
