/*
	Copyright (C) 2005  Michael C. Shultz

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions
	are met:

	1. Redistributions of source code must retain the above copyright
	   notice, this list of conditions and the following disclaimer.
	2. Redistributions in binary form must reproduce the above copyright
	   notice, this list of conditions and the following disclaimer in the
	   documentation and/or other materials provided with the distribution.

	THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
	IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
	OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
	IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
	INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
	DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
	THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
	THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

	Michael C. Shultz
	ringworm01@gmail.com
	Box 3238 Landers, CA 92285
*/

#include	<libMGPM.h>

int	rrFillDependencies( structProperty* property, char* availablePortName, char* availablePortDir, int* counterPtr );
int	rrFindDepends( structProperty* property, char* availablePortDir, char* availablePortName, int* counterPtr );

int	MGPMrCreateAllUpdateStatusDb( structProperty* property )
{
	FILE*	dependencyDbStream		= NULL;
	char	exact[]				= "exact";
	char	id[]				= "MGPMrCreateAllUpdateStatusDb";
	char*	availablePortDir		= NULL;
	char*	availablePortName		= NULL;
	char*	installedPortsDbPortDirPtr	= NULL;
	int	bufferSize			= 0xffff;
	int	counter				= 0;
	int	errorCode			= 0;
	int	installedPortsDbIDX		= 0;
	int	installedPortsDbQTY		= 0;
	int*	counterPtr			= NULL;
	
	counterPtr	= &counter;

	availablePortDir	= (char*)calloc( bufferSize, 1 );
	availablePortName	= (char*)calloc( bufferSize, 1 );

	fprintf( stdout,"%s\n", SINGLE_LINES );
	if( property->forced )
	{
		fprintf( stdout,"%s %s: Collecting installed port data \"forced mode\"\n", "portmanager", PACKAGE_VERSION );
	}
	else
	{
		fprintf( stdout,"%s %s: Collecting installed port data \n", "portmanager", PACKAGE_VERSION );

	}
	fprintf( stdout,"%s\n", SINGLE_LINES );
	while( fflush( stdout ) );

	property->availablePortsDb	= MGdbCreate(
						property->availablePortsDbFileName,
						property->fieldAvailablePortsDbPortName,
						property->fieldAvailablePortsDbPortDir,
						property->fieldAvailablePortsDbFlag,
						NULL);

	/*
	 * creates dependencyDb with header record, same as MGdbCreate.  In this case records will be added by
	 * fprint'ing to dependencyDbStream instead of using MGdbAdd because speed is important here so we don't
	 * want a dependencyDb object yet.
	 */
	MGmFopen( dependencyDbStream, property->availableDependenciesDbFileName, "w" );
	if( dependencyDbStream == 0 )
	{
		fprintf( stdout, "%s %s error: could not open %s\n", id, PACKAGE_VERSION, property->availableDependenciesDbFileName );
		perror( "system message" );
		return( 1 );
	}
	fprintf( dependencyDbStream, "%s%c%s%c%s%c%s%c%s%c%c", "fieldAvailableDependenciesDbPortDir", 0, "fieldAvailableDependenciesDbPortName", 0,
		"fieldAvailableDependenciesDbDependencyDir", 0,	"fieldAvailableDependenciesDbDependencyName", 0,
		"fieldAvailableDependenciesDbDependType", 0, LINEFEED );
	fclose( dependencyDbStream );

	property->availableDependenciesDb	= MGdbOpen( property->availableDependenciesDbFileName );

	if( MGrIfFileExist( property->configDbFileName ) )
	{
		property->configDb	= MGdbOpen( property->configDbFileName );
	}
	else
	{
		property->configDb	= MGdbCreate( property->configDbFileName, property->fieldConfigDbKey, property->fieldConfigDbValue, NULL );
	}

	property->installedPortsDb	= MGdbOpen( property->installedPortsDbFileName );

	if( MGrIfFileExist( property->ignoreDbFileName ) )
	{
		property->ignoreDb	= MGdbOpen( property->ignoreDbFileName );
	}
	else
	{
		property->ignoreDb	= MGdbCreate( property->ignoreDbFileName,
							property->fieldignoreDbPortDir,
							property->fieldIgnoreDbReason, NULL );
	}

	installedPortsDbQTY		= MGdbGetRecordQty( property->installedPortsDb );
	counter				= installedPortsDbQTY;
	installedPortsDbIDX		= 1;
	while( installedPortsDbIDX < installedPortsDbQTY )
	{
		installedPortsDbPortDirPtr	= MGdbGet( property->installedPortsDb, installedPortsDbIDX, property->fieldInstalledPortsDbPortDir );
		MGdbGoTop( property->availablePortsDb );
		if( MGdbSeek( property->availablePortsDb, property->fieldAvailablePortsDbPortDir, installedPortsDbPortDirPtr,
			property->fieldAvailablePortsDbPortDir, exact ) )
		{
			/*
			 * this port's dependency info allready gathered
			 */
			 installedPortsDbIDX++;
			 continue;
			 
		}
		MGmStrcpy( availablePortDir, installedPortsDbPortDirPtr );
		
		if( ( errorCode = rrFindDepends( property, availablePortDir, availablePortName, counterPtr ) ) == 100 )
		{
			property->installedPortsDb = MGdbDelete( property->installedPortsDb, installedPortsDbIDX );
			installedPortsDbQTY--;
		}
		else
		{
			if( errorCode )
			{
				fprintf( stdout, "%s %s error: %s returned errorCode %d\n", id, PACKAGE_VERSION, "rrFindDepends", errorCode );
				while( fflush( stdout ) );
				MGdbDestroy( property->availableDependenciesDb );
				MGdbDestroy( property->availablePortsDb );
				MGdbDestroy( property->configDb );
				MGdbDestroy( property->ignoreDb );
				MGdbDestroy( property->installedPortsDb );
				free( availablePortDir );
				free( availablePortName );
				return( 1 );
			}
		}
		installedPortsDbIDX++;
	}
	MGdbDestroy( property->availableDependenciesDb );
	MGdbDestroy( property->availablePortsDb );
	MGdbDestroy( property->configDb );
	MGdbDestroy( property->ignoreDb );
	MGdbDestroy( property->installedPortsDb );
	free( availablePortDir );
	free( availablePortName );
	return( 0 );
}

int	rrFindDepends( structProperty* property, char* availablePortDir, char* availablePortName, int* counterPtr )
{
	char	id[]			= "rrFindDepends";
	int	availablePortsDbIDX	= 0;
	int	errorCode		= 0;
	int	availablePortsDbQTY	= 0;

	if( ( errorCode = MGPMrGetPortName( property, availablePortDir, availablePortName ) ) )
	{
/*
		fprintf( stdout, "%s %s error: %s returned errorCode %d\n", id, PACKAGE_VERSION, "MGPMrGetPortName", errorCode );
*/
		return( 100 );
	}

	MGdbAdd( property->availablePortsDb, availablePortName, availablePortDir, "0", NULL);
	if( rrFillDependencies( property, availablePortName, availablePortDir, counterPtr ) )
	{
		fprintf( stdout, "%s %s error: %s returned an error\n", id, PACKAGE_VERSION, "rrFillDependencies" );
		while( fflush( stdout ) );
		return( 1 );
	}

	/*
 	 * fill dependencies for each record in property->availablePortsDb
 	 */		
	availablePortsDbQTY	= MGdbGetRecordQty( property->availablePortsDb );
	availablePortsDbIDX	= 1;
	while( availablePortsDbIDX < availablePortsDbQTY )
	{
		if( MGdbGet( property->availablePortsDb, availablePortsDbIDX, property->fieldAvailablePortsDbFlag )
			&&
			strncmp( MGdbGet( property->availablePortsDb, availablePortsDbIDX, property->fieldAvailablePortsDbFlag ), "0", 1 ) == 0 )
		{
			MGmStrcpy( availablePortDir, MGdbGet( property->availablePortsDb, availablePortsDbIDX, property->fieldAvailablePortsDbPortDir ) );
			MGmStrcpy( availablePortName, MGdbGet( property->availablePortsDb, availablePortsDbIDX, property->fieldAvailablePortsDbPortName ) );

			if( rrFillDependencies( property, availablePortName, availablePortDir, counterPtr ) )
			{
				fprintf( stdout, "%s %s error: %s returned an error\n", id, PACKAGE_VERSION, "rrFillDependencies" );
				while( fflush( stdout ) );
				return( 1 );
			}

			availablePortsDbQTY	= MGdbGetRecordQty( property->availablePortsDb );
			availablePortsDbIDX	= 0;
		}
		availablePortsDbIDX++;
	}
	return( 0 );
}

int	rrFillDependencies( structProperty* property, char* availablePortName, char* availablePortDir, int* counterPtr )
{
	char	BUILD_DEPENDS[]		= "BUILD_DEPENDS";
	char	RUN_DEPENDS[]		= "RUN_DEPENDS";
	char	exact[]			= "exact";
	char	id[]			= "rrFillDependencies";
	char*	availablePortDirPtr	= NULL;
	char*	buffer			= NULL;
	int	availablePortsDbIDX	= 0;
	int	bufferSize		= 0xffff;

	buffer		= calloc( bufferSize + 1, 1 );

	if( MGPMrDependencyParse( property, buffer, availablePortName, availablePortDir, BUILD_DEPENDS, counterPtr ) )
	{
		fprintf( stdout, "%s %s error: %s returned an error\n", id, PACKAGE_VERSION, "MGPMrDependencyParse" );
		while( fflush( stdout ) );
		free( buffer );
		return( 1 );
	}

	buffer[0]	= 0;

	if( MGPMrDependencyParse( property, buffer, availablePortName, availablePortDir, RUN_DEPENDS, counterPtr ) )
	{
		fprintf( stdout, "%s %s error: %s returned an error\n", id, PACKAGE_VERSION, "MGPMrDependencyParse" );
		while( fflush( stdout ) );
		free( buffer );
		return( 1 );
	}

	free( buffer );

	/*
	 * Flag availablePortsDb.availablePortName as "1" after dependencies added
	 */
	if( MGrStrlen(availablePortDir) < 3 )
	{
		fprintf( stdout, "%s error: MGrStrlen(availablePortDir)-=>%d\n", id, MGrStrlen(availablePortDir) );
		while( fflush( stdout ) );
		return( 1 );
	}
	MGdbGoTop( property->availablePortsDb );
	availablePortDirPtr		= MGdbSeek( property->availablePortsDb, property->fieldAvailablePortsDbPortDir, availablePortDir,
						property->fieldAvailablePortsDbPortDir, exact );
	availablePortsDbIDX		= MGdbGetRecno( property->availablePortsDb ) - 1;
	property->availablePortsDb	= MGdbDelete( property->availablePortsDb, availablePortsDbIDX );
	MGdbAdd( property->availablePortsDb, availablePortName, availablePortDir, "1", NULL);

	return( 0 );
}

