/*
	Copyright (C) 2005  Michael C. Shultz

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions
	are met:

	1. Redistributions of source code must retain the above copyright
	   notice, this list of conditions and the following disclaimer.
	2. Redistributions in binary form must reproduce the above copyright
	   notice, this list of conditions and the following disclaimer in the
	   documentation and/or other materials provided with the distribution.

	THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
	IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
	OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
	IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
	INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
	DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
	THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
	THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

	Michael C. Shultz
	ringworm01@gmail.com
	Box 3238 Landers, CA 92285
*/

#include	<libMGPM.h>

int	MGPMrGetPortName( structProperty* property, char* portDir, char* portName )
{
	FILE*	fileStream			= NULL;
	FILE*	makeStream			= NULL;
	char	exact[]				= "exact";
	char	id[]				= "MGPMrGetPortName";
	char*	availablePortsDbPortNamePtr	= NULL;
	char*	charPtr				= NULL;
	char*	command				= NULL;
	char*	optionsPtr			= NULL;
	char**	cmd;
	int	answer				= 0;
	int	bufferSize			= 0xffff;
	int	doubleBreak			= 0;

	command	= calloc( MAXBUFFERSIZE, 1 );

	MGdbGoTop( property->availablePortsDb );
	availablePortsDbPortNamePtr	= MGdbSeek( property->availablePortsDb, property->fieldAvailablePortsDbPortDir, portDir,
						property->fieldAvailablePortsDbPortName, exact );

	if( availablePortsDbPortNamePtr == NULL )
	{
		MGdbGoTop( property->installedPortsDb );
		if( MGdbSeek( property->installedPortsDb,
					property->fieldInstalledPortsDbPortDir,
					portDir,
					property->fieldInstalledPortsDbPortName,
					exact ) )
		{
			MGmStrcpy( portName, MGdbGet( property->installedPortsDb,
						MGdbGetRecno( property->installedPortsDb ) - 1,
						property->fieldInstalledPortsDbPortName ) );
		}
		else
		{
			portName[0]	= 0;
		}

		MGmStrcpy( command, PORTSDIR );
		MGmStrcat( command, portDir );
		MGmStrcat( command, "/Makefile" );

		if( MGrIfFileExist( command ) )
		{
			MGmFopen( fileStream, command, "r" );
			fclose( fileStream );
		}
		else
		{
			fprintf( stdout,"%s\n", SINGLE_LINES );
			fprintf( stdout,
				"%s %s is no longer in the ports collection, see /usr/ports/MOVED\nthis port must be removed for portmanager to continue\nis it OK to remove %s from your system? (yes/no/auto yes to all) [y/n/a] [y]\n",
				portName, portDir, portName );
			fprintf( stdout, "%s\n", SINGLE_LINES );
			fprintf( stdout, "Will timeout after 5 minutes and default to [Y] if no selection is made\n" );
			while( fflush( stdout ) );

			if( property->autoMoved == 0 )
			{
				/*
				 * set timeout default to Y
				 */
				alarm( property->timeout );
				signal( SIGALRM, MGPMrTimer );
				property->STDIN	= fopen( "/dev/stdin", "r" );	/* Flawfinder: ignore */
				property->timeOut = 0;
				doubleBreak	= 0;
				while( 1 == 1 )
				{
					answer	= getc( property->STDIN );
					fclose( property->STDIN );
					switch(answer)
					{
						case KEY_ENTER:
						{
							doubleBreak	= 1;
							break;
						}
						case KEY_A:
						{
							property->autoMoved = 1;
							doubleBreak	= 1;
							break;
						}
						case KEY_a:
						{
							property->autoMoved = 1;
							doubleBreak	= 1;
							break;
						}
						case KEY_Y:
						{
							doubleBreak	= 1;
							break;
						}
						case KEY_y:
						{
							doubleBreak	= 1;
							break;
						}
						case KEY_N:
						{
							fprintf( stdout, "you will have to deal with this manually then, portmanager shutting down\n" );
							while( fflush( stdout ) );
							exit(0);
						}
						case KEY_n:
						{
							fprintf( stdout, "you will have to deal with this manually then, portmanager shutting down\n" );
							while( fflush( stdout ) );
							exit(0);
						}
						default:
						{
							if( property->timeOut )
							{
								fprintf( stdout, "5 minute timeout, defaulting to [Y]\n");
								while( fflush( stdout ) );
								doubleBreak	= 1;
								break;
							}
							else
							{
								property->STDIN	= fopen( "/dev/stdin", "r" );	/* Flawfinder: ignore */
								fprintf( stdout, "Invalid option. Please enter either Y to upgrade or N\n");
								while( fflush( stdout ) );
							}
						}
					}
					if( doubleBreak )
					{
						break;
					}
				}
			}
			fprintf( stdout, "%s\n", SINGLE_LINES );
			while( fflush( stdout ) );

			if( property->timeOut  == 1 )
			{
				/*
				 * reset alarm, reopen STDIN after timeout
				 */
				alarm(0);
				property->timeOut	= 0;
				MGPMlogAdd( property, "installed ", portName,
					" removed from system after 5 minute timeout:  no longer in ports tree: see /usr/ports/MOVED", " ", " " );
			}
			else
			{
				fclose( property->STDIN );
				MGPMlogAdd( property, "installed ", portName,
					" removed from system:  no longer in ports tree:  see /usr/ports/MOVED", " ", " " );
			}

			cmd	= malloc( sizeof( char* ) * 4 );
			cmd[0]	= malloc( MAXSTRINGSIZE );
			cmd[1]	= malloc( MAXSTRINGSIZE );
			cmd[2]	= malloc( MAXSTRINGSIZE );
			cmd[3]	= 0;

			MGmStrcpy( cmd[0], "/usr/sbin/pkg_delete" );
			MGmStrcpy( cmd[1], "-f" );
			MGmStrcpy( cmd[2], portName );

			fprintf( stdout, "removing: %s\n", portName );
			fprintf( stdout,"%s %s %s\n", cmd[0], cmd[1], cmd[2] );
			while( fflush( stdout ) );

			MGmSystem( cmd, NULL );

			MGmStrcpy( cmd[0], "/bin/rm" );
			MGmStrcpy( cmd[1], "-r" );
			MGmStrcpy( cmd[2], PORTSDIR );
			MGmStrcat( cmd[2], portDir );
			fprintf( stdout, "executing: %s %s %s (OK to ignore error)\n", cmd[0], cmd[1], cmd[2] ); 
			while( fflush( stdout ) );
			MGmSystem( cmd, NULL );
			fprintf( stdout,"%s\n", SINGLE_LINES );

			free( cmd[2] );		
			free( cmd[1] );		
			free( cmd[0] );		
			free( cmd );

			free( command );
			return( 100 );
		}

		/*
		 * see if this port has anything special in configDb
		 */
		MGdbGoTop( property->configDb );
		optionsPtr	= MGdbSeek( property->configDb, property->fieldConfigDbKey, portDir, property->fieldConfigDbValue, exact );

		MGmStrcpy( command, "cd " );
		MGmStrcat( command, PORTSDIR );
		MGmStrcat( command, portDir );
		MGmStrcat( command, "; make " );
		MGmStrcat( command, " -V PKGNAME " );
		if( optionsPtr )
		{
			MGmStrcat( command, optionsPtr );
		}
		makeStream	= popen( command, "r" );
		if( makeStream == 0 )
		{
			fprintf( stderr, "%s error: reading make -V PKGNAME into availablePortName\n", id );
			perror( "system message" );
			while( fflush( stderr ) );
			free( command );
			return( 1 );
		}
		portName[0]	= 0;
		fread( portName, bufferSize-1, 1, makeStream );
		if( portName == NULL )
		{
			fprintf( stderr, "%s error: reading make -V PKGNAME into availablePortName\n", id );
			perror( "system message" );
			while( fflush( stderr ) );
			free( command );
			return( 1 );
		}
		charPtr	= portName;
		while( charPtr < portName + (bufferSize-1) )
		{
			if( charPtr[0]	== SPACE || charPtr[0] == TAB || charPtr[0] == LINEFEED || charPtr[0] == ':' )
			{
				charPtr[0]	= 0;
				break;
			}
			charPtr++;
		}
		pclose( makeStream );
	}
	else
	{
		MGdbGoTop( property->availablePortsDb );
		if( MGdbSeek( property->availablePortsDb,
			property->fieldAvailablePortsDbPortDir,
			portDir,
			property->fieldAvailablePortsDbPortName,
			exact ) )
		{
			MGmStrcpy( portName, MGdbGet( property->availablePortsDb,
							MGdbGetRecno( property->availablePortsDb ) - 1,
							property->fieldAvailablePortsDbPortName ) );
		}
		else
		{
			fprintf( stderr, "%s error: port name for %s not found in availablePortsDb\n", id, portDir );
			while( fflush( stderr ) );
			assert( 0 );
		}
	}
	free( command );
	return( 0 );
}
