/*
	Copyright (C) 2005  Michael C. Shultz

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions
	are met:

	1. Redistributions of source code must retain the above copyright
	   notice, this list of conditions and the following disclaimer.
	2. Redistributions in binary form must reproduce the above copyright
	   notice, this list of conditions and the following disclaimer in the
	   documentation and/or other materials provided with the distribution.

	THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
	IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
	OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
	IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
	INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
	DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
	THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
	THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

	Michael C. Shultz
	ringworm01@gmail.com
	Box 3238 Landers, CA 92285
*/

#include	<libMGPM.h>

int	MGPMrPackageDepends( structProperty* property , char* path )
{
	char		id[]			= "MGPMrPackageDepends";
	char		exact[]			= "exact";
	char		fieldPortDir[]		= "fieldPortDir";
	char		fieldPortName[]		= "fieldPortName";
	char*		dependencyDir		= NULL;
	char*		dependencyDirPtr	= NULL;
	char*		dependencyName		= NULL;
	char*		pkgDbFileName		= NULL;
	char*		portDir			= NULL;
	char*		portName		= NULL;
	classDb*	pkgDb			= NULL;
	int		errorCode		= 0;
	int		pkgDbIDX		= 0;
	int		pkgDbQTY		= 0;
	
	property->pmMode = 2;

	pkgDbFileName	= calloc( 255, 1 );
	dependencyDir	= calloc( 255, 1 );
	dependencyName	= calloc( 255, 1 );
	portDir		= calloc( 255, 1 );
	portName	= calloc( 255, 1 );

	MGmStrcpy( pkgDbFileName, SHAREDIR );
	MGmStrcat( pkgDbFileName, "/pkg.db" );
	
	pkgDb	= MGdbCreate( pkgDbFileName, "fieldPortName", "fieldPortDir", NULL );

	MGPMrReadConfigure( property, path );

	if( ( errorCode	= MGPMrSinglePortCreateStatusDb( property, path ) ) )
	{
		fprintf( stderr, "MGPMrPackageDepends error: MGPMrSinglePortCreateStatusDb returned an error\n" );
		while( fflush( stderr ) );
		MGdbDestroy( pkgDb );
		free( dependencyDir );
		free( dependencyName );
		free( pkgDbFileName );
		free( portDir );
		free( portName );
		return( 1 );
	}


	property->availableDependenciesDb	= MGdbOpen( property->availableDependenciesDbFileName );
	property->dependencyPortsDb		= MGdbOpen( property->dependencyPortsDbFileName );
	property->installedPortsDb		= MGdbOpen( property->installedPortsDbFileName );

	/*
	 * Seed pkgDb with immediate dependencies of path
	 * guarentee portname's come from /var/db/pkg/{portname} 
	 */
	MGdbGoTop( property->availableDependenciesDb );
	while( MGdbSeek( property->availableDependenciesDb,
			property->fieldAvailableDependenciesDbPortDir,
			path,
			property->fieldAvailableDependenciesDbPortDir,
			exact ) )
	{
		if( strcmp( "RUN_DEPENDS", MGdbGet( property->availableDependenciesDb,
					MGdbGetRecno( property->availableDependenciesDb ) - 1,
					property->fieldAvailableDependenciesDbDependType ) ) == 0 )
		{
			MGmStrcpy( portDir, MGdbGet( property->availableDependenciesDb,
					MGdbGetRecno( property->availableDependenciesDb ) - 1,
					property->fieldAvailableDependenciesDbDependencyDir ) );	

			/*
			 * Gaurentee that we get the name from /var/db/pkg/{port name}
			 */
			MGdbGoTop( property->installedPortsDb );
			if( MGdbSeek( property->installedPortsDb,
					property->fieldInstalledPortsDbPortDir,
					portDir,
					property->fieldInstalledPortsDbPortName,
					exact ) )
			{
				MGmStrcpy( portName, MGdbGet( property->installedPortsDb,
							MGdbGetRecno( property->installedPortsDb ) - 1,
							property->fieldInstalledPortsDbPortName ) );	
			}
			else
			{
				continue;
			}
			
			/*
			 * Below is to prevent dupes instead of just fprinting portName & portDir
			 */
			MGdbGoTop( pkgDb );
			if( MGdbSeek( pkgDb, fieldPortDir, portDir, fieldPortDir, exact ) == NULL )
			{
				MGdbAdd( pkgDb, portName, portDir, NULL );
			}
		}
	}

	/*
	 * Now that pkgDb is seeded recurse dependencyDb
	 */
	pkgDbQTY	= MGdbGetRecordQty( pkgDb );
	pkgDbIDX	= 1;
	while( pkgDbIDX < pkgDbQTY )
	{
		MGmStrcpy( portDir, MGdbGet( pkgDb, pkgDbIDX, fieldPortDir ) );
		MGdbGoTop( property->dependencyPortsDb );
		while( ( dependencyDirPtr = MGdbSeek( property->dependencyPortsDb,
							property->fieldDependencyDbPortDir,
							portDir,
							property->fieldDependencyDbDependencyPortDir,
							exact ) ) )
		{
			MGmStrcpy( dependencyDir, dependencyDirPtr );
			MGmStrcpy( dependencyName, MGdbGet( property->dependencyPortsDb,
							MGdbGetRecno( property->dependencyPortsDb ) - 1,
							property->fieldDependencyDbDependencyPortName ) );
			if( MGPMrIsRunDependency( property, portDir, dependencyDir ) == 0 )
			{
				continue;
			}
/*
fprintf( stdout, "DEBUG: dependencyDir-=>%s dependencyName-=>%s\n", dependencyDir, dependencyName );
while( fflush( stdout ) );
*/			
			MGdbGoTop( pkgDb );
			if( MGdbSeek( pkgDb, fieldPortDir, dependencyDir, fieldPortDir, exact ) )
			{
			}
			else
			{
				MGdbAdd( pkgDb, dependencyName, dependencyDir, NULL );
				pkgDbIDX = 0;
			}
		}
		pkgDbIDX++;
	}

	pkgDbQTY	= MGdbGetRecordQty( pkgDb );
	pkgDbIDX	= 1;
	while( pkgDbIDX < pkgDbQTY )
	{
		MGmStrcpy( dependencyDir, MGdbGet( pkgDb, pkgDbIDX, fieldPortDir ) );
		fprintf( stdout, "%s:%s\n", MGdbGet( pkgDb, pkgDbIDX, fieldPortName ), dependencyDir+1 );
		while( fflush( stdout ) );
		pkgDbIDX++;
	}

	MGdbDestroy( pkgDb );
	MGdbDestroy( property->availableDependenciesDb );
	MGdbDestroy( property->dependencyPortsDb );
	MGdbDestroy( property->installedPortsDb );

	free( dependencyDir );
	free( dependencyName );
	free( pkgDbFileName );
	free( portDir );
	free( portName );

	return(0);
}
