/*
	Copyright (C) 2005  Michael C. Shultz

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions
	are met:

	1. Redistributions of source code must retain the above copyright
	   notice, this list of conditions and the following disclaimer.
	2. Redistributions in binary form must reproduce the above copyright
	   notice, this list of conditions and the following disclaimer in the
	   documentation and/or other materials provided with the distribution.

	THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
	IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
	OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
	IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
	INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
	DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
	THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
	THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

	Michael C. Shultz
	ringworm01@gmail.com
	Box 3238 Landers, CA 92285
*/

#include	<libMGPM.h>

int	MGPMrParse(  structProperty* property, char* availablePortName, char* availablePortDir, char* dependType, char* buffer, int* counterPtr )
{
	FILE*	dependencyDbStream	= NULL;
	char	exact[]			= "exact";
	char	id[]			= "MGPMrParse";
	char*	dependencyPortDir	= NULL;
	char*	dependencyPortName	= NULL;
	char*	optionsPtr		= NULL;
	int	idx1			= 0;
	int	idx2			= 0;

	dependencyPortDir	= (char*)calloc( MAXBUFFERSIZE, 1 );
	dependencyPortName	= (char*)calloc( MAXBUFFERSIZE, 1 );
	
	idx1		= 0;
	idx2		= 0;

	/*
	 * displaying run dependencies ( every other record ) here....
	 */
	if( property->pmMode != 2 && strcmp( dependType, "RUN_DEPENDS" ) == 0 )
	{
		counterPtr[0]--;

		MGdbGoTop( property->configDb );
		optionsPtr	= MGdbSeek( property->configDb, property->fieldConfigDbKey, availablePortDir, property->fieldConfigDbValue, exact );
		if( optionsPtr )
		{
			fprintf( stdout, "%05d %s %s %s\n", counterPtr[0], availablePortName, availablePortDir, optionsPtr );
			while( fflush( stdout ) );
		}
		else
		{
			fprintf( stdout, "%05d %s %s\n", counterPtr[0], availablePortName, availablePortDir );
		}
		optionsPtr	= NULL;
	}

	MGmFopen( dependencyDbStream, property->availableDependenciesDbFileName, "a" );
	if( dependencyDbStream == 0 )
	{
		fprintf( stdout, "%s %s error: could not open %s\n", id, PACKAGE_VERSION, property->availableDependenciesDbFileName );
		perror( "system message" );
		return( 1 );
	}

	while( idx1 < MAXBUFFERSIZE )
	{
		if( buffer[idx1] == ':' )
		{
			idx1++;
			idx2	= idx1;
			while( idx2 < MAXBUFFERSIZE && buffer[idx2] != ' ' && buffer[idx2] != LINEFEED && buffer[idx2] != TAB && buffer[idx2] != ':' )
			{
				idx2++;
			}
			buffer[idx2]		= 0;
			if( idx1 + 11 < MAXBUFFERSIZE )
			{
				MGmStrcpy( dependencyPortDir, buffer + idx1 + 10 );
			}
			else
			{
				assert(0);
			}
			MGPMrGetPortName( property, dependencyPortDir, dependencyPortName );
			/*
			 * add dependency to availableDependenciesDb may contain dupes
			 */
/*
fprintf( stdout, "\t%s DEBUG:%s %s %s %s %s\n", id, availablePortDir, availablePortName, dependType, dependencyPortName, dependencyPortDir );
while( fflush( stdout ) );
*/
			fprintf( dependencyDbStream, "%s%c%s%c%s%c%s%c%s%c%c", availablePortDir, 0, availablePortName, 0, dependencyPortDir,
				0, dependencyPortName, 0, dependType, 0, LINEFEED );
/*

			MGdbGoTop( property->configDb );
			optionsPtr = MGdbSeek( property->configDb, property->fieldConfigDbKey, dependencyPortDir, property->fieldConfigDbValue, exact );
*/
			/*
			 * seen this error from a bad Makefile once
			 */
			if( MGrStrlen( dependencyPortName ) < 3 || MGrStrlen( dependencyPortDir ) < 3 )
			{
				fprintf( stdout, "%s %s error:  dependencyPortName-=>%s MGrStrlen( dependencyPortName )-=>%d dependencyPortDir-=>%s MGrStrlen( dependencyPortDir )-=>%d\n",
				id, PACKAGE_VERSION,dependencyPortName, MGrStrlen( dependencyPortName ), dependencyPortDir, MGrStrlen( dependencyPortDir ) );		 
				fprintf( stdout, "%s %s error:  Error in Makefile???\n", id, PACKAGE_VERSION );
				while( fflush( stdout ) );
				fclose( dependencyDbStream );
				free( dependencyPortDir );
				free( dependencyPortName );
				return( 1 );
			}

			/*
			 * if dependency not in availablePortsDb add it
			 */
			MGdbGoTop( property->availablePortsDb );
			if( MGdbSeek( property->availablePortsDb, property->fieldAvailablePortsDbPortDir, dependencyPortDir,
				property->fieldAvailablePortsDbPortDir, exact ) == 0 )
			{
				MGdbAdd( property->availablePortsDb, dependencyPortName, dependencyPortDir, "0", NULL);
			}
		}
		idx1++;
	}	

	fclose( dependencyDbStream );
	free( dependencyPortDir );
	free( dependencyPortName );

	return(0);
}
