/*
	Copyright (C) 2005  Michael C. Shultz

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions
	are met:

	1. Redistributions of source code must retain the above copyright
	   notice, this list of conditions and the following disclaimer.
	2. Redistributions in binary form must reproduce the above copyright
	   notice, this list of conditions and the following disclaimer in the
	   documentation and/or other materials provided with the distribution.

	THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
	IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
	OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
	IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
	INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
	DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
	THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
	THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

	Michael C. Shultz
	ringworm01@gmail.com
	Box 3238 Landers, CA 92285
*/

#include	<libMGPM.h>
/***********************************************************************/
/* awk script - used to parse pm-020.conf                              */
/***********************************************************************/
/*
#!/bin/sh

HELP="usage: dbConfigure.sh %%PREFIX%%/etc/portmanager/pm-020.conf %%PREFIX%%/share/portmanager/configure.db"
if test $1
then
	if test $2
	then
	
echo "keyzzNULLzzvaluezzNULLzz" > $2;awk 'BEGIN{ FS = "|" } NF == 3 && $1 !~ /#/  && $1 !~ /CATEGORY\/PORT/ && $2 !~ /#/ {print $1 "zzNULLzz" $2 "zzNULLzz"}' $1 >> $2

	else
		echo $HELP
	fi
else
	echo $HELP
fi
*/
/***********************************************************************/

int	rReadConfigureAwkConfigure( structProperty* property, char* txtFileName,  char* dbFileName );
int	rReadConfigureCleanPkgToolsDb( structProperty* property, char* pkgtoolsFileName );
int	rReadConfigureMergeRecords( structProperty* property, char* configDbFileName );
int	rReadConfigureParse( structProperty* property, char* configDbFileName, int flags );

int	MGPMrReadConfigure( structProperty* property, char* newPortDir )
{
	FILE*		fHandle			= NULL;
	FILE*		fHandle2		= NULL;
	FILE*		pHandle			= NULL;
	char		exact[]			= "exact";
	char		id[]			= "MGPMrReadConfigure";
	char		ignore[]		= "/IGNORE";
	char*		buffer			= NULL;
	char*		command			= NULL;
	char*		configFileNameOld	= NULL;
	char*		oldPortDir		= NULL;
	char*		optionsPtr		= NULL;
	char*		pkgtoolsFileName	= NULL;
	char*		portupgradeFileName	= NULL;
	char*		rubyFileName		= NULL;
	char**		cmd;
	classDb*	cacheDb;
	classDb*	configDb;
	classDb*	ignoreDb;

	buffer			= calloc( MAXBUFFERSIZE, 1 );
	command			= calloc( MAXBUFFERSIZE, 1 );
	configFileNameOld	= calloc( MAXSTRINGSIZE, 1 );
	oldPortDir		= calloc( MAXSTRINGSIZE, 1 );
	portupgradeFileName	= calloc( MAXSTRINGSIZE, 1 );
	rubyFileName		= calloc( MAXSTRINGSIZE, 1 );
	pkgtoolsFileName	= calloc( MAXSTRINGSIZE, 1 );

	/*
	 * Create/recreate installed.db
	 */
	cacheDb	= MGdbCreate(
			property->cacheDbFileName,
			property->fieldCacheDbPortDir,
			property->fieldCacheDbAvailablePortName,
			property->fieldCacheDbMakeFileSize,
			property->fieldCacheDbMakeFileTime,
			NULL );
	MGdbDestroy( cacheDb );
	if( MGPMrCreateInstalledDb( property ) )
	{
/*
		fprintf( stderr, "%s %s error: %s returned error\n", id, PACKAGE_VERSION, "MGPMrCreateInstalledDb" );
		while( fflush( stderr ) );
*/
		free( buffer );
		free( command );
		free( configFileNameOld );
		free( oldPortDir );
		free( pkgtoolsFileName );
		free( portupgradeFileName );
		free( rubyFileName );
		return( 1 );
	}

	/*************************************/
	/* update old conf file if it exists */
	/*************************************/
	MGmStrcpy( configFileNameOld, ETCDIR);
	MGmStrcat( configFileNameOld, "/portmanager/portmanager.conf" ); 

	if( MGrIfFileExist( configFileNameOld ) )
	{
		cmd	= malloc( sizeof( char* ) * 4 );
		cmd[0]	= malloc( MAXSTRINGSIZE );
		cmd[1]	= malloc( MAXSTRINGSIZE );
		cmd[2]	= malloc( MAXSTRINGSIZE );
		cmd[3]	= 0;

		MGmStrcpy( cmd[0], "/bin/mv" );
		MGmStrcpy( cmd[1], configFileNameOld );
		MGmStrcpy( cmd[2], configFileNameOld );
		MGmStrcat( cmd[2], "_OLD" );
		fprintf( stdout, "moving %s to %s\n", configFileNameOld, cmd[2] );
		fprintf( stdout, "portmanager configuration file is now called %s. Please review %s%s and %s and make any nessesary changes.\n",
			property->configConfFileName, property->configConfFileName, ".SAMPLE", configFileNameOld );
		while( fflush( stdout ) );

		MGmSystem( cmd, NULL );

		free( cmd[2] );		
		free( cmd[1] );		
		free( cmd[0] );		
		free( cmd );
	}

	/***************************************************************************/
	/* if pm-020.conf missing then copy pm-020.conf.SAMPLE to pm-020.conf */
	/***************************************************************************/
	if( MGrIfFileExist( property->configConfFileName ) == 0 )
	{
		cmd	= malloc( sizeof( char* ) * 4 );
		cmd[0]	= malloc( MAXSTRINGSIZE );
		cmd[1]	= malloc( MAXSTRINGSIZE );
		cmd[2]	= malloc( MAXSTRINGSIZE );
		cmd[3]	= 0;

		MGmStrcpy( cmd[0], "/bin/cp" );
		MGmStrcpy( cmd[1], property->configConfFileName );
		MGmStrcat( cmd[1], ".SAMPLE " );
		MGmStrcpy( cmd[2], property->configConfFileName );

		MGmSystem( cmd, NULL );

		free( cmd[2] );		
		free( cmd[1] );		
		free( cmd[0] );		
		free( cmd );
	}

	rReadConfigureAwkConfigure( property, property->configConfFileName,  property->configDbFileName );

	/*
	 * Create a record for port that isn't installed yet if portmanager
	 * is being used to add a new port
	 */
	if( MGrStrlen( newPortDir ) > 1 )
	{
		configDb	= MGdbOpen( property->configDbFileName );
		MGdbAdd( configDb, newPortDir, " ", NULL );
		MGdbDestroy( configDb );
	}

	/*
	 * Is ruby installed?
	 */
	MGmStrcpy( rubyFileName, BINDIR );
	MGmStrcat( rubyFileName, "/ruby" );
	MGmStrcpy( pkgtoolsFileName, ETCDIR );
	MGmStrcat( pkgtoolsFileName, "/pkgtools.conf" );
	MGmStrcpy( portupgradeFileName, PREFIXDIR );
	MGmStrcat( portupgradeFileName, "/sbin/portupgrade" );

	if( MGrIfFileExist( rubyFileName ) &&
		MGrIfFileExist( pkgtoolsFileName ) &&
		MGrIfFileExist( portupgradeFileName )  )
	{
		/*
		 *  run ruby script
		 */
		MGmStrcpy( command, rubyFileName );
		MGmStrcat( command, " " );
		MGmStrcat( command, SHAREDIR );
		MGmStrcat( command, "/pkgtools-to-portmanager.rb" );
		pHandle	= popen( command, "r" );
		fread( buffer, MAXBUFFERSIZE, 1, pHandle );
		pclose( pHandle );
		MGmStrcpy( configFileNameOld, ETCDIR );
		MGmStrcat( configFileNameOld, "/portmanager/pm-039-pkg-tools.conf" );
		MGmFopen( fHandle, configFileNameOld, "w" );
		fprintf( fHandle, "%s\n", buffer );
		fclose( fHandle );
		MGmStrcpy( command, SHAREDIR );
		MGmStrcat( command, "/pkgtools.db" );
		rReadConfigureAwkConfigure( property, configFileNameOld, command );
	}
	else
	{
		property->NoPkgtools	= 1;
	}
	MGmStrcpy( command, SHAREDIR );
	MGmStrcat( command, "/pkgtools.db" );

	if( property->NoPkgtools == 0 )
	{
		/*
		 * Add contents of pkgtools.db to property->configDbFileName
		 */

		rReadConfigureCleanPkgToolsDb( property, command );
		MGmStrcpy( command, SHAREDIR );
		MGmStrcat( command, "/pkgtools.db" );

		MGmFopen( fHandle, command, "r" );
		MGmFopen( fHandle2, property->configDbFileName, "a" )

		fread( buffer, 1, MGrFileSize( command ), fHandle );
		fwrite( buffer, 1, MGrFileSize( command ), fHandle2 );

		fclose( fHandle );
		fclose( fHandle2 );
	}

/*
	MGmStrcpy( command, "rm -f " );
	MGmStrcat( command, SHAREDIR );
	MGmStrcat( command, "/pkgtools.db" );
	system( command );
*/

	rReadConfigureMergeRecords( property, property->configDbFileName );


	rReadConfigureParse( property, property->configDbFileName, 0 ); /* combine records */
	rReadConfigureParse( property, property->configDbFileName, 1 ); /* for IGNORE, STOP and START don't combine records */

	configDb	= MGdbOpen( property->configDbFileName );
	ignoreDb	= MGdbOpen( property->ignoreDbFileName );

	MGdbGoTop( configDb );
	while( ( optionsPtr = MGdbSeek( configDb, property->fieldConfigDbKey, ignore, property->fieldConfigDbValue, exact ) ) )
	{
		if( optionsPtr[0] != '/' )
		{
			oldPortDir[0]	= '/';
			oldPortDir[1]	= 0;
			MGmStrcat( oldPortDir, optionsPtr );
		}
		else
		{
			MGmStrcpy( oldPortDir, optionsPtr );
		}
		MGdbAdd( ignoreDb, oldPortDir, "set to ignore in pm-020.conf", NULL );
	}

	MGdbDestroy( configDb );
	MGdbDestroy( ignoreDb );

	free( buffer );
	free( command );
	free( configFileNameOld );
	free( oldPortDir );
	free( pkgtoolsFileName );
	free( portupgradeFileName );
	free( rubyFileName );
	return( 0 );
}

int	rReadConfigureParse( structProperty* property, char* configDbFileName, int flags )
{
	char		id[]			= "rReadConfigureParse";
	char*		contains		= NULL;
	char*		key			= NULL;
	char*		rightSide		= NULL;
	char*		value			= NULL;
	classDb*	configDb;
	classDb*	installedPortsDb;
	int		configDbIDX		= 1;
	int		configDbQTY		= 0;

	contains	= calloc( MAXBUFFERSIZE, 1 );
	key		= calloc( MAXSTRINGSIZE, 1 );
	rightSide	= calloc( MAXBUFFERSIZE, 1 );
	value		= calloc( MAXBUFFERSIZE, 1 );

	configDb		= MGdbOpen( property->configDbFileName );
	installedPortsDb	= MGdbOpen( property->installedPortsDbFileName );
	
	configDbQTY	= MGdbGetRecordQty( configDb );
	configDbIDX	= 1;

	while( configDbIDX < configDbQTY )
	{
		MGmStrcpy( key, MGdbGet( configDb, configDbIDX, property->fieldConfigDbKey ) );
		if( flags  == 0 )
		{
			if( 	( strcmp( key, "fieldConfigDbKey" ) == 0 )	||
				( strcmp( key, "/IGNORE" ) == 0 )		||
				( strcmp( key, "/START" ) == 0 )		||
				( strcmp( key, "/STOP" ) == 0 ) )
			{
				configDbIDX++;
				continue;
			}
		}
		else
		{
			if( 	( strcmp( key, "fieldConfigDbKey" ) == 0 )	||
				( strcmp( key, "/IGNORE" ) && strcmp( key, "/START" ) &&  strcmp( key, "/STOP" ) ) )
			{
				configDbIDX++;
				continue;
			}
		}
		if( flags )
		{
			MGmStrcpy( value, MGdbGet( configDb, configDbIDX, property->fieldConfigDbValue ) );
			if( value[0] != '/' )
			{
				value[0]	= '/';
				MGmStrcpy( value + 1, MGdbGet( configDb, configDbIDX, property->fieldConfigDbValue ) );
			}
		}
		else
		{
			MGmBuffercpy( value, MGdbGet( configDb, configDbIDX, property->fieldConfigDbValue ), MAXBUFFERSIZE );
		}
		/*
		 * leading astrix
		 */
		if( key[1] == '*' || ( flags && value[1] == '*' ) )
		{
			if( flags )
			{
				MGmStrcpy( rightSide, value + 2 );
			}
			else
			{
				MGmStrcpy( rightSide, key + 2 );
			}
			/*
			 * leading and trailing astrix
			 */
			if( rightSide[MGrStrlen(rightSide)-1] == '*' )
			{
				rightSide[MGrStrlen(rightSide)-1] = 0;
				MGmStrcpy( contains, rightSide );
				configDb	= MGdbDelete( configDb, configDbIDX );
				MGdbGoTop( installedPortsDb );
				while( MGdbSeek( installedPortsDb,
						property->fieldInstalledPortsDbPortDir,
						contains,
						property->fieldInstalledPortsDbPortDir,
						"contains" ) )
				{
					if( flags )
					{
						MGmStrcpy( value, MGdbGet( installedPortsDb,
									MGdbGetRecno( installedPortsDb ) - 1,
									property->fieldInstalledPortsDbPortDir ) );
					}
					else
					{
						MGmStrcpy( key, MGdbGet( installedPortsDb,
									MGdbGetRecno( installedPortsDb ) - 1,
									property->fieldInstalledPortsDbPortDir ) );
					}
					MGdbAdd( configDb, key, value, NULL );
				}		 
				configDbIDX	= 0;
				configDbQTY	= MGdbGetRecordQty( configDb );
			}
			else	/* leading astrix only */
			{
				if( flags )
				{
					MGmStrcpy( contains, value + 2 );
				}
				else
				{
					MGmStrcpy( contains, key + 2 );
				}
				configDb	= MGdbDelete( configDb, configDbIDX );
				
				MGdbGoTop( installedPortsDb );
				while( MGdbSeek( installedPortsDb,
						property->fieldInstalledPortsDbPortDir,
						contains,
						property->fieldInstalledPortsDbPortDir,
						"endsWith" ) )
				{
					if( flags )
					{
						MGmStrcpy( value, MGdbGet( installedPortsDb,
									MGdbGetRecno( installedPortsDb ) - 1,
									property->fieldInstalledPortsDbPortDir ) );
					}
					else
					{
						MGmStrcpy( key, MGdbGet( installedPortsDb,
									MGdbGetRecno( installedPortsDb ) - 1,
									property->fieldInstalledPortsDbPortDir ) );
					}
					MGdbAdd( configDb, key, value, NULL );
				}
				configDbIDX	= 0;
				configDbQTY	= MGdbGetRecordQty( configDb );
			}
		}
		else
		{
			/*
			 * trailing astrix only
			 */
			if( flags )
			{
				if( value[MGrStrlen(value)-1] == '*' )
				{
					value[MGrStrlen(value)-1] = 0;
					MGmStrcpy( contains, value );
					configDb	= MGdbDelete( configDb, configDbIDX );
					MGdbGoTop( installedPortsDb );
					while( MGdbSeek( installedPortsDb,
							property->fieldInstalledPortsDbPortDir,
							contains,
							property->fieldInstalledPortsDbPortDir,
							"begins" ) )
					{
						MGmStrcpy( value, MGdbGet( installedPortsDb,
									MGdbGetRecno( installedPortsDb ) - 1,
									property->fieldInstalledPortsDbPortDir ) );
						MGdbAdd( configDb, key, value, NULL );
					}		 
					configDbIDX	= 0;
					configDbQTY	= MGdbGetRecordQty( configDb );
				}
			}
			else
			{
				if( key[MGrStrlen(key)-1] == '*' )
				{
					key[MGrStrlen(key)-1] = 0;
					MGmStrcpy( contains, key );
					configDb	= MGdbDelete( configDb, configDbIDX );
					MGdbGoTop( installedPortsDb );
					while( MGdbSeek( installedPortsDb,
							property->fieldInstalledPortsDbPortDir,
							contains,
							property->fieldInstalledPortsDbPortDir,
							"begins" ) )
					{
						MGmStrcpy( key, MGdbGet( installedPortsDb,
									MGdbGetRecno( installedPortsDb ) - 1,
									property->fieldInstalledPortsDbPortDir ) );
						MGdbAdd( configDb, key, value, NULL );
					}		 
					configDbIDX	= 0;
					configDbQTY	= MGdbGetRecordQty( configDb );
				}
			}
		}
		configDbIDX++;
	}

	MGdbDestroy( configDb );
	MGdbDestroy( installedPortsDb );
	free( contains );
	free( key );
	free( rightSide );
	free( value );

	/*
	 * don't combine IGNORE, STOP, START records
	 */
	if( flags  == 0 )
	{
		rReadConfigureMergeRecords( property, configDbFileName );
	}
	return( 0 );
}

/*
 * gets rid of records with NULL value field
 */
int	rReadConfigureCleanPkgToolsDb( structProperty* property, char* pkgtoolsFileName )
{
	char		id[]		= "rReadConfigureCleanPkgToolsDb";
	char*		value		= NULL;
	classDb*	pkgtoolsDb;
	int		pkgtoolsIDX	= 1;
	int		pkgtoolsQTY	= 0;

	value		= calloc( 0xfff, 1 );

	if( MGrIfFileExist( pkgtoolsFileName ) )
	{
		pkgtoolsDb	= MGdbOpen( pkgtoolsFileName );
		pkgtoolsQTY	= MGdbGetRecordQty( pkgtoolsDb );
	}
	else
	{
		free( value );
		return( 1 );
	}

	while( pkgtoolsIDX < pkgtoolsQTY )
	{
		MGmStrcpy( value, MGdbGet( pkgtoolsDb, pkgtoolsIDX, property->fieldConfigDbValue ) );
		if( MGrStrlen( value ) < 1 )
		{
			pkgtoolsDb	= MGdbDelete( pkgtoolsDb, pkgtoolsIDX );
			pkgtoolsQTY	= MGdbGetRecordQty( pkgtoolsDb );
			pkgtoolsIDX	= 0;
		}
		pkgtoolsIDX++;
	}

	free( value );
	MGdbDestroy( pkgtoolsDb );
	return( 0 );
}

/*
 * combines duplicate entries 
 */
int	rReadConfigureMergeRecords( structProperty* property, char* configDbFileName )
{
	char		id[]			= "rReadConfigureMergeRecords";
	char		exact[]			= "exact";
	char*		key			= NULL;
	char*		value			= NULL;
	classDb*	configDb;
	int		configDbIDX		= 1;
	int		configDbQTY		= 0;
	int		deleteRecordNumber	= 0;

	key	= calloc( MAXSTRINGSIZE, 1 );
	value	= calloc( MAXBUFFERSIZE, 1 );

	configDb	= MGdbOpen( configDbFileName );
	configDbQTY	= MGdbGetRecordQty( configDb );
	configDbIDX	= 1;
	while( configDbIDX < configDbQTY )
	{
		MGmStrcpy( key, MGdbGet( configDb, configDbIDX, property->fieldConfigDbKey ) );
		if( 	( strcmp( key, "fieldConfigDbKey" ) == 0 ) ||
			( strcmp( key, "/IGNORE" ) == 0 )	||
			( strcmp( key, "/START" ) == 0 )		||
			( strcmp( key, "/STOP" ) == 0 ) )
		{
			configDbIDX++;
			continue;
		}
		MGmBuffercpy( value, MGdbGet( configDb, configDbIDX, property->fieldConfigDbValue ), MAXBUFFERSIZE );
		MGdbGoTop( configDb );
		MGdbSeek( configDb, property->fieldConfigDbKey, key, property->fieldConfigDbKey, exact );
		/*
		 * Double seek to find duplicate keys/merge records
		 */
		if( MGdbSeek( configDb, property->fieldConfigDbKey, key, property->fieldConfigDbKey, exact ) )
		{
			deleteRecordNumber	= MGdbGetRecno( configDb ) - 1;
			MGmStrcat( value, " " );
			MGmBuffercat( value, MGdbGet( configDb, deleteRecordNumber, property->fieldConfigDbValue ), MAXBUFFERSIZE );
			configDb	= MGdbDelete( configDb, deleteRecordNumber );
			configDb	= MGdbDelete( configDb, configDbIDX );
			MGdbAdd( configDb, key, value, NULL );
			configDbIDX	= 0;
			configDbQTY	= MGdbGetRecordQty( configDb );
		}
		configDbIDX++;
	}

	MGdbDestroy( configDb );
	free( key );
	free( value );
	return( 0 );
}

/*
 * Uses awk to parse *.conf into *.db
 */
int	rReadConfigureAwkConfigure( structProperty* property, char* txtFileName,  char* dbFileName )
{
			FILE*	configDbStream		= NULL;	
			char	id[]			= "rReadConfigureAwkConfigure";
			char*	buffer			= NULL;
			char*	command			= NULL;
			char*	nullBase		= NULL;
			int	bufferIdx		= 0;
			int	configDbFileSize	= 0;

	buffer	= calloc( MAXBUFFERSIZE, 1 );
	command	= calloc( MAXBUFFERSIZE, 1 );

	/*********************/
	/* parse pm-020.conf */
	/*********************/
	MGmStrcpy( command, property->fieldConfigDbKey );
	MGmStrcat( command, "zzNULLzz" );
	MGmStrcat( command, property->fieldConfigDbValue );	
	MGmStrcat( command, "zzNULLzz" );

	MGmFopen( configDbStream, dbFileName, "w" );
	fprintf( configDbStream, "%s\n", command );
	fclose( configDbStream );

	/*
	 * to be secure this needs to be written as an awk sript,
	 * then the script run from execve
	 */
	MGmStrcpy( command, "awk 'BEGIN{ FS = \"|\" } NF == 3 && $1 !~ /#/  && $1 !~ /CATEGORY\\/PORT/ && $2 !~ /#/ {print \"/\" $1 \"zzNULLzz\" $2 \"zzNULLzz\"}' " );
	MGmStrcat( command, txtFileName );
	MGmStrcat( command, " >> " );
	MGmStrcat( command, dbFileName );
	if( system( command ) )
	{
		fprintf( stderr, "%s %s: error: command %s returned an error\n", id, PACKAGE_VERSION, command );
		while( fflush( stderr ) );
		assert( 0 );
	}

	configDbFileSize	= MGrFileSize(  dbFileName );
	MGmFopen( configDbStream, dbFileName, "r" );
	fread( buffer, MAXBUFFERSIZE, 1, configDbStream ); 
	buffer[MAXBUFFERSIZE - 1]	= 0;	/* so buffer may be treated as a string */

	fclose( configDbStream );

	/* convert zzNULLzz to NULLs */
	while( strstr( buffer + bufferIdx,  "zzNULLzz" ) )
	{
		nullBase	= strstr( buffer + bufferIdx,  "zzNULLzz" );
		nullBase[0]	= 0;
		nullBase[1]	= 0;
		memcpy( (char*)nullBase+1, (char*)nullBase+8, MGrBufferlen( nullBase+8, MAXBUFFERSIZE ) + 1  );
		bufferIdx	= (nullBase+1) - buffer;
		configDbFileSize	-= 7;
	}
	MGmFopen( configDbStream, dbFileName, "w" );
	fwrite( buffer, configDbFileSize, 1, configDbStream );

	free(buffer);
	free( command );
	fclose( configDbStream );

	return( 0 );
}
