/*
	Copyright (C) 2005  Michael C. Shultz

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions
	are met:

	1. Redistributions of source code must retain the above copyright
	   notice, this list of conditions and the following disclaimer.
	2. Redistributions in binary form must reproduce the above copyright
	   notice, this list of conditions and the following disclaimer in the
	   documentation and/or other materials provided with the distribution.

	THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
	IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
	OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
	IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
	INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
	DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
	THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
	THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

	Michael C. Shultz
	ringworm01@gmail.com
	Box 3238 Landers, CA 92285
*/

#include	<libMGPM.h>

int	rCheckForOldDependencies( 
		structProperty* property,
		char*	installedPortsDbPortDir,
		char*	oldDependencyName,
		char*	oldDependencyDir,
		int*	counterPtr );

int	MGPMrStatus( structProperty* property )
{
	char	id[]					= "MGPMrSinglePortStatus";
	char	availablePortsDbFlag[]			= "0";
	char	exact[]					= "exact";
	char*	availablePortsDbPortDirPtr		= NULL;
	char*	availablePortsDbPortNamePtr		= NULL;
	char*	installedPortsDbFlag			= NULL;
	char*	installedPortsDbPortDir			= NULL;
	char*	installedPortsDbPortName		= NULL;
	char*	installedPortsDbPortNamePtr		= NULL;
	char*	oldDependencyDir			= NULL;
	char*	oldDependencyName			= NULL;
	int	availablePortsDbIDX			= 0;
	int	availablePortsDbQTY			= 0;
	int	bufferSize				= 0xffff;
	int	counter					= 0;
	int	installedPortsDbIDX			= 0;
	int	old					= 0;
	int	returnCode				= 0;
	int*	counterPtr				= NULL;
	
	counterPtr	= &counter;
	
	oldDependencyName		= (char*)calloc( bufferSize, 1 );
	oldDependencyDir		= (char*)calloc( bufferSize, 1 );
	installedPortsDbPortDir		= (char*)calloc( bufferSize, 1 );
	installedPortsDbPortName	= (char*)calloc( bufferSize, 1 );

	/************************************************************************/
	/* start up messages							 */
	/************************************************************************/
	fprintf( stdout,"%s\n", SINGLE_LINES );
	if( property->forced )
	{
		fprintf( stdout," Port Status Report \"forced mode\"\n" );
	}
	else
	{
		fprintf( stdout," Port Status Report\n" );
	}
	fprintf( stdout,"%s\n", SINGLE_LINES );
	while( fflush( stdout ) );
	/************************************************************************/
	/* open/create data bases						*/
	/************************************************************************/
	property->availablePortsDb		= MGdbOpen( property->availablePortsDbFileName );
	property->availableDependenciesDb	= MGdbOpen( property->availableDependenciesDbFileName );
	property->dependencyPortsDb		= MGdbOpen( property->dependencyPortsDbFileName );
	property->installedPortsDb		= MGdbOpen( property->installedPortsDbFileName );
	property->oldPortsDb			= MGdbCreate(
							property->oldPortsDbFileName,
							property->fieldOldPortsDbOldPortDir,
							property->fieldOldPortsDbOldPortName,
							property->fieldOldPortsDbReason,
							property->fieldOldPortsDbNewPortName,
							property->fieldOldPortsDbNewPortDir,
							NULL );

	availablePortsDbQTY	= MGdbGetRecordQty( property->availablePortsDb );
	availablePortsDbIDX	= 1;
	 
	/*
	 * get each availablePortsDb record
	 */
	while( availablePortsDbIDX < availablePortsDbQTY )
	{
		old	= 0;
		availablePortsDbPortDirPtr	= MGdbGet( property->availablePortsDb,
							availablePortsDbIDX,
							property->fieldAvailablePortsDbPortDir );

		availablePortsDbPortNamePtr	= MGdbGet( property->availablePortsDb,
							availablePortsDbIDX,
							property->fieldAvailablePortsDbPortName );

		MGmStrcpy( availablePortsDbFlag, MGdbGet( property->availablePortsDb,
							availablePortsDbIDX,
							property->fieldAvailablePortsDbFlag ) );

		/*
		 * is availablePortsDb fieldAvailablePortsDbPortName in  installedPortsDb?
		 * yes - port may be current or built with old dependencies
		 * 	if property->forced =1 portmay be ORIGINAL or REBUILT or  built with old dependencies
		 * no  - port may be out of date or missing
		 */
		MGdbGoTop( property->installedPortsDb );
		installedPortsDbPortNamePtr	= MGdbSeek( property->installedPortsDb,
							property->fieldInstalledPortsDbPortName,
							availablePortsDbPortNamePtr,
							property->fieldInstalledPortsDbPortName,
							exact );
		/*
		 * if "is availablePortsDb fieldAvailablePortsDbPortName in installedPortsDb" = yes
		 * a) test it's dependents:
		 * 	1. If all RUN dependencies are in installedPortsDb then port may be current
		 *		a. Are all port's dependencies in dependencyPortsDb current?
		 *			1. yes port is current
		 *			2. No - port built with old dependencies
		 *	2. If NOT all RUN dependencies are in installedPortsDb then port is built with old/missing dependencies
		 */
		if( installedPortsDbPortNamePtr )
		{
			MGmStrcpy( installedPortsDbPortName, installedPortsDbPortNamePtr );
			
			installedPortsDbIDX	= MGdbGetRecno( property->installedPortsDb ) - 1;
			MGmStrcpy( installedPortsDbPortDir, MGdbGet( property->installedPortsDb,
								installedPortsDbIDX,
								property->fieldInstalledPortsDbPortDir ) );

			if( ( returnCode = rCheckForOldDependencies( property,
									installedPortsDbPortDir,
									oldDependencyName,
									oldDependencyDir,
									counterPtr ) ) )
			{
				if( returnCode	== 2 )
				{
					fprintf( stdout, "%05d PRISTINE MODE: %-35s %-35s has incorrect %s entry in +CONTENTS file\n",
						counter,
			 			installedPortsDbPortName,
			 			installedPortsDbPortDir,
						oldDependencyName ); 
					while( fflush( stdout ) );

					MGdbAdd( property->oldPortsDb,
						installedPortsDbPortDir,
						installedPortsDbPortName,
						"PRISTINE MODE: incorrect +CONTENTS file entry",
						oldDependencyName,
						oldDependencyDir,
						NULL );
				}
				else
				{
					fprintf( stdout, "%05d have:%-35s %-35s built with OLD dependency: %s\n",
						counter,
			 			installedPortsDbPortName,
			 			installedPortsDbPortDir,
						oldDependencyName ); 
					while( fflush( stdout ) );

					MGdbAdd( property->oldPortsDb,
						installedPortsDbPortDir,
						installedPortsDbPortName,
						"built with OLD dependency",
						oldDependencyName,
						oldDependencyDir,
						NULL );
				}

			}
			else
			{
				if( property->forced )
				{
					if( strcmp( availablePortsDbFlag, "2" ) == 0 )
					{
						fprintf( stdout, "%05d have:%-35s %-35s REBUILT\n",
						counter,
						installedPortsDbPortName,
						installedPortsDbPortDir ); 
						while( fflush( stdout ) );
					}
					else
					{
						fprintf( stdout, "%05d have:%-35s %-35s ORIGINAL\n",
							counter,
			 				installedPortsDbPortName,
							installedPortsDbPortDir ); 
						while( fflush( stdout ) );

						MGdbAdd( property->oldPortsDb,
							installedPortsDbPortDir,
							installedPortsDbPortName,
							"ORIGINAL",
							installedPortsDbPortName,
							installedPortsDbPortDir,
							NULL );
					}
				}
				else
				{
					MGdbGoTop( property->dependencyPortsDb );
					while( MGdbSeek( property->dependencyPortsDb,
							property->fieldDependencyDbPortDir,
							installedPortsDbPortDir,
							property->fieldDependencyDbPortDir,
							exact ) )
					{
						MGdbGoTop( property->installedPortsDb );

						if(  MGdbSeek( property->installedPortsDb,
							property->fieldInstalledPortsDbPortName,
							MGdbGet( property->dependencyPortsDb,
								MGdbGetRecno( property->dependencyPortsDb ) - 1,
								property->fieldDependencyDbDependencyPortName ),
							property->fieldInstalledPortsDbPortName,
							exact ) == NULL && MGPMrIsRunDependency( property, installedPortsDbPortDir,
												MGdbGet( property->dependencyPortsDb,
												MGdbGetRecno( property->dependencyPortsDb ) - 1,
												property->fieldDependencyDbDependencyPortDir ) ) )
						{
							fprintf( stdout, "%05d have:%-35s %-35s built with OLD dependency: %s\n",
								counter,
			 					installedPortsDbPortName,
			 					installedPortsDbPortDir,
								MGdbGet( property->dependencyPortsDb,
									MGdbGetRecno( property->dependencyPortsDb ) - 1,
									property->fieldDependencyDbDependencyPortName ) ); 
							while( fflush( stdout ) );

							MGdbAdd( property->oldPortsDb,
								installedPortsDbPortDir,
								installedPortsDbPortName,
								"built with OLD dependency",
								MGdbGet( property->dependencyPortsDb,
									MGdbGetRecno( property->dependencyPortsDb ) - 1,
									property->fieldDependencyDbDependencyPortName ),
								MGdbGet( property->dependencyPortsDb,
									MGdbGetRecno( property->dependencyPortsDb ) - 1,
									property->fieldDependencyDbDependencyPortDir ),
								NULL );
							old	= 1;
							break;
						}
						else
						{
							old = 0;
						}

					}		
					if( old == 0 )
					{
						fprintf( stdout, "%05d have:%-35s %-35s CURRENT\n",
							counter,
							installedPortsDbPortName,
							installedPortsDbPortDir ); 
						while( fflush( stdout ) );
					}
				}
			}
		}
		/*
		 * if "is availablePortsDb fieldAvailablePortsDbPortName in installedPortsDb" = no
		 * a) port is either out of date or missing
		 * 	1. If availablePortsDbPortDirPtr is in installedPortsDb then port is old
		 *	2. If availablePortsDbPortDirPtr is NOT in installedPortsDb then port is missing
		 */
		else
		{
			counterPtr[0]++;
			MGmStrcpy( installedPortsDbPortDir, availablePortsDbPortDirPtr );		
			MGdbGoTop( property->installedPortsDb );

			if( MGdbSeek(  property->installedPortsDb,
				property->fieldInstalledPortsDbPortDir,
				installedPortsDbPortDir,
				property->fieldInstalledPortsDbPortDir,
				exact ) )
			{
				MGmStrcpy( installedPortsDbPortName,
					MGdbGet( property->installedPortsDb,
						MGdbGetRecno( property->installedPortsDb ) -1,
						property->fieldInstalledPortsDbPortName ) );


				fprintf( stdout, "%05d have:%-35s %-35s OLD available: %s\n",
					counter,
			 		installedPortsDbPortName,
			 		installedPortsDbPortDir,
					availablePortsDbPortNamePtr ); 
				while( fflush( stdout ) );

				MGdbAdd( property->oldPortsDb,
					installedPortsDbPortDir,
					installedPortsDbPortName,
					"OLD",
					availablePortsDbPortNamePtr,
					availablePortsDbPortDirPtr,
					NULL );
			}		
			else
			{
				fprintf( stdout, "%05d ----:%-35s %-35s MISSING\n",
					counter,
			 		availablePortsDbPortNamePtr,
					availablePortsDbPortDirPtr ); 
				while( fflush( stdout ) );

				MGdbGoTop( property->availableDependenciesDb );
				if( MGdbSeek( property->availableDependenciesDb,
						property->fieldAvailableDependenciesDbDependencyDir,
						availablePortsDbPortDirPtr,
						property->fieldAvailableDependenciesDbDependencyDir,
						exact ) )
				{
					MGdbAdd( property->oldPortsDb,
						availablePortsDbPortDirPtr,
						availablePortsDbPortNamePtr,
						"MISSING dependency of",
						MGdbGet( property->availableDependenciesDb,
							MGdbGetRecno( property->availableDependenciesDb ) - 1,
							property->fieldAvailableDependenciesDbPortName ),
						MGdbGet( property->availableDependenciesDb,
							MGdbGetRecno( property->availableDependenciesDb ) - 1,
							property->fieldAvailableDependenciesDbPortDir ),
						NULL );
				}
				else
				{
					MGdbAdd( property->oldPortsDb,
						availablePortsDbPortDirPtr,
						availablePortsDbPortNamePtr,
						"MISSING",
						availablePortsDbPortNamePtr,
						availablePortsDbPortDirPtr,
						NULL );
				}
			}
		}
		availablePortsDbIDX++;
	}

	/************************************************************************/
	/* clean up and return							*/
	/************************************************************************/
	free( installedPortsDbFlag );
	free( installedPortsDbPortDir );
	free( installedPortsDbPortName );
	free( oldDependencyDir );
	free( oldDependencyName );

	MGdbDestroy( property->availableDependenciesDb );
	MGdbDestroy( property->availablePortsDb );
	MGdbDestroy( property->dependencyPortsDb );
	MGdbDestroy( property->installedPortsDb );
	MGdbDestroy( property->oldPortsDb );

	fprintf( stdout,"%s\n", DOUBLE_LINES );
	while( fflush( stdout ) );

	return( 0 );
}

/*
 * returns 1 is dependencies are old/missing and 0 if all is well
 * if returning 1, fill oldDependencyName oldDependencyDir
 */
int	rCheckForOldDependencies( 
		structProperty* property,
		char*	installedPortsDbPortDir,
		char*	oldDependencyName,
		char*	oldDependencyDir,
		int*	counterPtr )
{
	char	id[]						= "rCheckForOldDependencies";
	char*	availableDependenciesDbDependencyTypePtr	= NULL;
	char	exact[]						= "exact";
	char*	availableDependenciesDbDependencyNamePtr	= NULL;
	char*	installedPortsDbPortNamePtr			= NULL;
	
	counterPtr[0]++;

	/*
	 * Are all availableDependencies belonging to installedPortsDbPortName in installedPortsDb?
	 */
	MGdbGoTop( property->availableDependenciesDb );

	/*
	 * seek through availableDependenciesDb while
	 *	installedPortsDbPortDir is in availableDependenciesDb and its type
	 * 	is a BUILD_DEPENDS desired out come is installedPortsDbPortDir with
	 *	a RUN_DEPENDS at finish time
	 */
	while( ( availableDependenciesDbDependencyNamePtr	= MGdbSeek( property->availableDependenciesDb,
									property->fieldAvailableDependenciesDbPortDir,
									installedPortsDbPortDir,
									property->fieldAvailableDependenciesDbDependencyName,
									exact ) ) )
	{
		availableDependenciesDbDependencyTypePtr	= MGdbGet( property->availableDependenciesDb,
									MGdbGetRecno(  property->availableDependenciesDb ) -1,
									property->fieldAvailableDependenciesDbDependType );

		if( availableDependenciesDbDependencyTypePtr && strcmp( availableDependenciesDbDependencyTypePtr, "BUILD_DEPENDS" ) == 0 )
		{
			continue;
		}
		break;
	}			
											 
	/*
	 * installedPortsDbPortName has no available Dependencies, we are done here
	 */
	if( availableDependenciesDbDependencyNamePtr == NULL )
	{
		oldDependencyDir[0]	= 0;
		oldDependencyName[0]	= 0;
		return( 0 );
	}

	/*
	 * installedPortsDbPortName has at least one dependency, and its ether a RUN_DEPENDS, LIB_DEPENDS or a DEPENDS
	 */
	while( availableDependenciesDbDependencyNamePtr )
	{
		MGdbGoTop( property->installedPortsDb );
		installedPortsDbPortNamePtr			= MGdbSeek( property->installedPortsDb,
									property->fieldInstalledPortsDbPortName,
									availableDependenciesDbDependencyNamePtr,
									property->fieldInstalledPortsDbPortName,
									exact );
		/*
		 * availableDependency is not in installedPortsDb
		 */
		if( installedPortsDbPortNamePtr == NULL )
		{
			MGmStrcpy( oldDependencyName, availableDependenciesDbDependencyNamePtr );
			MGmStrcpy( oldDependencyDir, MGdbGet(  property->availableDependenciesDb,
							MGdbGetRecno(  property->availableDependenciesDb ) - 1,
							property->fieldAvailableDependenciesDbDependencyDir ) );
			return( 1 );
		}
		availableDependenciesDbDependencyNamePtr	= MGdbSeek( property->availableDependenciesDb,
									property->fieldAvailableDependenciesDbPortDir,
									installedPortsDbPortDir,
									property->fieldAvailableDependenciesDbDependencyName,
									exact );
	}

	if( property->pristine || property->forced )
	{
		/*
		 * One last test is to check that each of installedPortsDbPortDir's dependencies found in dependencyDb are indeed in
		 * installedDB, this verifies that +CONTENTS file is correct
		 */
		 MGdbGoTop( property->dependencyPortsDb );
		 while( MGdbSeek( property->dependencyPortsDb,
	 			property->fieldDependencyDbPortDir,
				installedPortsDbPortDir,
				property->fieldDependencyDbPortDir,
				exact ) )
		{
			MGmStrcpy( oldDependencyName, MGdbGet( property->dependencyPortsDb,
							MGdbGetRecno( property->dependencyPortsDb ) - 1,
							property->fieldDependencyDbDependencyPortName ) );

			MGdbGoTop( property->installedPortsDb );
			if( MGdbSeek( property->installedPortsDb,
				property->fieldInstalledPortsDbPortName,
				oldDependencyName,
				property->fieldInstalledPortsDbPortName,
				exact ) == NULL )
			{
				MGmStrcpy( oldDependencyDir, MGdbGet( property->dependencyPortsDb,
							MGdbGetRecno( property->dependencyPortsDb ) - 1,
							property->fieldDependencyDbDependencyPortDir ) );
				return( 2 );
			}
		}
	}
	oldDependencyDir[0]	= 0;
	oldDependencyName[0]	= 0;
	return( 0 );
}
