/*
	Copyright (C) 2005  Michael C. Shultz

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions
	are met:

	1. Redistributions of source code must retain the above copyright
	   notice, this list of conditions and the following disclaimer.
	2. Redistributions in binary form must reproduce the above copyright
	   notice, this list of conditions and the following disclaimer in the
	   documentation and/or other materials provided with the distribution.

	THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
	IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
	OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
	IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
	INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
	DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
	THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
	THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

	Michael C. Shultz
	ringworm01@gmail.com
	Box 3238 Landers, CA 92285
*/
#include	<libMGPM.h>

int	rBsdPortMkPatch( structProperty* property, structLocalProperty* localProperty );
int	rCleanDir( char* portDir, char* workDir );
int	rCleanUp( structProperty* property, structLocalProperty* localProperty );

int	MGPMrUpdate( structProperty* property, char* oldPortDir, char* oldPortName )
{
	FILE*	pHandle						= NULL;
	char	BROKEN[]					= "BROKEN";
	char	FORBIDDEN[]					= "FORBIDDEN";
	char	IGNORE[]					= "IGNORE";
	char	START[]						= "/START";
	char	STOP[]						= "/STOP";
	char	backUp[]					= "--back-up";
	char	bu[]						= "-bu";
	char	exact[]						= "exact";
	char	id[]						= "MGPMrUpdate";
	char	ip[]						= "-ip";
	char	l[]						= "-l";
	char	log[]						= "--log";
	char	strike[]					= "0";
	char	y[]						= "-y";
	char	yes[]						= "--yes";
	char*	availableDependenciesDbDependencyDirPtr		= NULL;
	char*	availableDependenciesDbDependencyNamePtr	= NULL;
	char*	bufferBase					= NULL;
	char*	ipPtr						= NULL;
	char*	oldPortDirPtr					= NULL;
	char*	startPortDirPtr					= NULL;
	char*	stinker						= NULL;
	char*	stopPortDirPtr					= NULL;
	char*	strikePtr					= NULL;
	char**	cmd;
	int	answer						= 1;
	int	availableDependenciesDbIDX			= 0;
	int	availableDependenciesDbQTY			= 0;
	int	availablePortsDbIDX				= 0;
	int	doubleBreak					= 0;
	int	errorCode					= 0;
	int	idx						= 0;
	int	installedPortsDbIDX				= 0;
	int	installedPortsDbQTY				= 0;

	structLocalProperty	localProperty;

	property->optionsChanged	= 0;

	localProperty.environment	= malloc( sizeof( char** ) * 4 );
	localProperty.environment[0]	= malloc( MAXSTRINGSIZE );
	localProperty.environment[1]	= malloc( MAXSTRINGSIZE );
	localProperty.environment[2]	= malloc( MAXSTRINGSIZE );
	localProperty.environment[3]	= 0;

	localProperty.CURDIR			= calloc( MAXSTRINGSIZE, 1 );
	localProperty.afterOptionsFileSize	= calloc( MAXSTRINGSIZE, 1 );
	localProperty.afterOptionsFileTime	= calloc( MAXSTRINGSIZE, 1 );
	localProperty.batchCheck		= calloc( MAXSTRINGSIZE, 1 );
	localProperty.beforeOptionsFileSize	= calloc( MAXSTRINGSIZE, 1 );
	localProperty.beforeOptionsFileTime	= calloc( MAXSTRINGSIZE, 1 );
	localProperty.buffer			= calloc( MAXBUFFERSIZE, 1 );
	localProperty.command			= calloc( MAXBUFFERSIZE, 1 );
	localProperty.commandRestart		= calloc( MAXBUFFERSIZE, 1 );
	localProperty.installedPortName		= calloc( MAXSTRINGSIZE, 1 );
	localProperty.newPortDir		= calloc( MAXSTRINGSIZE, 1 );
	localProperty.newPortName		= calloc( MAXSTRINGSIZE, 1 );
	localProperty.options			= calloc( MAXBUFFERSIZE, 1 );
	localProperty.optionsCheck		= calloc( MAXSTRINGSIZE, 1 );
	localProperty.optionsDir		= calloc( MAXSTRINGSIZE, 1 );
	localProperty.startPortCmd		= calloc( MAXSTRINGSIZE, 1 );
	localProperty.startPortDir		= calloc( MAXSTRINGSIZE, 1 );
	localProperty.stopPortCmd		= calloc( MAXSTRINGSIZE, 1 );
	localProperty.stopPortDir		= calloc( MAXSTRINGSIZE, 1 );
	localProperty.workDir			= calloc( MAXSTRINGSIZE, 1 );
	localProperty.workFullPath		= calloc( MAXSTRINGSIZE, 1 );

	property->availableDependenciesDb	= MGdbOpen( property->availableDependenciesDbFileName );
	property->availablePortsDb		= MGdbOpen( property->availablePortsDbFileName );
	property->commandLineDb			= MGdbOpen( property->commandLineDbFileName );
	property->configDb			= MGdbOpen( property->configDbFileName );
	property->ignoreDb			= MGdbOpen( property->ignoreDbFileName );
	property->installedPortsDb		= MGdbOpen( property->installedPortsDbFileName );
	property->strikesDb			= MGdbOpen( property->strikesDbFileName );

	if( ( strcmp( "/sysutils/portmanager", oldPortDir ) == 0 || strcmp( "/local/sysutils/portmanager", oldPortDir ) == 0 ) && property->forced )
	{
		fprintf( stdout, "ignoring portmanager, will not self update in forced mode by design\n" );
		while( fflush( stdout ) );
		MGPMlogAdd( property, "ignoring portmanager ", "will not self update in forced mode by design",
								oldPortDir, "added to ignore.db", " " );

		MGdbAdd( property->ignoreDb, oldPortDir, "skipping portmanager, will not self update in forced mode by design\n", NULL );
		rCleanUp( property, &localProperty );
		return( 0 );
	}

	/*
	 * setup environment
	 */
	MGmStrcpy( localProperty.environment[0], "PATH=/sbin:/bin:/usr/sbin:/usr/bin:/usr/local/bin:/usr/X11R6/bin" );

	if( getenv( "CCACHE_DIR" ) != NULL )
	{
		MGmStrcpy( localProperty.environment[1], "CCACHE_DIR=" );
		MGmStrcat( localProperty.environment[1], getenv( "CCACHE_DIR" ) );
	}
	else
	{
		localProperty.environment[1][0]	= 0;
	}

	if( getenv( "CCACHE_PATH" ) != NULL )
	{
		MGmStrcpy( localProperty.environment[2], "CCACHE_PATH=/usr/bin:/usr/local/bin" );
	}
	else
	{
		localProperty.environment[2][0]	= 0;
	}

	/* 
	 * test for bsd.ports.mk patch
	 */
	if( property->bsdPortMkPatchedBlock == 0 )
	{
		property->bsdPortMkPatched	= rBsdPortMkPatch( property, &localProperty );
	}

	/*
	 * store localProperty.CURDIR
	 */
	MGmStrcpy( localProperty.CURDIR, "cd " );
	MGmStrcat( localProperty.CURDIR, PORTSDIR );
	MGmStrcat( localProperty.CURDIR, oldPortDir );
	MGmStrcat( localProperty.CURDIR, " && make -V .CURDIR" );
	localProperty.buffer[0]	= 0;
	pHandle		= popen( localProperty.CURDIR, "r" );
	fread( localProperty.buffer, MAXBUFFERSIZE, 1, pHandle );
	pclose( pHandle );
	idx	= 0;
	while( idx < MAXSTRINGSIZE )
	{
		if( localProperty.buffer[idx] == LINEFEED || localProperty.buffer[idx] == TAB || localProperty.buffer[idx] == SPACE )
		{
			localProperty.buffer[idx]	= 0;
			break;
		}
		idx++;
	}
	MGmStrcpy( localProperty.CURDIR, localProperty.buffer );


	/* 
	 * store localProperty.workDir
	 */
	MGmStrcpy( localProperty.workDir, "cd " );
	MGmStrcat( localProperty.workDir, PORTSDIR );
	MGmStrcat( localProperty.workDir, oldPortDir );
	MGmStrcat( localProperty.workDir, "; make -V WRKDIRPREFIX" );
	localProperty.buffer[0]	= 0;
 	pHandle		= popen( localProperty.workDir, "r" );
	fread( localProperty.buffer, MAXSTRINGSIZE, 1, pHandle );
	pclose( pHandle );
	idx	= 0;
	while( idx < MAXSTRINGSIZE )
	{
		if( localProperty.buffer[idx] == LINEFEED || localProperty.buffer[idx] == TAB || localProperty.buffer[idx] == SPACE )
		{
			localProperty.buffer[idx]	= 0;
			break;
		}
		idx++;
	}
	MGmStrcpy( localProperty.workDir, localProperty.buffer );

	/* 
	 * build localProperty.workFullPath
	 */
	MGmStrcpy( localProperty.workFullPath, localProperty.workDir );
	MGmStrcat( localProperty.workFullPath, localProperty.CURDIR );
	MGmStrcat( localProperty.workFullPath, "/work" );

	/* 
	 * store localProperty.optionsDir
	 */
	MGmStrcpy( localProperty.optionsDir, "cd " );
	MGmStrcat( localProperty.optionsDir, PORTSDIR );
	MGmStrcat( localProperty.optionsDir, oldPortDir );
	MGmStrcat( localProperty.optionsDir, "; make -V OPTIONSFILE" );
	localProperty.buffer[0]	= 0;
 	pHandle		= popen( localProperty.optionsDir, "r" );
	fread( localProperty.buffer, MAXSTRINGSIZE, 1, pHandle );
	pclose( pHandle );
	idx	= 0;
	while( idx < MAXSTRINGSIZE )
	{
		if( localProperty.buffer[idx] == LINEFEED || localProperty.buffer[idx] == TAB || localProperty.buffer[idx] == SPACE )
		{
			localProperty.buffer[idx]	= 0;
			break;
		}
		idx++;
	}
	MGmStrcpy( localProperty.optionsDir, localProperty.buffer );

	/*
	 * BEFORE check for /var/db/ports/{port dir name}/options
	 * get size/time of file for later comparison
	 *
	 */
	
	if( MGrIfFileExist( localProperty.optionsDir ) )
	{
		MGmStrcpy( localProperty.beforeOptionsFileTime, MGrIntToString( (int)MGrFileTime( localProperty.optionsDir ) ) );
		MGmStrcpy( localProperty.beforeOptionsFileSize, MGrIntToString( MGrFileSize( localProperty.optionsDir ) ) );	
	}
	else
	{
		localProperty.beforeOptionsFileTime[0]	= 0;
		localProperty.beforeOptionsFileSize[0]	= 0;
	}

	/*
	 * gather stop/start instructions if they exist
	 */
	MGdbGoTop( property->configDb );
	while( MGdbSeek( property->configDb,
			property->fieldConfigDbKey,
			STOP,
			property->fieldConfigDbKey,
			exact )	)
	{
		stopPortDirPtr	= MGdbGet( property->configDb,  	
					MGdbGetRecno( property->configDb ) - 1,
					property->fieldConfigDbValue );

		if( strncmp( oldPortDir, MGdbGet( property->configDb,
						MGdbGetRecno( property->configDb ) - 1,
						property->fieldConfigDbValue ),
						MGrStrlen( oldPortDir ) ) )
		{
			continue;
		}

		if( stopPortDirPtr[MGrStrlen( oldPortDir )] == SPACE )
		{
			strncpy( localProperty.stopPortDir, stopPortDirPtr, MGrStrlen( oldPortDir ) );
			MGmStrcpy( localProperty.stopPortCmd, stopPortDirPtr + MGrStrlen( oldPortDir ) + 1 );
			break;
		}
		else
		{
			break;
		}
	}					
	MGdbGoTop( property->configDb );
	while( MGdbSeek( property->configDb,
			property->fieldConfigDbKey,
			START,
			property->fieldConfigDbKey,
			exact ) )

	{
		startPortDirPtr	= MGdbGet( property->configDb,  	
					MGdbGetRecno( property->configDb ) - 1,
					property->fieldConfigDbValue );

		if( strncmp( oldPortDir, MGdbGet( property->configDb,
					MGdbGetRecno( property->configDb ) - 1,
					property->fieldConfigDbValue ),
					MGrStrlen( oldPortDir ) ) )
		{
			continue;
		}

		if( startPortDirPtr[MGrStrlen( oldPortDir )] == SPACE )
		{
			strncpy( localProperty.startPortDir, startPortDirPtr, MGrStrlen( oldPortDir ) );
			MGmStrcpy( localProperty.startPortCmd, startPortDirPtr + MGrStrlen( oldPortDir ) + 1 );
		}
		else
		{
			break;
		}
	}

	/*
	 * fill options if any
	 */
	MGdbGoTop( property->configDb );
	if( MGdbSeek( property->configDb, property->fieldConfigDbKey, oldPortDir, property->fieldConfigDbValue, exact ) )
	{ 
		MGmStrcpy( localProperty.options, MGdbGet( property->configDb,
							MGdbGetRecno( property->configDb ) - 1,
							property->fieldConfigDbValue ) );
	}
	else
	{
		localProperty.options[0]	= 0;
	}

	/*
	 *	Do update in interactive mode (command line switch -ui )
	 */
	if( property->interactive )
	{
		fprintf( stdout, "ok to update/rebuild %s %s (yes/no/auto yes to all) [y/n/a] [y]?\n",  oldPortDir, oldPortName );
		while( fflush( stdout ) );
		answer	= getc(stdin);

		switch(answer)
		{
			case KEY_ENTER:
			{
				break;
			}
			case KEY_A:
			{
				answer	= getc(stdin);
				property->interactive	= 0;
				break;
			}
			case KEY_a:
			{
				property->interactive	= 0;
				answer	= getc(stdin);
				break;
			}
			case KEY_Y:
			{
				answer	= getc(stdin);
				break;
			}
			case KEY_y:
			{
				answer	= getc(stdin);
				break;
			}
			case KEY_N:
			{
				fprintf( stdout, "\nAdding  %s %s to ignore list\n", oldPortName, oldPortDir );
				while( fflush( stdout ) );
				MGdbAdd( property->ignoreDb, oldPortDir, "inteactivly skipped", NULL );
				answer	= getc(stdin);
				rCleanUp( property, &localProperty );
				return(0);
			}
			case KEY_n:
			{
				fprintf( stdout, "\nAdding  %s %s to ignore list\n", oldPortName, oldPortDir );
				while( fflush( stdout ) );
				MGdbAdd( property->ignoreDb, oldPortDir, "inteactivly skipped", NULL );
				answer	= getc(stdin);
				rCleanUp( property, &localProperty );
				return(0);
			}
			default:
			{
				fprintf( stdout, "Invalid option. Please enter either Y to upgrade or N to skip\n");
				while( fflush( stdout ) );
				answer	= getc(stdin);
			}
		}
	}

	/************************************************************************/
	/*			Command "1" " make -V OPTIONS"		*/
	/************************************************************************/

	MGmStrcpy( localProperty.batchCheck, "cd " );
	MGmStrcat( localProperty.batchCheck, PORTSDIR );
	MGmStrcat( localProperty.batchCheck, oldPortDir );
	MGmStrcat( localProperty.batchCheck, " && make -V BATCH" );
	
 	pHandle		= popen( localProperty.batchCheck, "r" );
	localProperty.buffer[0]	= 0;
	fread( localProperty.buffer, MAXBUFFERSIZE, 1, pHandle );
	pclose( pHandle );
	idx	= 0;
	while( idx < MAXBUFFERSIZE )
	{
		if( localProperty.buffer[idx]	== LINEFEED )
		{
			localProperty.buffer[idx]	= 0;
			break;
		}
		idx++;
	}
	if( MGrStrlen( localProperty.buffer ) > 0 )	/* BATCH is defined */
	{
		fprintf( stdout, "%s\n", SINGLE_LINES );
		fprintf( stdout, "%s %s:\n\t commands 1 & 2 of 14 OPTIONS check and \"make config\" skipped because BATCH is defined\n",
			id,  PACKAGE_VERSION );
		fprintf( stdout, "\t recommend not defining BATCH so options may at least be set to their defaults\n" );
		while( fflush( stdout ) );
	}
	else
	{
		/*
		 * If no /var/db/ports/{port dir name}/options file exists
		 * yet make -V OPTIONS shows OPTIONS as being defined
		 * make is going to bluescreen, we need to handle that here
		 */

		MGmStrcpy( localProperty.optionsCheck, "cd " );
		MGmStrcat( localProperty.optionsCheck, PORTSDIR );
		MGmStrcat( localProperty.optionsCheck, oldPortDir );
		MGmStrcat( localProperty.optionsCheck, " && make -V OPTIONS" );

		fprintf( stdout, "%s\n", SINGLE_LINES );
		fprintf( stdout, "%s %s command: #1 of 14  %s\n", id, PACKAGE_VERSION, localProperty.optionsCheck );
		fprintf( stdout, "%s\n", SINGLE_LINES );
		while( fflush( stdout ) );

 		pHandle		= popen( localProperty.optionsCheck, "r" );
		localProperty.buffer[0]	= 0;
		fread( localProperty.buffer, MAXBUFFERSIZE, 1, pHandle );
		pclose( pHandle );
		idx	= 0;
		while( idx < MAXBUFFERSIZE )
		{
			if( localProperty.buffer[idx] == LINEFEED )
			{
				localProperty.buffer[idx] = 0;
				break;
			}
			idx++;
		}
		/************************************************************************/
		/*			Command "2" " make conf"				*/
		/************************************************************************/
		if( MGrBufferlen( localProperty.buffer, MAXBUFFERSIZE ) > 4 )
		{
/*
fprintf( stderr, "%s DEBUG: MGrStrlen( localProperty.buffer )-=>%d localProperty.buffer-=>%s localProperty.optionsDir->%s\n", id,
	MGrStrlen( localProperty.buffer ), localProperty.buffer );
while( fflush( stderr ) );
*/
			/*
			 * If here we know OPTIONS is defined, need to run make config
			 * if /var/db/ports/{package name}/options exists
			 */
			if( MGrIfFileExist( localProperty.optionsDir ) == 0 )
			{
				MGmStrcpy( localProperty.command, "cd " );
				MGmStrcat( localProperty.command, PORTSDIR );
				MGmStrcat( localProperty.command, oldPortDir );
				MGmStrcat( localProperty.command, " && make config " );
				if( MGrStrlen( localProperty.options ) > 0 )
				{
					MGmStrcat( localProperty.command, localProperty.options );
				}

				fprintf( stdout, "%s\n", SINGLE_LINES );
				fprintf( stdout, "update %s \n", oldPortName );
				fprintf( stdout, "%s %s command: #2 of 14  %s\n", id, PACKAGE_VERSION, localProperty.command );
				fprintf( stdout, "%s\n", SINGLE_LINES );
				while( fflush( stdout ) );
				system(localProperty.command);
			}
			MGmStrcpy( localProperty.afterOptionsFileTime,
					MGrIntToString( (int)MGrFileTime( localProperty.optionsDir ) ) );
			MGmStrcpy( localProperty.afterOptionsFileSize,
					MGrIntToString( MGrFileSize( localProperty.optionsDir ) ) );	

			/*
			 * test to see if any options changed
			 */
			if( MGrStrlen( localProperty.beforeOptionsFileTime ) !=  MGrStrlen( localProperty.afterOptionsFileTime ) 	||
				MGrStrlen( localProperty.beforeOptionsFileSize ) !=  MGrStrlen( localProperty.afterOptionsFileSize )	||
				strcmp( localProperty.beforeOptionsFileTime, localProperty.afterOptionsFileTime ) 		||
				strcmp( localProperty.beforeOptionsFileSize, localProperty.afterOptionsFileSize ) )
			{
				/*
				 * options changed so need to purge oldPortDir's records
				 * from availablePortsDb and availableDependenciesDb
				 */
				MGPMlogAdd( property, "options changed so returning ", oldPortName, oldPortDir, " to out of date pool ", " " );
				property->optionsChanged	= 1;
				MGdbGoTop( property->availablePortsDb );
				while( MGdbSeek( property->availablePortsDb,
						property->fieldAvailablePortsDbPortDir,
						oldPortDir,
						property->fieldAvailablePortsDbPortDir,
						exact ) )
				{
					fprintf( stdout, "%s info: purging availablePortsDb of old %s record\n",
						id,
						MGdbGet( property->availablePortsDb,
							MGdbGetRecno( property->availablePortsDb ) - 1,
							property->fieldAvailablePortsDbPortName ) );
					while( fflush( stdout ) );
					property->availablePortsDb	= MGdbDelete( property->availablePortsDb,
											MGdbGetRecno( property->availablePortsDb ) - 1 );
					MGdbGoTop( property->availablePortsDb );
				}

				MGdbGoTop( property->availableDependenciesDb );
				while( MGdbSeek( property->availableDependenciesDb,
						property->fieldAvailableDependenciesDbPortDir,
						oldPortDir,
						property->fieldAvailableDependenciesDbPortDir,
						exact ) )
				{
				fprintf( stdout, "%s info: purging availableDependenciesDb of old %s dependency records %s\n",
					id,
					MGdbGet( property->availableDependenciesDb,
						MGdbGetRecno( property->availableDependenciesDb ) - 1,
						property->fieldAvailableDependenciesDbPortName ),
					MGdbGet( property->availableDependenciesDb,
						MGdbGetRecno( property->availableDependenciesDb ) - 1,
						property->fieldAvailableDependenciesDbDependencyName ) );
					while( fflush( stdout ) );
					property->availableDependenciesDb = MGdbDelete( property->availableDependenciesDb, 
											MGdbGetRecno( property->availableDependenciesDb ) - 1 );
					MGdbGoTop( property->availableDependenciesDb );
				}
				/*
				 * add oldPortDir's new records back into availablePortsDb
				 * and availableDependenciesDb
				 */
				fprintf( stdout, "%s info: adding new/modified %s records\n", id, oldPortDir );
				while( fflush( stdout ) );
				MGPMrSinglePortCreateStatusDb( property, oldPortDir );
				rCleanUp( property, &localProperty );
				return( 0 );
			}
		}

	}

	/************************************************************************/
	/* 3 strikes check							*/
	/************************************************************************/
	MGdbGoTop( property->strikesDb );
	strikePtr		= MGdbSeek( property->strikesDb, property->fieldStrikesDbPortDir, oldPortDir, property->fieldStrikesIDX, exact );  
	if( strikePtr )
	{
		MGmStrcpy( strike, strikePtr );
		if( strike[0]	== '2' )
		{
			MGdbAdd( property->ignoreDb, oldPortDir, "looping, 3rd attempt at make", NULL );
			if( property->log )
			{
				MGmStrcpy( localProperty.command, "echo \"make Strike 3 " );
				MGmStrcat( localProperty.command, oldPortName );
				MGmStrcat( localProperty.command, " " );
				MGmStrcat( localProperty.command, oldPortDir );
				MGmStrcat( localProperty.command, " marking port as ignore/looping " );
				MGmStrcat( localProperty.command, " \">> /var/log/portmanager.log" );
				system( localProperty.command );
			}
			rCleanUp( property, &localProperty );
			return( 0 );
		}
		if( strike[0]	== '1' )
		{
			fprintf( stdout, "%s %s WARNING: This is the second time %s is being built,\n", id, PACKAGE_VERSION, oldPortName );
			fprintf( stdout, "one more attempt and it will be marked as ignore.\n\n" );
			while( fflush( stdout ) );
			MGdbGoTop( property->strikesDb );
			strikePtr		= MGdbSeek( property->strikesDb,
							property->fieldStrikesDbPortDir,
							oldPortDir,
							property->fieldStrikesIDX,
							exact );
			property->strikesDb	= MGdbDelete( property->strikesDb, MGdbGetRecno( property->strikesDb ) - 1 );
			MGdbAdd( property->strikesDb, oldPortDir, "2", NULL );
			if( property->log )
			{
				MGmStrcpy( localProperty.command, "echo \"make Strike 2 " );
				MGmStrcat( localProperty.command, oldPortName );
				MGmStrcat( localProperty.command, " " );
				MGmStrcat( localProperty.command, oldPortDir );
				MGmStrcat( localProperty.command, " \">> /var/log/portmanager.log" );
				system( localProperty.command );
			}
/*
 * dualOriginCheck( property, oldPortName ); used to be here in old portmanager ver < 0.3.1, seems to be no longer
 * needed, keep this note just in case
 */
		}
	}
	else
	{
		MGdbAdd( property->strikesDb, oldPortDir, "1", NULL );
	}

	/************************************************************************/
	/*			Command "3" " make check-conflicts "		*/
	/************************************************************************/
	MGmStrcpy( localProperty.command, "cd " );
	MGmStrcat( localProperty.command, PORTSDIR );
	MGmStrcat( localProperty.command, oldPortDir );
	MGmStrcat( localProperty.command, " && make check-conflicts " );
	if( MGrStrlen( localProperty.options ) > 0 )
	{
		MGmStrcat( localProperty.command, localProperty.options );
	}

	fprintf( stdout, "%s\n", SINGLE_LINES );
	fprintf( stdout, "checking for conflicts before building %s\n", oldPortName );
	fprintf( stdout, "%s %s command: #3 of 14  %s\n", id, PACKAGE_VERSION, localProperty.command );
	fprintf( stdout, "%s\n", SINGLE_LINES );
	while( fflush( stdout ) );
	if( ( errorCode = system(localProperty.command) ) )
	{
		/*
		 * We have a conflicting port installed, pkg_delete -f'ing the stinker
		 *
		 */
		localProperty.buffer[0]	= 0;
		pHandle		= popen( localProperty.command, "r" );
		fread( localProperty.buffer, MAXBUFFERSIZE, 1, pHandle );
		if( feof( pHandle ) && !ferror( pHandle ) )
		{
			bufferBase	= strstr( localProperty.buffer, "conflicts with installed package" );
			if( bufferBase )
			{
				/*
				 *
				 * Sample of what to expect in the buffer
				 *
				 * ===>  linux_base-rh-7.3 conflicts with installed package(s):
				 * linux-XFree86-libs-4.3.99.902_2
				 * linux_base-rh-9
				 *
				 */
				bufferBase	= localProperty.buffer
						+ MGrStrlen(oldPortName)
						+ MGrStrlen( "===>  conflicts with installed package" );

				/*
				 *
				 * Now for the fun part, rather than trying to parse every possible
				 * port in this buffer, we'll just cycle through the install ports database
				 * and see if any of the names show up in this buffer somewhere, they show up
				 * then they are conflicting ports
				 *
				 */
				installedPortsDbIDX	= 0;
				installedPortsDbQTY	= MGdbGetRecordQty( property->installedPortsDb );
				while( installedPortsDbIDX < installedPortsDbQTY )
				{
					MGmStrcpy( localProperty.installedPortName, MGdbGet( property->installedPortsDb,
										installedPortsDbIDX,
										property->fieldInstalledPortsDbPortName ) );
					stinker = strstr( bufferBase, localProperty.installedPortName );
					if( stinker )
					{
						fprintf( stdout, "OK to remove conflicting port %s Yy/Nn/Aa [Y]\?\n", localProperty.installedPortName );
						fprintf( stdout, "Will timeout after 5 minutes and default to [Y] if no selection is made\n" );
						while( fflush( stdout ) );
						if( property->autoConflicts == 0 )
						{
							/*
							 * set timeout default to Y
							 */
							alarm( property->timeout );
							signal( SIGALRM, MGPMrTimer );
							property->STDIN	= fopen( "/dev/stdin", "r" );	/* Flawfinder: ignore */
							property->timeOut = 0;
							doubleBreak	= 0;
							while( 1 == 1 )
							{
								answer	= getc( property->STDIN );
								fclose( property->STDIN );
								switch(answer)
								{
									case KEY_ENTER:
									{
										doubleBreak	= 1;
										break;
									}
									case KEY_A:
									{
										doubleBreak	= 1;
										property->autoConflicts	= 1;
										break;
									}
									case KEY_a:
									{
										doubleBreak	= 1;
										property->autoConflicts	= 1;
										break;
									}
									case KEY_Y:
									{
										doubleBreak	= 1;
										break;
									}
									case KEY_y:
									{
										doubleBreak	= 1;
										break;
									}
									case KEY_N:
									{
										fprintf( stdout,
											"\nAdding  %s %s conflicts with installed port %s added to ignore list\n",
											oldPortName, oldPortDir, localProperty.installedPortName );
										while( fflush( stdout ) );
										MGPMlogAdd( property,
											oldPortName,
											oldPortDir,
											"conflicts with installed port ",
											localProperty.installedPortName,
											" added to ignore.db" );
										MGdbAdd( property->ignoreDb, oldPortDir, "conflicts with another port", NULL );
										rCleanUp( property, &localProperty );
										return(0);
									}
									case KEY_n:
									{
										fprintf( stdout,
											"\nAdding  %s %s conflicts with installed port %s added to ignore list\n",
											oldPortName, oldPortDir, localProperty.installedPortName );
										while( fflush( stdout ) );
										MGPMlogAdd( property,
											oldPortName,
											oldPortDir,
											"conflicts with installed port ",
											localProperty.installedPortName,
											" added to ignore.db" );
										MGdbAdd( property->ignoreDb, oldPortDir, "conflicts with another port", NULL );
										rCleanUp( property, &localProperty );
										return(0);
									}
									default:
									{
										if( property->timeOut )
										{
											fprintf( stdout, "5 minute timeout, defaulting to [Y]\n");
											while( fflush( stdout ) );
											doubleBreak	= 1;
											break;
										}
										else
										{
											property->STDIN	= fopen( "/dev/stdin", "r" );	/* Flawfinder: ignore */
											fprintf( stdout, "Invalid option. Please enter either Y to upgrade or N\n");
											while( fflush( stdout ) );
										}
									}
									
								}
								if( doubleBreak )
								{
									break;
								}
							}
						}

						fprintf( stdout, "%s\n", SINGLE_LINES );
						while( fflush( stdout ) );
						if( property->timeOut  == 1 )
						{
							/*
							 * reset alarm, reopen STDIN after timeout
							 */
							alarm(0);
							property->timeOut	= 0;

							MGPMlogAdd( property, "conflicting installed ", localProperty.installedPortName,
								" removed from system after 5 minute timeout: conflicted with ", oldPortName, " " );
						}
						else
						{
							MGPMlogAdd( property, "conflicting installed ", localProperty.installedPortName,
								" removed from system: conflicted with ", oldPortName, " " );
						}

						MGmStrcpy( localProperty.command, "pkg_delete -f " );
						MGmStrcat( localProperty.command, localProperty.installedPortName );		
						fprintf( stdout, "executing: %s\n", localProperty.command );
						while( fflush( stdout ) );
						errorCode	= system( localProperty.command );
						if( property->log )
						{
							MGmStrcpy( localProperty.command, "echo \"removed conflicting port " );
							MGmStrcat( localProperty.command, localProperty.installedPortName );
							MGmStrcat( localProperty.command, " " );
							MGmStrcat( localProperty.command, " \">> /var/log/portmanager.log" );
							system( localProperty.command );
						}
					}
					installedPortsDbIDX++;
				}
			}
			else
			{
				/*
				 * the make check-conflicts localProperty.command worked but there is no
				 * "conflicts with installed package" in the buffer
				 * let this pass to make and see what happens....
				 */
				errorCode	= 0;
				fprintf( stderr, "no \"conflicts with installed package\" in buffer" );
				while( fflush( stderr ) );
			}
		} 
		else
		{
			/*
			 * the make check-conflicts localProperty.command failed for some reason so well just
			 * let this pass to make and see what happens....
			 */
			errorCode	= 0;
			fprintf( stderr, "check-conflicts command failed\n" );
			fprintf( stderr, " feof( pHandle )-=>%d ferror( pHandle )-=>%d\n", feof( pHandle ), ferror( pHandle ) );
			while( fflush( stderr ) );
		}
		pclose( pHandle );
	}
	if(errorCode)
	{

		fprintf( stderr, "%s %s error: %s conflicts with another installed port\n", id, PACKAGE_VERSION, oldPortName );
		fprintf( stderr, "you will have to remove the conflicting port before upgrading %s\n", oldPortName );
		fprintf( stderr, "marking %s as ignored\n", oldPortName );
		fprintf( stdout, "%s\n", DOUBLE_LINES );
		while( fflush( stderr ) );
		if( property->log )
		{
			MGmStrcpy( localProperty.command, "echo \"      " );
			MGmStrcat( localProperty.command, oldPortName );
			MGmStrcat( localProperty.command, " " );
			MGmStrcat( localProperty.command, oldPortDir );
			MGmStrcat( localProperty.command, " conflicts with another installed port so marked ignore\" >> /var/log/portmanager.log" );
			system( localProperty.command );
		}
		MGdbAdd( property->ignoreDb, oldPortDir, "conflicts with another installed port", NULL );
		rCleanUp( property, &localProperty );
		return( 0 );
	}
	/************************************************************************/
	/*			Command "4" " make -V FORBIDDEN "		*/
	/************************************************************************/
	if( ( errorCode = MGPMrPortBrokeCheck( property, localProperty.options, oldPortDir, oldPortName, FORBIDDEN  ) ) )
	{
		switch( errorCode )
		{
			case	CRASH:
			{
				rCleanUp( property, &localProperty );
				return( CRASH );
			}
			case	SKIP:
			{
				rCleanUp( property, &localProperty );
				return( OK );
			}
		}
			
	}

	/************************************************************************/
	/*			Command "5" " make -V BROKEN "		*/
	/************************************************************************/
	if( ( errorCode = MGPMrPortBrokeCheck( property, localProperty.options, oldPortDir, oldPortName, BROKEN  ) ) )
	{
		switch( errorCode )
		{
			case	CRASH:
			{
				rCleanUp( property, &localProperty );
				return( CRASH );
			}
			case	SKIP:
			{
				rCleanUp( property, &localProperty );
				return( OK );
			}
		}
			
	}

	/************************************************************************/
	/*			Command "6" " make -V IGNORE "		*/
	/************************************************************************/
	if( ( errorCode = MGPMrPortBrokeCheck( property, localProperty.options, oldPortDir, oldPortName, IGNORE  ) ) )
	{
		switch( errorCode )
		{
			case	CRASH:
			{
				rCleanUp( property, &localProperty );
				return( CRASH );
			}
			case	SKIP:
			{
				rCleanUp( property, &localProperty );
				return( OK );
			}
		}
			
	}

	/************************************************************************/
	/*			Command "7" " make clean "			*/
	/************************************************************************/
	fprintf( stdout, "%s\n", SINGLE_LINES );
	fprintf( stdout, "intitial clean of work directories \n" );
	fprintf( stdout, "%s %s command: #7 of 14:  \n", id, PACKAGE_VERSION );
	fprintf( stdout, "%s\n", SINGLE_LINES );
	while( fflush( stdout ) );

	availableDependenciesDbQTY	= MGdbGetRecordQty( property->availableDependenciesDb );
	MGdbGoTop( property->availableDependenciesDb );
	oldPortDirPtr	= MGdbSeek( property->availableDependenciesDb,
				property->fieldAvailableDependenciesDbPortDir,
				oldPortDir,
				property->fieldAvailableDependenciesDbPortDir,
				exact );

	availableDependenciesDbIDX	= MGdbGetRecno( property->availableDependenciesDb )-1;

	availableDependenciesDbDependencyDirPtr		= MGdbGet( property->availableDependenciesDb,
								availableDependenciesDbIDX,
								property->fieldAvailableDependenciesDbDependencyDir );

	availableDependenciesDbDependencyNamePtr	= MGdbGet( property->availableDependenciesDb,
								availableDependenciesDbIDX,
								property->fieldAvailableDependenciesDbDependencyName );

	while( oldPortDirPtr && availableDependenciesDbIDX < availableDependenciesDbQTY && strcmp( oldPortDir, oldPortDirPtr ) == 0 )
	{
		rCleanDir( availableDependenciesDbDependencyDirPtr, localProperty.workDir );

		oldPortDirPtr					= MGdbGet( property->availableDependenciesDb,
									availableDependenciesDbIDX,
									property->fieldAvailableDependenciesDbPortDir );
		availableDependenciesDbDependencyDirPtr		= MGdbGet( property->availableDependenciesDb,
									availableDependenciesDbIDX,
									property->fieldAvailableDependenciesDbDependencyDir );
		availableDependenciesDbDependencyNamePtr	= MGdbGet( property->availableDependenciesDb,
									availableDependenciesDbIDX,
									property->fieldAvailableDependenciesDbDependencyName );
		availableDependenciesDbIDX++;
	}
	rCleanDir( oldPortDir, localProperty.workDir );

	/************************************************************************/
	/*			Command "8" make fetch				*/
	/************************************************************************/
	MGmStrcpy( localProperty.command, "cd " );
	MGmStrcat( localProperty.command, PORTSDIR );
	MGmStrcat( localProperty.command, oldPortDir );
	MGmStrcat( localProperty.command, " && make fetch " );
	if( MGrStrlen( localProperty.options ) > 0 )
	{
		MGmStrcat( localProperty.command, localProperty.options );
	}
	fprintf( stdout, "%s\n", SINGLE_LINES );
	fprintf( stdout, "fetch %s \n", oldPortName );
	fprintf( stdout, "%s %s command: #8 of 14  %s\n", id, PACKAGE_VERSION, localProperty.command );
	fprintf( stdout, "%s\n", SINGLE_LINES );
	while( fflush( stdout ) );
	if( ( errorCode = system(localProperty.command) ) )
	{
		if( property->log )
		{
			MGmStrcpy( localProperty.command, "echo \"      " );
			MGmStrcat( localProperty.command, oldPortName );
			MGmStrcat( localProperty.command, " " );
			MGmStrcat( localProperty.command, oldPortDir );
			MGmStrcat( localProperty.command, " failed to fetch\" >> /var/log/portmanager.log" );
			system( localProperty.command );
		}
		fprintf( stdout, "%s %s error: make fetch returned an error, adding %s to ignore.db\n", id, PACKAGE_VERSION, oldPortDir );
		while( fflush( stdout ) );
		MGdbAdd( property->ignoreDb, oldPortDir, "failed during make fetch", NULL );
		rCleanUp( property, &localProperty );
		return(0);
	}

	/************************************************************************/
	/*			Command "9" " make"				*/
	/************************************************************************/
	if( MGrStrlen( localProperty.options ) > 0 )
	{
		cmd	= malloc( sizeof( char** ) * 3 );
		cmd[0]	= malloc( MAXSTRINGSIZE );
		cmd[1]	= malloc( MAXBUFFERSIZE );
		cmd[2]	= 0;
		MGmStrcpy( cmd[0], PORTSDIR );
		MGmStrcat( cmd[0], oldPortDir );
		if( chdir( cmd[0] ) != 0 )
		{
			fprintf( stderr, "%s error: chdir %s failed\n", id, cmd[0] ); 
			perror( "chroot" );
			while( fflush( stderr ) );
			assert( 0 );
		}
		MGmStrcpy( cmd[0], "/usr/bin/make" );
		MGmBuffercpy( cmd[1], localProperty.options, MAXBUFFERSIZE );
		fprintf( stdout, "%s\n", SINGLE_LINES );
		fprintf( stdout, "update %s \n", oldPortName );
		fprintf( stdout, "%s %s command: #9 of 14  %s\nOPTIONS-=>%s\n", id, PACKAGE_VERSION, cmd[0], cmd[1] );
		fprintf( stdout, "%s\n", SINGLE_LINES );
		while( fflush( stdout ) );
		MGmSystem( cmd, localProperty.environment );	
		free( cmd[0] );
		free( cmd[1] );
		free( cmd );
	}
	else
	{
		cmd	= malloc( sizeof( char** ) * 2 );
		cmd[0]	= malloc( MAXSTRINGSIZE );
		cmd[1]	= 0;
		MGmStrcpy( cmd[0], PORTSDIR );
		MGmStrcat( cmd[0], oldPortDir );
		if( chdir( cmd[0] ) != 0 )
		{
			fprintf( stderr, "%s error: chdir %s failed\n", id, cmd[0] ); 
			perror( "chroot" );
			while( fflush( stderr ) );
			assert( 0 );
		}
		MGmStrcpy( cmd[0], "/usr/bin/make" );
		fprintf( stdout, "%s\n", SINGLE_LINES );
		fprintf( stdout, "update %s \n", oldPortName );
		fprintf( stdout, "%s %s command: #9 of 14  %s\n", id, PACKAGE_VERSION, cmd[0] );
		fprintf( stdout, "%s\n", SINGLE_LINES );
		while( fflush( stdout ) );
		MGmSystem( cmd, localProperty.environment );	
		free( cmd[0] );
		free( cmd );
	}

	if( MGrIfDirEntry( localProperty.workFullPath, ".build_done" ) == 0 )
	{
		fprintf( stderr, "%s\n", SINGLE_LINES );
		fprintf( stderr, "%s error:  %s %s failed during make, adding to ignore.db\n", id, oldPortName, oldPortDir );
		fprintf( stderr, "%s\n", SINGLE_LINES );
		while( fflush( stderr ) );
		MGPMlogAdd( property, oldPortName, oldPortDir, "port failed to build during make, adding to ignore.db", " ",  " " );
		MGdbAdd( property->ignoreDb, oldPortDir, "port failed to build during make\n", NULL );
		rCleanUp( property, &localProperty );
		return(0);
	}

	/************************************************************************/
	/*			Command "10" pkg_create -b			*/
	/************************************************************************/
	if( strcmp( oldPortName, "not installed" ) == 0 )
	{
		fprintf( stdout, "%s\n", SINGLE_LINES );
		fprintf( stdout, "backing up installed %s before removing it \n", oldPortName );
		fprintf( stdout, "%s %s localProperty.command: #10 of 14 N/A - new port\n", id, PACKAGE_VERSION );
		fprintf( stdout, "%s\n", SINGLE_LINES );
		while( fflush( stdout ) );
	}
	else
	{
		MGmStrcpy( localProperty.command, "rm -f /tmp/*.db; cp " );
		MGmStrcat( localProperty.command, SHAREDIR );
		MGmStrcat( localProperty.command, "/*.db /tmp" );
		fprintf( stdout, "storing databases command:%s\n", localProperty.command );
		while( fflush( stdout ) );
		system( localProperty.command );

		MGmStrcpy( localProperty.command, "cd " );
		MGmStrcat( localProperty.command, TEMPDIR );
		MGmStrcat( localProperty.command, " && pkg_create -b " );
		MGmStrcat( localProperty.command, oldPortName );

		fprintf( stdout, "%s\n", SINGLE_LINES );
		fprintf( stdout, "backing up installed %s before removing it \n", oldPortName );
		fprintf( stdout, "%s %s command: #10 of 14 %s\n", id, PACKAGE_VERSION, localProperty.command );
		fprintf( stdout, "%s\n", SINGLE_LINES );
		while( fflush( stdout ) );
		system(localProperty.command);

		MGmStrcpy( localProperty.command, "cp /tmp/*.db " );
		MGmStrcat( localProperty.command, SHAREDIR );
		fprintf( stdout, "restoring databases localProperty.command:%s\n", localProperty.command );
		while( fflush( stdout ) );
		system( localProperty.command );
	}
	/************************************************************************/
	/*			Command "11" " make deinstall "			*/
	/************************************************************************/
	/*
	 * inforced mass update mode portmanager  CANNOT loose its databases
	 */
	if( ( strcmp( oldPortDir, "/sysutils/portmanager" ) == 0 || strcmp( oldPortDir, "/local/sysutils/portmanager" ) == 0 ) &&
		property->pmMode == SINGLE )
	{
		MGmStrcpy( localProperty.command, "cd " );
		MGmStrcat( localProperty.command, PORTSDIR );
		MGmStrcat( localProperty.command, "/sysutils/portmanager; " );
		MGmStrcat( localProperty.command, "make -V PKG_DBDIR" );
		pHandle	= popen( localProperty.command, "r" );
		localProperty.buffer[0]	= 0;
		fread( localProperty.buffer, MAXSTRINGSIZE, 1, pHandle );
		pclose( pHandle );
		idx	= 0;
		while( idx < MAXSTRINGSIZE )
		{
			if( localProperty.buffer[idx] == LINEFEED || localProperty.buffer[idx] == TAB || localProperty.buffer[idx] == SPACE )
			{
				localProperty.buffer[idx]	= 0;
				break;
			}
			idx++;
		}
		MGmStrcpy( localProperty.command, "rm -r " );
		MGmStrcat( localProperty.command, localProperty.buffer );
		MGmStrcat( localProperty.command, "/portmanager*" );
		fprintf( stdout, "executing: %s\n", localProperty.command );
		while( fflush( stdout ) );
		system( localProperty.command );
	}
	else
	{
		/*
		 * have to do this here before db's are wacked
		 */
		if( ( strcmp( oldPortDir, "/sysutils/portmanager" ) == 0 || strcmp( oldPortDir, "/local/sysutils/portmanager" ) == 0 ) &&
			property->pmMode == MULTI )
		{
			/*
			 * have to do this here before db's are wacked
			 */
			MGmStrcpy( localProperty.commandRestart, "portmanager -u" );
			MGdbGoTop( property->commandLineDb );
			while( ( ipPtr = MGdbSeek( property->commandLineDb, property->fieldCommandLineDbKey, ip,  property->fieldCommandLineDbValue, exact ) ) )
			{
				MGmStrcat( localProperty.commandRestart, " " );
				MGmStrcat( localProperty.commandRestart, " -ip " );
				MGmStrcat( localProperty.commandRestart, ipPtr );
			}
		}
		if( strcmp( oldPortName, "not installed" ) == 0 )
		{
			fprintf( stdout, "%s\n", SINGLE_LINES );
			fprintf( stdout, "removing old %s \n", oldPortName );
			fprintf( stdout, "%s %s command: #11 of 14 N/A new port\n", id, PACKAGE_VERSION );
			fprintf( stdout, "%s\n", SINGLE_LINES );
			while( fflush( stdout ) );
		}
		else
		{
			/*
			 * use localProperty.stopPortCmd if it is defined
			 */
			if( strcmp( oldPortDir, localProperty.stopPortDir ) == 0 )
			{
				fprintf( stdout, "%s %s info: running %s stop localProperty.command %s\n",
					id,
					PACKAGE_VERSION,
					localProperty.stopPortDir,
					localProperty.stopPortCmd );
				while( fflush( stdout ) );

				localProperty.buffer[0]	= 0;
 				pHandle	= popen( localProperty.stopPortCmd, "r" );
				fread( localProperty.buffer, MAXBUFFERSIZE, 1, pHandle );
				pclose( pHandle );

				MGPMlogAdd( property, "stopping ", oldPortName, oldPortDir, " execution by running ", localProperty.stopPortCmd );

				fprintf( stdout, "%s reply: %s\n", localProperty.stopPortCmd, localProperty.buffer );
				while( fflush( stdout ) );
			}

			/*
			 * remove old port
			 */
			MGmStrcpy( localProperty.command, "pkg_delete -f ");
			MGmStrcat( localProperty.command, oldPortName);
			fprintf( stdout, "%s\n", SINGLE_LINES );
			fprintf( stdout, "removing old %s \n", oldPortName );
			fprintf( stdout, "%s %s command: #11 of 14 %s\n", id, PACKAGE_VERSION, localProperty.command );
			fprintf( stdout, "%s\n", SINGLE_LINES );
			while( fflush( stdout ) );
			if( ( system(localProperty.command) ) )
			{
				localProperty.command[0]	= 0;
				MGmStrcpy( localProperty.command, "cd " );
				MGmStrcat( localProperty.command, PORTSDIR );
				MGmStrcat( localProperty.command, oldPortDir );
				MGmStrcat( localProperty.command, " && make deinstall " );

				fprintf( stdout, "%s\n", SINGLE_LINES );
				fprintf( stdout, "trying one more way to remove %s...\n", oldPortName );
				fprintf( stdout, "%s %s command: #11 of 14  %s\n", id, PACKAGE_VERSION, localProperty.command );
				fprintf( stdout, "%s\n", SINGLE_LINES );
				while( fflush( stdout ) );
				system(localProperty.command);
			}
		}
	}
	/************************************************************************/
	/*			Command "12" make reinstall			*/
	/************************************************************************/
	fprintf( stdout, "%s\n", SINGLE_LINES );
	while( fflush( stdout ) );
	/*
	 * Store databases so a second instance of portmanager may be run
	 */
	MGmStrcpy( localProperty.command, "rm -f /tmp/*.db; cp " );
	MGmStrcat( localProperty.command, SHAREDIR );
	MGmStrcat( localProperty.command, "/*.db /tmp" );
	fprintf( stdout, "storing databases command:%s\n", localProperty.command );
	while( fflush( stdout ) );
	system( localProperty.command );

	MGmStrcpy( localProperty.command, "cd " );
	MGmStrcat( localProperty.command, PORTSDIR );
	MGmStrcat( localProperty.command, oldPortDir );
	MGmStrcat( localProperty.command, " && make reinstall " );
	if( MGrStrlen( localProperty.options ) > 0 )
	{
		MGmStrcat( localProperty.command, localProperty.options );
	}

	fprintf( stdout, "installing updated %s \n", oldPortDir );
	fprintf( stdout, "%s %s command: #12 of 14 %s\n", id, PACKAGE_VERSION, localProperty.command );
	fprintf( stdout, "%s\n", SINGLE_LINES );
	while( fflush( stdout ) );
	if( ( errorCode = system( localProperty.command ) ) )
	{
		if( property->log )
		{
			MGmStrcpy( localProperty.command, "echo \"      " );
			MGmStrcat( localProperty.command, oldPortName );
			MGmStrcat( localProperty.command, " " );
			MGmStrcat( localProperty.command, oldPortDir );
			MGmStrcat( localProperty.command, " restoring original port from backup\" >> /var/log/portmanager.log" );
			system( localProperty.command );
		}


		MGmStrcpy(localProperty.command, "pkg_add ");
		MGmStrcat(localProperty.command, TEMPDIR);
		MGmStrcat(localProperty.command, "/");
		MGmStrcat(localProperty.command, oldPortName);
		MGmStrcat(localProperty.command, ".tgz");
		fprintf( stdout, "%s\n", SINGLE_LINES );
		fprintf( stderr, "restoring original port from backup \n" );
		fprintf( stderr, "%s %s command: #12 of 14!! ***Emergancy restore***  %s\n", id, PACKAGE_VERSION, localProperty.command );
		fprintf( stdout, "%s\n", SINGLE_LINES );
		while( fflush( stdout ) );
		if( ( errorCode = system( localProperty.command ) ) )
		{
			if( property->log )
			{
				MGmStrcpy( localProperty.command, "echo \"      " );
				MGmStrcat( localProperty.command, oldPortName );
				MGmStrcat( localProperty.command, " " );
				MGmStrcat( localProperty.command, oldPortDir );
				MGmStrcat( localProperty.command, " failed to restore from backup\" >> /var/log/portmanager.log" );
				system( localProperty.command );
			}
			fprintf( stdout, "%s %s error: FAILED TO RESTORE BACKUP COPY, really sorry ! \n", id, PACKAGE_VERSION );
			fprintf( stdout, "%s %s info: adding %s to ignore.db\n", id, PACKAGE_VERSION, oldPortDir );
			while( fflush( stdout ) );

			MGmStrcpy( localProperty.command, "cp /tmp/*.db " );
			MGmStrcat( localProperty.command, SHAREDIR );
			fprintf( stdout, "restoring databases localProperty.command:%s\n", localProperty.command );
			while( fflush( stdout ) );
			system( localProperty.command );

			MGdbAdd( property->ignoreDb, oldPortDir, "failed during (6) emergancy restore", NULL );
			rCleanUp( property, &localProperty );
			return(0);
		}
		if( property->log )
		{
			MGmStrcpy( localProperty.command, "echo \"      " );
			MGmStrcat( localProperty.command, oldPortName );
			MGmStrcat( localProperty.command, " " );
			MGmStrcat( localProperty.command, oldPortDir );
			MGmStrcat( localProperty.command, " restored from backup and marked ignore\" >> /var/log/portmanager.log" );
			system( localProperty.command );
		}

		MGmStrcpy( localProperty.command, "cp /tmp/*.db " );
		MGmStrcat( localProperty.command, SHAREDIR );
		fprintf( stdout, "restoring databases localProperty.command:%s\n", localProperty.command );
		while( fflush( stdout ) );
		system( localProperty.command );

		fprintf( stdout, "%s\n", SINGLE_LINES );
		fprintf( stdout, "%s %s error: RESTORED BACKUP COPY\n",	id, PACKAGE_VERSION );
		fprintf( stdout, "%s %s info: adding %s to ignore.db\n", id, PACKAGE_VERSION, oldPortDir );
		MGdbAdd( property->ignoreDb, oldPortDir, "performed (6) emergancy restore", NULL );
		fprintf( stdout, "%s\n", SINGLE_LINES );
		while( fflush( stdout ) );
		rCleanUp( property, &localProperty );
		return(0);
	}
	else
	{
		if( strcmp( oldPortName, "not installed" ) )
		{
			MGmStrcpy( localProperty.command, "cd " );
			MGmStrcat( localProperty.command, TEMPDIR);
			MGmStrcat( localProperty.command, "; rm -f ");
			MGmStrcat( localProperty.command, oldPortName);
			MGmStrcat( localProperty.command, ".tgz" );
			fprintf( stdout, "%s\n", SINGLE_LINES );
			fprintf( stderr, "deleting backup copy, installation of updated %s successful\n", oldPortDir );
			fprintf( stdout, "%s %s localProperty.command: #12 of 14 %s\n", id, PACKAGE_VERSION, localProperty.command );
			fprintf( stdout, "%s\n", SINGLE_LINES );
			while( fflush( stdout ) );
			system( localProperty.command );
		}
	}
	MGmStrcpy( localProperty.command, "cp /tmp/*.db " );
	MGmStrcat( localProperty.command, SHAREDIR );
	fprintf( stdout, "restoring databases localProperty.command:%s\n", localProperty.command );
	while( fflush( stdout ) );
	system( localProperty.command );

	/*
	 * use localProperty.startPortCmd if it is defined
	 */
	if( strcmp( oldPortDir, localProperty.startPortDir ) == 0 )
	{
		fprintf( stdout, "%s %s info: running %s start localProperty.command %s\n",
			id,
			PACKAGE_VERSION,
			localProperty.startPortDir,
			localProperty.startPortCmd );
		while( fflush( stdout ) );
		localProperty.buffer[0]	= 0;
 		pHandle	= popen( localProperty.startPortCmd, "r" );
		fread( localProperty.buffer, MAXBUFFERSIZE, 1, pHandle );
		pclose( pHandle );

		MGPMlogAdd( property, "starting ", oldPortName, oldPortDir, " execution by running ", localProperty.startPortCmd );

		fprintf( stdout, "%s reply: %s\n", localProperty.startPortCmd, localProperty.buffer );
		while( fflush( stdout ) );
	}

	/************************************************************************/
	/*			Command "13" make package			*/
	/************************************************************************/
	if( property->backUp )
	{
		MGmStrcpy( localProperty.command, "cd " );
		MGmStrcat( localProperty.command, PORTSDIR );
		MGmStrcat( localProperty.command, oldPortDir );
		MGmStrcat( localProperty.command, " && make package " );
		if( MGrStrlen( localProperty.options ) > 0 )
		{
			MGmStrcat( localProperty.command, localProperty.options );
		}

		fprintf( stdout, "%s\n", SINGLE_LINES );
		fprintf( stderr, "making package of updated %s \n", oldPortDir );
		fprintf( stdout, "%s %s command: #13 of 14 %s\n", id, PACKAGE_VERSION, localProperty.command );
		fprintf( stdout, "%s\n", SINGLE_LINES );
		while( fflush( stdout ) );
		system(localProperty.command);

		MGmStrcpy( localProperty.command, "cp /tmp/*.db " );
		MGmStrcat( localProperty.command, SHAREDIR );
		fprintf( stdout, "restoring databases command:%s\n", localProperty.command );
		while( fflush( stdout ) );
		system( localProperty.command );
	}
	else
	{
		fprintf( stdout, "%s\n", SINGLE_LINES );
		fprintf( stderr, "--back-up not selected skipping making package of updated %s \n", oldPortDir );
		fprintf( stdout, "%s\n", SINGLE_LINES );
		while( fflush( stdout ) );
	}

	/************************************************************************/
	/*			Command "14" make clean				*/
	/************************************************************************/
	fprintf( stdout, "%s\n", SINGLE_LINES );
	fprintf( stderr, "terminal clean of work directories\n" );
	fprintf( stdout, "%s %s localProperty.command: #14 of 14\n", id, PACKAGE_VERSION  );
	fprintf( stdout, "%s\n", SINGLE_LINES );
	while( fflush( stdout ) );

	/*
	 * if portmanager updated there won't be any databases, so need to clean the old fashoned way
	 */
	if( ( strcmp( oldPortDir, "/sysutils/portmanager" ) == 0 || strcmp( oldPortDir, "/local/sysutils/portmanager" ) == 0 ) &&
		property->pmMode == MULTI )
	{
		rCleanDir( oldPortDir, localProperty.workDir );
	}
	else
	{
		availableDependenciesDbQTY	= MGdbGetRecordQty( property->availableDependenciesDb );
		MGdbGoTop( property->availableDependenciesDb );
		oldPortDirPtr	= MGdbSeek( property->availableDependenciesDb,
					property->fieldAvailableDependenciesDbPortDir,
					oldPortDir,
					property->fieldAvailableDependenciesDbPortDir,
					exact );

		availableDependenciesDbIDX	= MGdbGetRecno( property->availableDependenciesDb )-1;

		availableDependenciesDbDependencyDirPtr		= MGdbGet( property->availableDependenciesDb,
									availableDependenciesDbIDX,
									property->fieldAvailableDependenciesDbDependencyDir );

		availableDependenciesDbDependencyNamePtr	= MGdbGet( property->availableDependenciesDb,
									availableDependenciesDbIDX,
									property->fieldAvailableDependenciesDbDependencyName );

		while( oldPortDirPtr && availableDependenciesDbIDX < availableDependenciesDbQTY && strcmp( oldPortDir, oldPortDirPtr ) == 0 )
		{
			rCleanDir( availableDependenciesDbDependencyDirPtr, localProperty.workDir );

			oldPortDirPtr					= MGdbGet( property->availableDependenciesDb,
										availableDependenciesDbIDX,
										property->fieldAvailableDependenciesDbPortDir );
			availableDependenciesDbDependencyDirPtr		= MGdbGet( property->availableDependenciesDb,
										availableDependenciesDbIDX,
										property->fieldAvailableDependenciesDbDependencyDir );
			availableDependenciesDbDependencyNamePtr	= MGdbGet( property->availableDependenciesDb,
										availableDependenciesDbIDX,
										property->fieldAvailableDependenciesDbDependencyName );
			availableDependenciesDbIDX++;
		}
		rCleanDir( oldPortDir, localProperty.workDir );
	}

	/*
	 * portmanager auto restart after updating itself
	 */
	if( ( strcmp( oldPortDir, "/sysutils/portmanager" ) == 0 || strcmp( oldPortDir, "/local/sysutils/portmanager" ) == 0 ) &&
		property->pmMode == MULTI )
	{
		/*
		 * reconstruct the localProperty.command line
		 */
		MGdbGoTop( property->commandLineDb );
		if( ( ipPtr = MGdbSeek( property->commandLineDb, property->fieldCommandLineDbKey, l,  property->fieldCommandLineDbValue, exact ) ) )
		{
			MGmStrcat( localProperty.commandRestart, " " );
			MGmStrcat( localProperty.commandRestart, "-l" );
		}
		MGdbGoTop( property->commandLineDb );
		if( ( ipPtr = MGdbSeek( property->commandLineDb, property->fieldCommandLineDbKey, log,  property->fieldCommandLineDbValue, exact ) ) )
		{
			MGmStrcat( localProperty.commandRestart, " " );
			MGmStrcat( localProperty.commandRestart, "-l" );
		}
		MGdbGoTop( property->commandLineDb );
		if( ( ipPtr = MGdbSeek( property->commandLineDb, property->fieldCommandLineDbKey, bu,  property->fieldCommandLineDbValue, exact ) ) )
		{
			MGmStrcat( localProperty.commandRestart, " " );
			MGmStrcat( localProperty.commandRestart, "-bu" );
		}
		MGdbGoTop( property->commandLineDb );
		if( ( ipPtr = MGdbSeek( property->commandLineDb, property->fieldCommandLineDbKey, backUp,  property->fieldCommandLineDbValue, exact ) ) )
		{
			MGmStrcat( localProperty.commandRestart, " " );
			MGmStrcat( localProperty.commandRestart, "-bu" );
		}
		MGdbGoTop( property->commandLineDb );
		if( ( ipPtr = MGdbSeek( property->commandLineDb, property->fieldCommandLineDbKey, y,  property->fieldCommandLineDbValue, exact ) ) )
		{
			MGmStrcat( localProperty.commandRestart, " " );
			MGmStrcat( localProperty.commandRestart, "-y" );
		}
		MGdbGoTop( property->commandLineDb );
		if( ( ipPtr = MGdbSeek( property->commandLineDb, property->fieldCommandLineDbKey, yes,  property->fieldCommandLineDbValue, exact ) ) )
		{
			MGmStrcat( localProperty.commandRestart, " " );
			MGmStrcat( localProperty.commandRestart, "--yes" );
		}
		MGmStrcat( localProperty.commandRestart, " --resume" );
		fprintf(stdout, "%s\n", SINGLE_LINES);
		fprintf(stdout,	"portmanager just updated itself, executing:\n\t%s\n", localProperty.commandRestart );
		fprintf(stdout, "%s\n", SINGLE_LINES);
		while( fflush( stdout ) );
		system( localProperty.commandRestart );
		exit( 0 );

	}

	/*
	 * If forced mode Mark port as being built
	 */
	if( property->pmMode || property->forced )
	{
		MGdbGoTop( property->availablePortsDb );
		if( strncmp( MGdbSeek( property->availablePortsDb,
					property->fieldAvailablePortsDbPortDir,
					oldPortDir,
					property->fieldAvailablePortsDbFlag,
					exact ),
					"2", 1 ) )
		{
			availablePortsDbIDX		= MGdbGetRecno( property->availablePortsDb ) - 1;
			MGmStrcpy( localProperty.newPortName, MGdbGet( property->availablePortsDb, availablePortsDbIDX, property->fieldAvailablePortsDbPortName ) );
			MGmStrcpy( localProperty.newPortDir, MGdbGet( property->availablePortsDb, availablePortsDbIDX, property->fieldAvailablePortsDbPortDir ) );
			property->availablePortsDb = MGdbDelete( property->availablePortsDb, availablePortsDbIDX );
			MGdbAdd( property->availablePortsDb, localProperty.newPortName, localProperty.newPortDir, "2", NULL );			
		}
	}
	rCleanUp( property, &localProperty );
	return( 0 );
}
/****************************************/
/****************************************/
/****************************************/
int	rCleanUp( structProperty* property, structLocalProperty* localProperty )
{
	/* 
	 * Revert bsd.port.mk patch
	 */
	if( property->bsdPortMkPatchedBlock == 0 )
	{
		property->bsdPortMkPatched	= MGPMrBsdPortMkRevertPatch( property );
	}

	MGdbDestroy( property->availableDependenciesDb );
	MGdbDestroy( property->availablePortsDb );
	MGdbDestroy( property->commandLineDb );
	MGdbDestroy( property->configDb );
	MGdbDestroy( property->ignoreDb );
	MGdbDestroy( property->installedPortsDb );
	MGdbDestroy( property->strikesDb );


	free( localProperty->environment[0] );
	free( localProperty->environment[1] );
	free( localProperty->environment[2] );
	free( localProperty->environment );

	free( localProperty->CURDIR );
	free( localProperty->afterOptionsFileSize );
	free( localProperty->afterOptionsFileTime );
	free( localProperty->batchCheck );
	free( localProperty->beforeOptionsFileSize );
	free( localProperty->beforeOptionsFileTime );
	free( localProperty->buffer );
	free( localProperty->command );
	free( localProperty->commandRestart );
	free( localProperty->installedPortName );
	free( localProperty->newPortDir );
	free( localProperty->newPortName );
	free( localProperty->options );
	free( localProperty->optionsCheck );
	free( localProperty->optionsDir );
	free( localProperty->startPortCmd );
	free( localProperty->startPortDir );
	free( localProperty->stopPortCmd );
	free( localProperty->stopPortDir );
	free( localProperty->workDir );
	free( localProperty->workFullPath );

	return(0);
}
/**********************************/
int	rCleanDir( char* portDir, char* workDir )
{
	FILE*	pHandle		= NULL;
	char	id[]		= "rCleanDir";
	char*	buffer		= NULL;
	char*	command		= NULL;
	char*	curDir		= NULL;
	int	idx		= 0;

	buffer		= calloc( MAXBUFFERSIZE, 1 );
	command		= calloc( MAXBUFFERSIZE, 1 );
	curDir		= calloc( MAXBUFFERSIZE, 1 );

	/*
	 * get .CURDIR
	 */
	MGmStrcpy( curDir, "cd " );
	MGmStrcat( curDir, PORTSDIR );
	MGmStrcat( curDir, portDir );
	MGmStrcat( curDir, "; make -V .CURDIR" );
	buffer[0]	= 0;
	pHandle		= popen( curDir, "r" );
	fread( buffer, MAXBUFFERSIZE, 1, pHandle );
	pclose( pHandle );
	idx	= 0;
	while( idx < MAXSTRINGSIZE )
	{
		if( buffer[idx] == LINEFEED || buffer[idx] == TAB || buffer[idx] == SPACE )
		{
			buffer[idx]	= 0;
			break;
		}
		idx++;
	}
	MGmStrcpy( curDir, buffer );
	
	MGmStrcpy( command, "/bin/rm -r " );
	MGmStrcat( command, workDir );
	MGmStrcat( command, curDir );

	MGmStrcat( command, "/work 2>&1" );
	fprintf( stdout, "===>  Cleaning for %s\n", portDir );	
	while( fflush( stdout ) );

	buffer[0]	= 0;
	pHandle		= popen( command, "r" );
	fread( buffer, MAXBUFFERSIZE, 1, pHandle );
	pclose( pHandle );
	free( buffer );	
	free( command );
	free( curDir );
	return( 0 );
}

/*
 * returns 1 if bsd.port.mk is patched, 0 otherwise
 */
int rBsdPortMkPatch( structProperty* property, structLocalProperty* localProperty )
{
	FILE*	pHandle		= NULL;
	char	id[]		= "rBsdPortMkPatch";
	int	bufferIdx	= 0;
		
	/*
	 * test for make -V PKGNAME, if there bsd.port.mk is assumed ok, otherwise replace bsd.port.mk from backup
	 */
	MGmStrcpy( localProperty->command, "cd " );
	MGmStrcat( localProperty->command, PORTSDIR );
	MGmStrcat( localProperty->command, "/sysutils/portmanager; make -V PKGNAME" );
	pHandle	= popen( localProperty->command, "r" );
	localProperty->buffer[0]	= 0;
	fread( localProperty->buffer, MAXBUFFERSIZE - 1, 1, pHandle );
	pclose( pHandle );
	bufferIdx	= 0;
	while( bufferIdx < MAXBUFFERSIZE )
	{
		if( localProperty->buffer[bufferIdx] == LINEFEED || localProperty->buffer[bufferIdx] == TAB || localProperty->buffer[bufferIdx] == SPACE )
		{
			localProperty->buffer[bufferIdx]	= 0;
			break;
		}
		bufferIdx++;
	}

	/*
	 * bsd.port.mk is broken, log this fact then replace bsd.port.mk from backup and test again, if ok block it
	 */
	if( MGrStrlen( localProperty->buffer ) < 5 )
	{
		MGmStrcpy( localProperty->command, "cp " );
		MGmStrcat( localProperty->command, SHAREDIR );
		MGmStrcat( localProperty->command, "/bsd.port.mk-BACKUP " );
		MGmStrcat( localProperty->command, PORTSDIR );
		MGmStrcat( localProperty->command, "/Mk/bsd.port.mk" );
		if( system( localProperty->command ) )
		{
			fprintf( stdout, "%s %s error: %s %s %s %s %s\n",
				id, PACKAGE_VERSION, "bsd.port.mk", "/usr/ports/Mk", "corrupt, restored from back up", " ", localProperty->command );
			while( fflush( stdout ) );
			MGPMlogAdd( property, "bsd.port.mk", "/usr/ports/Mk", "corrupt", "restored from back up", localProperty->command );

			/*
			 * test for make -V PKGNAME, if there bsd.port.mk is assumed ok, otherwise replace bsd.port.mk from backup
			 */
			MGmStrcpy( localProperty->command, "cd " );
			MGmStrcat( localProperty->command, PORTSDIR );
			MGmStrcat( localProperty->command, "/sysutils/portmanager; make -V PKGNAME" );
			pHandle	= popen( localProperty->command, "r" );
			localProperty->buffer[0]	= 0;
			fread( localProperty->buffer, MAXBUFFERSIZE - 1, 1, pHandle );
			pclose( pHandle );
			bufferIdx	= 0;
			while( bufferIdx < MAXBUFFERSIZE )
			{
				if( localProperty->buffer[bufferIdx] == LINEFEED || localProperty->buffer[bufferIdx] == TAB || localProperty->buffer[bufferIdx] == SPACE )
				{
					localProperty->buffer[bufferIdx]	= 0;
					break;
				}
				bufferIdx++;
			}
			if( MGrStrlen( localProperty->buffer ) < 5 )
			{
				fprintf( stdout, "%s %s error: %s %s %s %s %s\n",
					id, PACKAGE_VERSION, "bsd.port.mk", "/usr/ports/Mk", "unable to restored from back up", " ", localProperty->command );
				while( fflush( stdout ) );
				MGPMlogAdd( property, "bsd.port.mk", "/usr/ports/Mk", "corrupt", "unable to restored from back up", localProperty->command );
				exit( 0 );
			}
			else
			{
				property->bsdPortMkPatchedBlock	= 1;
				return(  0 );
			}
		}
		else
		{
			fprintf( stdout, "%s %s error: %s %s %s %s %s\n",
				id, PACKAGE_VERSION, "bsd.port.mk", "/usr/ports/Mk", "unable to restored from back up", " ", localProperty->command );
			while( fflush( stdout ) );
			MGPMlogAdd( property, "bsd.port.mk", "/usr/ports/Mk", "corrupt", "unable to restored from back up", localProperty->command );
			exit( 0 );
		}
	}

	/*
	 * test for make -V PORTMANAGER, if there bsd.port.mk allready patched, otherwise save original bsd.port.mk then patch it
	 */
	MGmStrcpy( localProperty->command, "cd " );
	MGmStrcat( localProperty->command, PORTSDIR );
	MGmStrcat( localProperty->command, "/sysutils/portmanager; make -V PORTMANAGER" );
	pHandle	= popen( localProperty->command, "r" );
	localProperty->buffer[0]	= 0;
	fread( localProperty->buffer, MAXBUFFERSIZE - 1, 1, pHandle );
	pclose( pHandle );
	bufferIdx	= 0;
	while( bufferIdx < MAXBUFFERSIZE )
	{
		if( localProperty->buffer[bufferIdx] == LINEFEED || localProperty->buffer[bufferIdx] == TAB || localProperty->buffer[bufferIdx] == SPACE )
		{
			localProperty->buffer[bufferIdx]	= 0;
			break;
		}
		bufferIdx++;
	}

	if( MGrStrlen( localProperty->buffer ) < 5 )
	{
		/*
		 * save original bsd.port.mk as back up
		 */
		MGmStrcpy( localProperty->command, "cp " );
		MGmStrcat( localProperty->command, PORTSDIR );
		MGmStrcat( localProperty->command, "/Mk/bsd.port.mk " );
		MGmStrcat( localProperty->command, SHAREDIR );
		MGmStrcat( localProperty->command, "/bsd.port.mk-BACKUP" );
		if( system( localProperty->command ) )
		{
			fprintf( stdout, "%s %s error: %s %s %s %s %s\n",
				id, PACKAGE_VERSION, "bsd.port.mk", "/usr/ports/Mk", "unable to back up", " ", localProperty->command );
			while( fflush( stdout ) );
			MGPMlogAdd( property, "bsd.port.mk", "/usr/ports/Mk", "unable to back up", " ", localProperty->command );
			property->bsdPortMkPatchedBlock	= 1;
			return(  0 );
		}

		/*
		 * Patch portmanager into bsd.port.mk
		 */
		MGmStrcpy( localProperty->command, "cd " );
		MGmStrcat( localProperty->command, PORTSDIR );
		MGmStrcat( localProperty->command, "/Mk; patch < " );
		MGmStrcat( localProperty->command, SHAREDIR );
		MGmStrcat( localProperty->command, "/patch-bsd.port.mk-0.3.6;" );
		fprintf( stdout, "patching bsd.port.mk-=>%s\n", localProperty->command );
		while( fflush( stdout ) );
		system( localProperty->command );
	}
	return( 1 );
}


/*
 *	Revert bsd.port.mk patch
 */
int MGPMrBsdPortMkRevertPatch( structProperty* property )
{
	char	id[]		= "MGPMrBsdPortMkRevertPatch";
	char*	command		= NULL;

	command			= calloc( MAXBUFFERSIZE, 1 );

	if( property->bsdPortMkPatched )
	{
		MGmStrcpy( command, "cd " );
		MGmStrcat( command, PORTSDIR );
		MGmStrcat( command, "/Mk; patch -R < " );
		MGmStrcat( command, SHAREDIR );
		MGmStrcat( command, "/patch-bsd.port.mk-0.3.6;" );
		fprintf( stdout, "reverting bsd.port.mk patch -=>%s\n", command );
		while( fflush( stdout ) );
		system( command );
	}
	free( command );
	return( 0 );
}
